<?php
// Encoding: UTF-8 (without BOM) (auto-detect: °°°°°)
require('_head.inc.php');

// ----- Handle form requests and status messages -----
if (UnbUi::IsFormSuccess())
{
	UnbSession::SetVar('SuccessMessage', 'User saved');
	if ($_POST['continue_alter_user'])
		UnbUi::RedirectAndExit('alter_user.php?userid=' . $_POST['userid']);
	else
		UnbUi::RedirectAndExit('list_users.php');
}
if (UnbUi::IsFormError())
{
	UnbUi::ReportError(UnbUi::GetFormErrorMessage());
}
if ($msg = UnbSession::GetVar('SuccessMessage', true))
{
	UnbUi::ReportSuccess($msg);
}

// ----- Load the user -----
$user = UnbUser::FindByUserId(intval($_REQUEST['userid']));
if (!isset($user))
{
	UnbUi::ReportError('Invalid user ID');
}
elseif ($user->GetUserId() != UnbSession::GetSessionUserId() && !UnbSession::IsAdministrator())
{
	UnbUi::ReportError('Access denied');
}
else
{
	?>
	<h1>Alter user</h1>

	<h2>General data</h2>

	<?php UnbUi::BeginForm('alter_user.php') ?>
	<?php UnbUi::FormUser($user->GetUserId()) ?>

	<table cellspacing="0" class="grid first-10em">
	<tr>
		<td>Display name:</td>
		<td><?php UnbUi::UserDisplayName($user) ?></td>
		<td class="form-description">This name will be displayed above your messages.</td>
	</tr>
	<tr>
		<td>Creation time:</td>
		<td>
			<span title="<?php echo UnbUi::DateTime($user->GetCreatedTime(), false, true) ?>"><?php echo UnbUi::RelativeTime($user->GetCreatedTime()) ?></span>
		</td>
	</tr>
	<tr>
		<td>Last login time:</td>
		<td>
			<span title="<?php echo UnbUi::DateTime($user->GetLastLoginTime(), false, true) ?>"><?php echo UnbUi::RelativeTime($user->GetLastLoginTime()) ?></span>
		</td>
	</tr>
	<tr>
		<td>Rating:</td>
		<td>
			<?php
			$rating = $user->GetMessagesRating();
			if ($rating[0] > 0)
				echo $rating[2] . ' (from ' . $rating[0] . ' ratings with an average weight of ' . ($rating[1] / $rating[0]) . ')';
			else
				echo '<em>(No ratings yet)</em>';
			?>
		</td>
	</tr>

	<tr>
		<td colspan="3">
			<h2>Login information</h2>
		</td>
	</tr>
	<?php
	if ($user->GetLoginPassword())
	{
		?>
		<tr>
			<td>Current password:</td>
			<td><?php UnbUi::UserCurrentPassword() ?></td>
			<td class="form-description">You need to enter your current password to change your login name or password.</td>
		</tr>
		<?php
	}
	else
	{
		?>
		<tr>
			<td colspan="3">
				<p class="warningbox">
				<strong>Attention:</strong> You cannot log into this account unless you set a login name and password below.
				</p>
			</td>
		</tr>
		<?php
	}
	?>
	<tr>
		<td>Login name:</td>
		<td><?php UnbUi::UserLoginName($user) ?></td>
		<td class="form-description">The login name is used for logging into the system and is independent from the display name.<br />
			It should contain letters and numbers only, no special or local characters or spaces.</td>
	</tr>
	<tr>
		<td><?php echo $user->GetLoginPassword() ? 'Change' : 'Set' ?> password:</td>
		<td><?php UnbUi::UserLoginPassword($user) ?></td>
		<td class="form-description">Keep this password in confidence and don’t give it away to anybody.<br />
			The password is only changed when you enter a new one in the input field.</td>
	</tr>
	<tr>
		<td>Repeat password:</td>
		<td><?php UnbUi::UserRepeatPassword() ?></td>
		<td class="form-description">To avoid typing errors, enter the password for a second time (only if you are changing it).</td>
	</tr>
	<tr>
		<td colspan="2"><?php UnbUi::UserResetAutologinPassword($user) ?></td>
		<td class="form-description">Reset your autologin password if somebody stole your browser cookies.<br />
			You’ll need to manually log in from other browsers once then.</td>
	</tr>

	<tr>
		<td colspan="3">
			<h2>Access</h2>
		</td>
	</tr>
	<tr>
		<td>Special privileges:</td>
		<?php
		if (UnbSession::IsAdministrator())
		{
			?>
			<td>
				<?php UnbUi::UserAdministrator($user) ?><br />
				<?php UnbUi::UserModerator($user) ?><br />
				<?php UnbUi::UserTrusted($user) ?><br />
			</td>
			<td class="form-description">Administrator/moderator/trusted user status.</td>
			<?php
		}
		else
		{
			?>
			<td colspan="2">
				<?php
				$statusStr = '';
				if ($user->IsAdministrator())
					$statusStr .= ($statusStr ? ', ' : '') . 'Administrator';
				if ($user->IsModerator())
					$statusStr .= ($statusStr ? ', ' : '') . 'Moderator';
				if ($user->IsTrusted())
					$statusStr .= ($statusStr ? ', ' : '') . 'Trusted user';
				if ($statusStr)
					echo $statusStr;
				else
					echo '<em>(None)</em>';
				?>
			</td>
			<?php
		}
		?>
	</tr>
	<tr>
		<td>Additional keys:</td>
		<?php
		if (UnbSession::IsAdministrator())
		{
			?>
			<td><?php UnbUi::UserAdditionalKeysList($user) ?></td>
			<td class="form-description">List of keys that the user can use to gain access, one per line.</td>
			<?php
		}
		else
		{
			?>
			<td colspan="2">
				<?php
				$additionalKeysList = UnbUi::GetUserAdditionalKeys($user);
				if (strlen($additionalKeysList))
					echo nl2br(htmlspecialchars($additionalKeysList));
				else
					echo '<em>(None)</em>';
				?>
			</td>
			<?php
		}
		?>
	</tr>
	<tr>
		<td colspan="3">
			<h2>VCard entries</h2>
		</td>
	</tr>
	<?php
	foreach ($user->GetVcardEntries() as $type => $entry)
	{
		?>
		<tr>
			<td><?php echo htmlspecialchars($type) ?>:</td>
			<td colspan="2"><?php UnbUi::UserAlterVcardEntry($type, $entry) ?></td>
		</tr>
		<?php
	}
	?>
	<tr>
		<td colspan="2">
			Add a new entry:
		</td>
		<td class="form-description">To add a new entry, select the entry type and enter the entry data.</td>
	</tr>
	<tr>
		<td><?php UnbUi::UserVcardTypes() ?></td>
		<td colspan="2"><?php UnbUi::UserAddVcardEntry() ?></td>
	</tr>

	<tr>
		<td></td>
		<td colspan="2">
			<br />
			<input type="submit" class="default icon13 ok" value="Save changes" /> &nbsp;
			<input type="submit" name="continue_alter_user" class="icon13 ok" value="Save and continue" /> &nbsp;
			<a href="list_users.php" class="buttonlike icon13 back">Back to users list</a>
			<?php
			if (UnbSession::IsAdministrator())
			{
				?>
				&nbsp; <a href="delete_user.php?userid=<?php echo $user->GetUserId() ?>" class="buttonlike icon13 delete">Go delete this user</a>
				<?php
			}
			?>
		</td>
	</tr>
	</table>
	<?php UnbUi::EndForm() ?>

	<script type="text/javascript">//<![CDATA[
	$("#userDisplayNameTb").focus();
	//]]></script>

	<?php
}

require('_foot.inc.php');
?>