-- Create database

-- DROP DATABASE IF EXISTS "unb2test";
CREATE DATABASE "unb2test" WITH ENCODING='UTF8';

-- Create schema, if desired

CREATE SCHEMA "unb2test";
SET search_path TO 'unb2test';

-- Create base relations

CREATE TABLE "keylist" (
	"KeylistId" INTEGER NOT NULL,
	"UserId" INTEGER NOT NULL,
	PRIMARY KEY ("KeylistId", "UserId"));

CREATE TABLE "user" (
	"UserId" INTEGER NOT NULL PRIMARY KEY,
	"CreatedTime" TIMESTAMP NOT NULL,
	"LastLoginTime" TIMESTAMP,
	"DisplayName" VARCHAR(255),
	"AdditionalKeylistId" INTEGER,
	"LoginName" VARCHAR(255),
	"LoginPassword" VARCHAR(255));
CREATE UNIQUE INDEX "user_DisplayName" ON "user" ("DisplayName");
CREATE UNIQUE INDEX "user_LoginName" ON "user" ("LoginName");

CREATE TABLE "user_vcard" (
	"UserId" INTEGER NOT NULL,
	"Type" VARCHAR(255) NOT NULL,
	"Data" TEXT NOT NULL,
	"Visibility" SMALLINT NOT NULL DEFAULT 0,
	PRIMARY KEY ("UserId", "Type"));
CREATE INDEX "user_vcard_UserId" ON "user_vcard" ("UserId");

CREATE TABLE "user_config" (
	"UserId" INTEGER NOT NULL,
	"Name" VARCHAR(255) NOT NULL,
	"Data" VARCHAR(255) NOT NULL,
	PRIMARY KEY ("UserId", "Name"));
CREATE INDEX "user_config_UserId" ON "user_config" ("UserId");

CREATE TABLE "message" (
	"MessageId" INTEGER NOT NULL PRIMARY KEY,
	"CreatedTime" TIMESTAMP NOT NULL,
	"Owner" INTEGER NOT NULL,
	"PrimaryReference" INTEGER,
	"EntryPoint" BOOLEAN NOT NULL DEFAULT FALSE,
	"PageName" VARCHAR(255),
	"ReadAccessKeylistId" INTEGER,
	"AlterAccessKeylistId" INTEGER,
	"ReplyAccessKeylistId" INTEGER,
	"ModerationFlags" SMALLINT NOT NULL DEFAULT 0,
	"SearchOptimisedContent" MEDIUMTEXT,
	"SearchOptimisedSubject" VARCHAR(255),
	"SearchRevision" SMALLINT,
	"LastBeginEdit" TIMESTAMP);
CREATE UNIQUE INDEX "message_PageName" ON "message" ("PageName");

CREATE TABLE "message_reference" (
	"MessageId" INTEGER NOT NULL,
	"Reference" INTEGER NOT NULL,
	PRIMARY KEY ("MessageId", "Reference"));

CREATE TABLE "message_revision" (
	"MessageId" INTEGER NOT NULL,
	"RevisionNumber" SMALLINT NOT NULL,
	"CreatedTime" TIMESTAMP NOT NULL,
	"Author" INTEGER NOT NULL,
	"Subject" VARCHAR(255),
	"Content" MEDIUMTEXT,
	"HtmlContent" MEDIUMTEXT,
	"Summary" VARCHAR(255),
	"ModerationState" SMALLINT NOT NULL DEFAULT 0,
	"Data" MEDIUMBLOB,
	"HasData" BOOLEAN NOT NULL DEFAULT FALSE,
	PRIMARY KEY ("MessageId", "RevisionNumber"));
CREATE INDEX "message_revision_CreatedTime" ON "message_revision" ("CreatedTime");

CREATE TABLE "tag" (
	"TagId" INTEGER NOT NULL PRIMARY KEY,
	"TagName" VARCHAR(255) NOT NULL,
	"ParentTag" INTEGER,
	"Meaning" SMALLINT,
	"Description" VARCHAR(255),
	"UseAccessKeylistId" INTEGER,
	"ReadAccessKeylistId" INTEGER,
	"Flags" SMALLINT NOT NULL DEFAULT 0);

CREATE TABLE "message_revision_tag" (
	"MessageId" INTEGER NOT NULL,
	"RevisionNumber" SMALLINT NOT NULL,
	"TagId" INTEGER NOT NULL,
	"TagData" VARCHAR(255),
	PRIMARY KEY ("MessageId", "RevisionNumber", "TagId"));

CREATE TABLE "message_rating" (
	"UserId" INTEGER NOT NULL,
	"MessageId" INTEGER NOT NULL,
	"Rating" SMALLINT NOT NULL,
	"Weight" SMALLINT NOT NULL,
	PRIMARY KEY ("UserId", "MessageId"));
CREATE INDEX "message_rating_MessageId" ON "message_rating" ("MessageId");

-- Create foreign keys for referential integrity

ALTER TABLE "keylist" ADD FOREIGN KEY ("UserId") REFERENCES "user" ("UserId") ON DELETE CASCADE;

ALTER TABLE "user_vcard" ADD FOREIGN KEY ("UserId") REFERENCES "user" ("UserId") ON DELETE CASCADE;

ALTER TABLE "user_config" ADD FOREIGN KEY ("UserId") REFERENCES "user" ("UserId") ON DELETE CASCADE;

ALTER TABLE "message" ADD FOREIGN KEY ("Owner") REFERENCES "user" ("UserId");
ALTER TABLE "message" ADD FOREIGN KEY ("PrimaryReference") REFERENCES "message" ("MessageId") ON DELETE SET NULL;
-- ALTER TABLE "message" ADD FOREIGN KEY ("MessageId", "SearchRevision") REFERENCES "message_revision" ("MessageId", "RevisionNumber");

ALTER TABLE "message_reference" ADD FOREIGN KEY ("MessageId") REFERENCES "message" ("MessageId") ON DELETE CASCADE;
ALTER TABLE "message_reference" ADD FOREIGN KEY ("Reference") REFERENCES "message" ("MessageId") ON DELETE CASCADE;

ALTER TABLE "message_revision" ADD FOREIGN KEY ("MessageId") REFERENCES "message" ("MessageId") ON DELETE CASCADE;
ALTER TABLE "message_revision" ADD FOREIGN KEY ("Author") REFERENCES "user" ("UserId");

ALTER TABLE "tag" ADD FOREIGN KEY ("ParentTag") REFERENCES "tag" ("TagId");

ALTER TABLE "message_revision_tag" ADD FOREIGN KEY ("MessageId", "RevisionNumber") REFERENCES "message_revision" ("MessageId", "RevisionNumber") ON DELETE CASCADE;
ALTER TABLE "message_revision_tag" ADD FOREIGN KEY ("TagId") REFERENCES "tag" ("TagId") ON DELETE CASCADE;

ALTER TABLE "message_rating" ADD FOREIGN KEY ("UserId") REFERENCES "user" ("UserId") ON DELETE CASCADE;
ALTER TABLE "message_rating" ADD FOREIGN KEY ("MessageId") REFERENCES "message" ("MessageId") ON DELETE CASCADE;

