<?php
// PHP Code Documentation Generator
// Copyright 2005-9 by Yves Goergen
// Website: http://unclassified.de/
// All rights reserved.

define('endl', "\n");
#set_time_limit(10);
$INFO = array();

$INFO = array(

// BEGIN of configuration ---------------------------------

// Root path to analyse files in (input path)
'rootpath' => '../unb_lib/',
// Files to ignore
'ignore_files_regex' => '^unb_all.lib.php$',
// Regular expression of text to recognise relevant files
'file_tag' => '_UNB Components_i',
// Path to write this documentation into (output path)
'outpath' => '../doc/code_reference/',

// Page title and caption for generated reference files
'title' => 'UNB Components Code Reference',
// Page footline for generated reference files
'footline' => '&copy; ' . date('Y') .
	' Yves Goergen. <a href="http://unclassified.de/go/unb2">UNB Components Homepage</a>. Reference generated on ' . date('Y-m-d') . '.',

// END of configuration -----------------------------------

);

echo '<html>
<head>
<title>PHP Code Documentation Generator</title>
</head>
<body>
<p><big>PHP Code Documentation Generator</big><br>
<i><small>&copy; 2005-9 by <a href="http://unclassified.de/">Yves Goergen</a></small></i></p>';

echo '<p><b>Configuration:</b><br>
	Input path: ' . $INFO['rootpath'] . '<br>
	Output path: ' . $INFO['outpath'] . '<br>
	Title: ' . $INFO['title'] . '<br>
	Footline: ' . $INFO['footline'] . '</p>';

$startTime = array_sum(explode(' ', microtime()));

if (file_exists($INFO['outpath']))
{
	if (!is_dir($INFO['outpath'])) die('Output path exists but is not a directory!');
	echo 'Deleting existing output directory...<br>';
	RecursiveDeleteDirectory($INFO['outpath']);
}

mkdir($INFO['outpath']);
#copy('coderef.css', $INFO['outpath'] . 'coderef.css');
WriteCssFile($INFO['outpath'] . 'coderef.css');

$files = GetFiles();

list($effLines, $effFiles) = CountEffectiveLines($files);

$items = array();
foreach ($files as $file)
{
	GetItems($file, $items);
}

// Save all items for FindLinks()
$globalitems = $items;
$globalitems_class = array();
foreach ($globalitems as $key => $item)
{
	if ($item['type'] !== 'file' &&
		$item['type'] !== 'function' &&
		$item['type'] !== 'class' &&
		$item['type'] !== 'constant')
	{
		$globalitems[$key] = null;
		continue;
	}
	if ($item['type'] === 'file')
	{
		$globalitems[$key]['name'] = basename($item['name']);
		#echo 'Trimmed filename ' . $item['name'] . ' to ' . $globalitems[$key]['name'] . '<br>';
	}
	foreach ($globalitems as $key2 => $item2) if ($key !== $key2)
	{
		if ($item['name'] === $item2['name'])
		{
			// Name is not unique
			if ($item2['type'] == 'function' &&
			    $item2['class'] != '')
			{
				#$globalitems[$key2]['name'] = $item2['class'] . '.' . $item2['name'];
				#echo 'Ambigous name ' . $item['name'] . ': added class name: ' . $globalitems[$key2]['name'] . '<br>';
				$globalitems[$key2]['ambigous'] = true;
			}
			else
			{
				$globalitems[$key2] = null;
				echo 'Ambigous name ' . $item['name'] . ': removed item<br>';
			}
		}
	}
	if ($item['class'] != '')
	{
		$globalitems_class[] = $item;
	}
}
usort($globalitems, 'FindLinksCmpFn');   // longest first, important for greedy matching
usort($globalitems_class, 'FindLinksCmpFn_class');   // longest first, important for greedy matching

$constants = array();
$functions = array();
$classes = array();
foreach ($items as $item)
{
	if ($item['type'] == 'constant' && $item['class'] == '')
	{
		$constants[$item['name']] = $item;
	}
	if ($item['type'] == 'function' && $item['class'] == '')
	{
		$functions[$item['name']] = $item;
	}
	if ($item['type'] == 'class')
	{
		$classes[$item['name']] = $item;
	}
}
ksort($constants);
ksort($functions);
ksort($classes);

WriteIndex();

WriteFilesOverview($items);
WriteFunctionsOverview($functions);
WriteClassesOverview($classes);
WriteConstantsOverview($constants);

foreach ($files as $file)
{
	WriteFile($file, $items);
}
foreach ($items as $item)
{
	if ($item['type'] == 'function')
		WriteFunction($item);
	if ($item['type'] == 'class')
		WriteClass($item, $items);
}

$time = array_sum(explode(' ', microtime())) - $startTime;

echo '<p><b>Finished</b> in ' . round($time, 3) . ' seconds.</p>';

echo '<p><a href="' . $INFO['outpath'] . '">Go to the generated reference</a></p>';

echo '</body></html>';

// No code below this line


// ==============================  GetFiles  ==============================

function GetFiles($cwd = '', $files = null)
{
	global $INFO;

	if (!isset($files)) $files = array();
	$handle = opendir($INFO['rootpath'] . ($cwd ? $cwd : './'));
	if ($handle === false) return $ret;
	while ($file = readdir($handle))
	{
		if ($file{0} == '.' || $file == 'codedoc.php' || preg_match('_\.css\.php$_', $file))
		{
		}
		elseif (is_dir($INFO['rootpath'] . $cwd . $file))
		{
			$files = GetFiles($cwd . $file . '/', $files);
		}
		elseif (preg_match('#' . $INFO['ignore_files_regex'] . '#i', $file))
		{
		}
		elseif (preg_match('/^.+\.(php|js)$/', $file))
		{
			$files[] = $cwd . $file;
		}
	}
	closedir($handle);
	sort($files);
	return $files;
}

function RecursiveDeleteDirectory($path)
{
	$handle = opendir($path);
	if ($handle === false) return;
	while ($file = readdir($handle))
	{
		if ($file == '.' || $file == '..')
		{
		}
		elseif (is_dir($path . '/' . $file))
		{
			RecursiveDeleteDirectory($path . '/' . $file);
			rmdir($path . '/' . $file);
		}
		else
		{
			unlink($path . '/' . $file);
		}
	}
	closedir($handle);
	rmdir($path);
}


// ==============================  CountEffectiveLines  ==============================

function CountEffectiveLines($files)
{
	global $INFO;

	$line_count = 0;
	$file_count = 0;
	foreach ($files as $file)
	{
		$lines = file($INFO['rootpath'] . $file);

		$data = join('', $lines);
		$data = preg_replace('_/\*.*?\*/_s', '', $data);
		$data = preg_replace('_<!--.*?-->_s', '', $data);
		$lines = explode("\n", $data);

		foreach ($lines as $line)
		{
			$line = preg_replace('_<!--.*?-->_', '', $line);
			$line = preg_replace('_(//|#).*$_', '', $line);
			$line = preg_replace('_/\*.*?\*/_', '', $line);
			$line = trim($line);
			if ($line !== '' &&
				!in_array($line, array('{', '}', '<?php', '?>', '/*', '*/', '<?php } ?>', '{{')))
			{
				$line_count++;
				#if ($line_count < 1000) echo htmlentities($line) . "\n";
			}
		}
		$file_count++;
	}
	return array($line_count, $file_count);
}


// ==============================  GetItems  ==============================

/*
PHP-Quelltext-Dokumentation
	Format:
---
// Funktionsbeschreibung
//
// [Ausfhrliche Beschreibung]
//
// {in|out|in/out} Parameter = (Typ) Beschreibung
//
// [returns (Typ) Beschreibung]
// [uses (Typ) Beschreibung]
// [exports (Typ) Beschreibung]
// [see Beschreibung]
[]
function ...
---
	Auch eine Beschreibung fr andere Datenobjekte definieren
	"in/out" oder "in,out"?
*/
function GetItems($file, &$items)
{
	global $INFO;

	$lines = file($INFO['rootpath'] . $file);
	if (!is_array($items)) $items = array();
	$className = '';
	if ($lines)
	{
		$myFile = false;
		#if (preg_match('_/abbc\._i', $file)) $myFile = true;
		#if (preg_match('_/ute-_i', $file)) $myFile = true;
		for ($i = 0; $i < 20; $i++)
		{
			if (preg_match($INFO['file_tag'], $lines[$i])) { $myFile = true; break; }
		}
		if (!$myFile) return;

		// Scan for file comment
		$nodoc = false;
		$item = array();
		$item['file'] = $file;
		$item['name'] = $file;
		$item['type'] = 'file';
		$mode = 0;
		foreach ($lines as $lineno => $line)
		{
			if ($mode == 0 && preg_match('_^<\?(php)?_i', $line)) $mode++;
			elseif ($mode == 1 || $mode == 2)
			{
				if (preg_match('_^//\s?(.*)$_', $line, $m))
				{
					if ($mode == 1 && trim($m[1]) == '') $mode++;
					elseif ($mode == 2 && @strpos($file, trim($m[1])) === false)
						$item['desc'] .= ($item['desc'] ? endl : '') . trim($m[1]);
				}
				else
					$mode = 3;
			}
			if ($lineno > 50 || $mode > 2) break;
		}
		if (strpos($item['desc'], '[nodoc]') !== false) $nodoc = true;
		if (!$nodoc /*&& $item['desc']*/) $items[] = $item;

		foreach ($lines as $lineno => $line)
		{
			if (preg_match('_^\s*((?:public\s+|protected\s+|private\s+|static\s+)*)function\s+(.*?)\((.*?)\)_i', $line, $m))
			{
				$nodoc = false;
				$item = array();
				$item['file'] = $file;
				$item['lineno'] = $lineno + 1;
				$item['type'] = 'function';
				$item['access'] = '';
				if (strpos($m[1], 'public') !== false) $item['access'] = 'public';
				if (strpos($m[1], 'protected') !== false) $item['access'] = 'protected';
				if (strpos($m[1], 'private') !== false) $item['access'] = 'private';
				$item['static'] = strpos($m[1], 'static') !== false;
				$item['name'] = trim($m[2]);
				if (preg_match('_(.js|.js.php)$_i', $file))
					$item['lang'] = 'js';
				elseif (preg_match('_(.php)$_i', $file))
					$item['lang'] = 'php';
				else
					$item['lang'] = '';
				if ($className) $item['class'] = $className;
				$item['raw_params'] = array();
				foreach (explode(',', $m[3]) as $p)
				{
					if (trim($p) != '') $item['raw_params'][] = trim($p);
				}

				// Go back and look at previous lines
				$i = $lineno - 1;
				while ($i >= 0 &&
				       preg_match('_^\s*//_', $lines[$i]))
					$i--;
				$mode = 1;
				$item['params'] = array();
				for ($i++; $i < $lineno; $i++)
				{
					preg_match('_^\s*//\s?(.*)$_', rtrim($lines[$i]), $m);
					$str = rtrim($m[1]);
					if ($mode == 1 && $str == '') { $mode = 2; continue; }
					if (preg_match('_^\s?(in|out|in,out|in/out)\s_', $str)) $mode = 3;
					if (preg_match('_^\s?[0-9a-z\_A-Z]+\s+=\s_', $str)) $mode = 3;
					if (preg_match('_^\s?(returns:?)\s_i', $str)) $mode = 4;
					if (preg_match('_^\s?(uses:?)\s_i', $str)) $mode = 5;
					if (preg_match('_^\s?(exports:?)\s_i', $str)) $mode = 6;
					if (preg_match('_^\s?(see:?)\s_i', $str)) $mode = 7;
					if (preg_match('_^\s?(throws:?)\s_i', $str)) $mode = 8;
					if (preg_match('_^\s?(privileges:?)\s_i', $str)) $mode = 9;
					if ($mode == 1) $item['desc'] .= ($item['desc'] ? endl : '') . $str;
					if ($mode == 2) $item['longdesc'] .= ($item['longdesc'] ? endl : '') . $str;
					if ($mode == 3)
					{
						if (preg_match('_^\s*(in|out)_i', $str) ||
							preg_match('_^\s*[0-9a-z\_]+\s+=\s_i', $str))
						{
							$item['params'][] = array();
							while (trim($str))
							{
								if (preg_match('_^\s*(in|out|in,out|in/out)(?!\s+=\s)\s+(.*)$_i', $str, $m))
								{
									$item['params'][sizeof($item['params']) - 1]['dir'] = trim($m[1]);
									$str = $m[2];
								}
								elseif (preg_match('_^\s*=\s+(.*)$_i', $str, $m))
								{
									$item['params'][sizeof($item['params']) - 1]['desc'] = trim($m[1]);
									$str = '';
								}
								elseif (preg_match('_^\s*([a-z0-9\_.]*?)\s+(.*)$_i', $str, $m))
								{
									$item['params'][sizeof($item['params']) - 1]['name'] = trim($m[1]);
									$str = $m[2];
								}
								else
								{
									$str = '';
								}
							}
						}
						elseif (trim($str) != '')
						{
							$item['params'][sizeof($item['params']) - 1]['desc'] .=
								($item['params'][sizeof($item['params']) - 1]['desc'] ? endl : '') .
								trim($str);
						}
						else
						{
							$mode = 2;   // Go back to the long description after an empty line after parameters
						}
					}
					if ($mode == 4)
					{
						if (preg_match('_^\s*returns:?\s+(.*)$_i', $str, $m))
						{
							$item['returns'] = trim($m[1]);
						}
						elseif (trim($str) != '')
						{
							$item['returns'] .= ($item['returns'] ? endl : '') . trim($str);
						}
						else
						{
							$mode = 2;   // Go back to the long description after an empty line
						}
					}
					if ($mode == 5)
					{
						if (preg_match('_^\s*uses:?\s+(.*)$_i', $str, $m))
						{
							$item['uses'] .= ($item['uses'] ? endl : '') . trim($m[1]);
						}
						elseif (trim($str) != '')
						{
							$item['uses'] .= ($item['uses'] ? endl : '') . trim($str);
						}
						else
						{
							$mode = 2;   // Go back to the long description after an empty line
						}
					}
					if ($mode == 6)
					{
						if (preg_match('_^\s*exports:?\s+(.*)$_i', $str, $m))
						{
							$item['exports'] .= ($item['exports'] ? endl : '') . trim($m[1]);
						}
						elseif (trim($str) != '')
						{
							$item['exports'] .= ($item['exports'] ? endl : '') . trim($str);
						}
						else
						{
							$mode = 2;   // Go back to the long description after an empty line
						}
					}
					if ($mode == 7)
					{
						if (preg_match('_^\s*see:?\s+(.*)$_i', $str, $m))
						{
							$item['see'] .= ($item['see'] ? endl : '') . trim($m[1]);
						}
						elseif (trim($str) != '')
						{
							$item['see'] .= ($item['see'] ? endl : '') . trim($str);
						}
						else
						{
							$mode = 2;   // Go back to the long description after an empty line
						}
					}
					if ($mode == 8)
					{
						if (preg_match('_^\s*throws:?\s+(.*)$_i', $str, $m))
						{
							$item['throws'] .= ($item['throws'] ? endl : '') . trim($m[1]);
						}
						elseif (trim($str) != '')
						{
							$item['throws'] .= ($item['throws'] ? ' ' : '') . trim($str);
						}
						else
						{
							$mode = 2;   // Go back to the long description after an empty line
						}
					}
					if ($mode == 9)
					{
						if (preg_match('_^\s*privileges:?\s+(.*)$_i', $str, $m))
						{
							$item['privileges'] = trim($m[1]);
						}
						elseif (trim($str) != '')
						{
							$item['privileges'] .= ($item['privileges'] ? endl : '') . trim($str);
						}
						else
						{
							$mode = 2;   // Go back to the long description after an empty line
						}
					}
				}

				// Synchronise parameters
				if ($item['params'])
				{
					foreach($item['raw_params'] as $rp)
					{
						if (preg_match('_^(&?)\s*\$([^ =]+)\s*(=\s*(.*))?$_', $rp, $m))
						{
							$foundParam = false;
							foreach ($item['params'] as $i => $p)
							{
								if ($m[2] == $p['name'])
								{
									$foundParam = true;
									if ($m[1] == '&') $item['params'][$i]['reference'] = true;
									if ($m[3] != '')
									{
										$item['params'][$i]['optional'] = true;
										$item['params'][$i]['default'] = trim($m[4]);
									}
								}
							}
							if (!$foundParam)
							{
								// Parameter is in the code but not in the comment
								$item['params'][] = array(
									'name' => $m[2],
									'optional' => $m[3] != '',
									'default' => trim($m[4])
									);
							}
						}
					}
				}
				elseif ($item['raw_params'])
				{
					foreach($item['raw_params'] as $rp)
					{
						if (preg_match('_^(&?)\s*\$([^ =]+)\s*(=\s*(.*))?$_', $rp, $m))
						{
							$p = array();
							$p['name'] = $m[2];
							if ($m[1] == '&') $p['reference'] = true;
							if ($m[3] != '')
							{
								$p['optional'] = true;
								$p['default'] = trim($m[4]);
							}
							$item['params'][] = $p;
						}
					}
				}

				if (strpos($item['desc'], '[nodoc]') !== false) $nodoc = true;
				if (!$nodoc) $items[] = $item;
			}

			if (preg_match('_^\s*(abstract\s+)?class (.*?)(extends (.*?))?({|$)_i', $line, $m))
			{
				$className = trim($m[2]);

				$nodoc = false;
				$item = array();
				$item['file'] = $file;
				$item['lineno'] = $lineno;
				$item['type'] = 'class';
				$item['name'] = trim($m[2]);
				$item['abstract'] = strlen($m[1]) > 0;
				$item['extends'] = trim($m[4]);

				// Go back and look at previous lines
				$i = $lineno - 1;
				while ($i >= 0 &&
				       preg_match('_^\s*//_', $lines[$i]))
					$i--;
				$mode = 1;
				for ($i++; $i < $lineno; $i++)
				{
					preg_match('_^\s*//\s?(.*)$_', rtrim($lines[$i]), $m);
					$str = trim($m[1]);
					$item['desc'] .= ($item['desc'] ? endl : '') . $str;
				}

				if (strpos($item['desc'], '[nodoc]') !== false) $nodoc = true;
				if (!$nodoc) $items[] = $item;
			}
			if (preg_match('_^\s*}\s*//\s*class_i', $line, $m))
			{
				$className = '';
			}

			if ($className != '' &&
			    preg_match('_^\s*(var|private|protected|public)\s+(static\s+)?\$([^=;]+)(=([^;]+))?;\s*(//(.*))?_i', $line, $m))
			{
				$nodoc = false;
				$item = array();
				$item['file'] = $file;
				$item['lineno'] = $lineno;
				$item['type'] = 'field';
				$item['access'] = 'public';
				if (strpos($m[1], 'public') !== false) $item['access'] = 'public';
				if (strpos($m[1], 'protected') !== false) $item['access'] = 'protected';
				if (strpos($m[1], 'private') !== false) $item['access'] = 'private';
				$item['static'] = trim($m[2]) != '';
				$item['class'] = $className;
				$item['name'] = trim($m[3]);
				$item['default'] = trim($m[5]);
				$lineComment = $m[7];

				// Go back and look at previous lines
				$i = $lineno - 1;
				while ($i >= 0 &&
				       preg_match('_^\s*//_', $lines[$i]))
					$i--;
				$mode = 1;
				for ($i++; $i < $lineno; $i++)
				{
					preg_match('_^\s*//\s?(.*)$_', rtrim($lines[$i]), $m);
					$str = trim($m[1]);
					if (strpos($str, '[nodoc]') !== false) $nodoc = true;
					$item['desc'] .= ($item['desc'] ? endl : '') . $str;
				}

				$item['desc'] .= ($item['desc'] ? endl : '') . trim($lineComment);

				if (strpos($item['desc'], '[nodoc]') !== false) $nodoc = true;
				if (!$nodoc) $items[] = $item;
			}

			if ($className != '' &&
			    preg_match('_^\s*const\s+([^=;]+)(=([^;]+))?;\s*(//(.*))?_i', $line, $m))
			{
				$nodoc = false;
				$item = array();
				$item['file'] = $file;
				$item['lineno'] = $lineno;
				$item['type'] = 'constant';
				$item['class'] = $className;
				$item['name'] = trim($m[1]);
				$item['value'] = trim($m[3]);
				$lineComment = $m[5];

				// Go back and look at previous lines
				$i = $lineno - 1;
				while ($i >= 0 &&
				       preg_match('_^\s*//_', $lines[$i]))
					$i--;
				$mode = 1;
				for ($i++; $i < $lineno; $i++)
				{
					preg_match('_^\s*//\s?(.*)$_', rtrim($lines[$i]), $m);
					$str = trim($m[1]);
					if (strpos($str, '[nodoc]') !== false) $nodoc = true;
					$item['desc'] .= ($item['desc'] ? endl : '') . $str;
				}

				$item['desc'] .= ($item['desc'] ? endl : '') . trim($lineComment);

				if (strpos($item['desc'], '[nodoc]') !== false) $nodoc = true;
				if (!$nodoc) $items[] = $item;
			}

			if (preg_match('_^\s*define\(\'?(.*?)\'?,(.*?)\);\s*(//(.*))?_i', $line, $m))
			{
				$nodoc = false;
				$item = array();
				$item['file'] = $file;
				$item['lineno'] = $lineno;
				$item['type'] = 'constant';
				$item['name'] = trim($m[1]);
				$item['value'] = trim($m[2]);
				$item['desc'] = trim($m[4]);

				// Go back and look at previous lines
				$i = $lineno - 1;
				while ($i >= 0 &&
				       preg_match('_^\s*//_', $lines[$i]))
					$i--;
				$mode = 1;
				for ($i++; $i < $lineno; $i++)
				{
					preg_match('_^\s*//\s?(.*)$_', rtrim($lines[$i]), $m);
					$str = trim($m[1]);
					if (strpos($str, '[nodoc]') !== false) $nodoc = true;
					$item['desc'] .= ($item['desc'] ? endl : '') . $str;
				}

				if (strpos($item['desc'], '[nodoc]') !== false) $nodoc = true;
				if (!$nodoc) $items[] = $item;
			}
		}
	}
}


// ==============================  GetItemLink  ==============================

function GetItemLink(&$item)
{
	if (!is_array($item)) return '#noarray';
	if ($item['type'] == 'file') return GetFileLink($item);
	if ($item['type'] == 'function') return GetFunctionLink($item);
	if ($item['type'] == 'class') return GetClassLink($item);
	if ($item['type'] == 'constant') return GetConstantLink($item);
	return '#notype';
}


// ==============================  GetFileLink  ==============================

function GetFileLink(&$item)
{
	if (is_array($item))
		$link = $item['file'];
	elseif (is_string($item))
		$link = $item;
	else
		return;
	$link = 'file_' . preg_replace('_[^a-z0-9]_i', '_', strtolower($link)) . '.html';
	return $link;
}


// ==============================  GetFunctionLink  ==============================

function GetFunctionLink(&$item)
{
	if ($item['type'] == 'function')
	{
		$link = $item['name'];
		if ($item['class'] != '') $link = $item['class'] . '.' . $link;
		$link = 'function_' . preg_replace('_[^a-z0-9]_i', '_', strtolower($link)) . '.html';
		return $link;
	}
}


// ==============================  GetClassLink  ==============================

function GetClassLink(&$item)
{
	if ($item['type'] == 'class')
	{
		$link = $item['name'];
		$link = 'class_' . preg_replace('_[^a-z0-9]_i', '_', strtolower($link)) . '.html';
		return $link;
	}
	if ($item['type'] == 'function')
	{
		$link = $item['class'];
		$link = 'class_' . preg_replace('_[^a-z0-9]_i', '_', strtolower($link)) . '.html';
		return $link;
	}
}


// ==============================  GetConstantLink  ==============================

function GetConstantLink(&$item)
{
	if ($item['type'] == 'constant' && $item['class'] != '')
	{
		$link = 'class_' . preg_replace('_[^a-z0-9]_i', '_', strtolower($item['class'])) . '.html';
		$link .= '#' . preg_replace('_[^a-z0-9]_i', '_', strtolower($item['name']));
		return $link;
	}
	if ($item['type'] == 'constant')
	{
		$link = $item['name'];
		$link = 'constants.html#' . preg_replace('_[^a-z0-9]_i', '_', strtolower($link));
		return $link;
	}
}


// ==============================  WriteIndex  ==============================

function WriteIndex()
{
	global $INFO, $effLines, $effFiles;

	ob_start();

	ShowHeader(null, 'Index');

echo '<p class="indent">
<a href="files.html">Files overview</a><br>
<a href="functions.html">Functions overview</a><br>
<a href="classes.html">Classes overview</a><br>
<a href="constants.html">Constants overview</a><br>
</p>

<h2>Source code statistics</h2>
<p class="indent">
Effective lines of code: ' . $effLines . '<br>
Effective code files: ' . $effFiles . '<br>
</p>
';

	ShowFooter();

	$text = ob_get_contents();
	ob_end_clean();

	$filename = $INFO['outpath'] . 'index.html';
	$fp = fopen($filename, 'w');
	fwrite($fp, $text);
	fclose($fp);
}


// ==============================  WriteFilesOverview  ==============================

function WriteFilesOverview(&$items)
{
	global $INFO;

	ob_start();

	ShowHeader(null, 'Files overview');

	$haveFiles = false;
	foreach ($items as $item) if ($item['type'] == 'file')
	{
		$haveFiles = true;
		break;
	}

	if ($haveFiles)
	{
		echo '<p>
<table cellspacing="0" cellpadding="0">
';
		foreach ($items as $item) if ($item['type'] == 'file')
		{
			echo '<tr>
	<td class="rpad"><a href="' . GetFileLink($item) . '">' . $item['file'] . '</a></td>
	<td>
		';
			if ($item['desc'])
				echo FindLinks(nl2br_(htmlspecialchars(trim($item['desc']))));
			else
				echo '<span class="grey">No description available.</span>';
			echo '
	</td>
</tr>
';
		}
		echo '</table>
</p>
';
	}
	else
	{
		echo '<p>No files found.</p>';
	}

	ShowFooter();

	$text = ob_get_contents();
	ob_end_clean();

	$filename = $INFO['outpath'] . 'files.html';
	$fp = fopen($filename, 'w');
	fwrite($fp, $text);
	fclose($fp);
}


// ==============================  WriteFunctionsOverview  ==============================

function WriteFunctionsOverview(&$items)
{
	global $INFO;

	ob_start();

	ShowHeader(null, 'Functions overview');

	$haveFunctions = false;
	foreach ($items as $item) if ($item['type'] == 'function')
	{
		$haveFunctions = true;
		break;
	}

	if ($haveFunctions)
	{
		echo '<p>
<table cellspacing="0" cellpadding="0">
';
		foreach ($items as $item) if ($item['type'] == 'function')
		{
			$extra = '';
			if ($item['lang'] == 'js')
				$extra .= 'JS';
			if ($extra)
				$extra = '&nbsp; <small>[' . $extra . ']</small>';

			echo '<tr>
	<td class="rpad"><a href="' . GetFunctionLink($item) . '">' . $item['name'] . '</a>' . $extra . '</td>
	<td>
		';
			if ($item['desc'])
				echo FindLinks(nl2br_(htmlspecialchars(trim($item['desc']))));
			else
				echo '<span class="grey">No description available.</span>';
			echo '
	</td>
</tr>
';
		}
		echo '</table>
</p>
';
	}
	else
	{
		echo '<p>No global functions found.</p>';
	}

	ShowFooter();

	$text = ob_get_contents();
	ob_end_clean();

	$filename = $INFO['outpath'] . 'functions.html';
	$fp = fopen($filename, 'w');
	fwrite($fp, $text);
	fclose($fp);
}


// ==============================  WriteClassesOverview  ==============================

function WriteClassesOverview(&$items)
{
	global $INFO;

	ob_start();

	ShowHeader(null, 'Classes overview');

	$haveClasses = false;
	foreach ($items as $item) if ($item['type'] == 'class')
	{
		$haveClasses = true;
		break;
	}

	if ($haveClasses)
	{
		echo '<p>
<table cellspacing="0" cellpadding="0">
';
		foreach ($items as $item) if ($item['type'] == 'class')
		{
			echo '<tr>
	<td class="rpad"><a href="' . GetClassLink($item) . '">' . $item['name'] . '</a></td>
	<td>
		';
			if ($item['desc'])
				echo FindLinks(nl2br_(htmlspecialchars(firstparagraph(trim($item['desc'])))));
			else
				echo '<span class="grey">No description available.</span>';
			echo '
	</td>
</tr>
';
		}
		echo '</table>
</p>
';
	}
	else
	{
		echo '<p>No classes found.</p>';
	}

	ShowFooter();

	$text = ob_get_contents();
	ob_end_clean();

	$filename = $INFO['outpath'] . 'classes.html';
	$fp = fopen($filename, 'w');
	fwrite($fp, $text);
	fclose($fp);
}


// ==============================  WriteConstantsOverview  ==============================

function WriteConstantsOverview(&$items)
{
	global $INFO;

	ob_start();

	ShowHeader(null, 'Constants overview');

	$haveConstants = false;
	foreach ($items as $item) if ($item['type'] == 'constant')
	{
		$haveConstants = true;
		break;
	}

	if ($haveConstants)
	{
		echo '<p>
<table cellspacing="0" cellpadding="0">
';
		foreach ($items as $item) if ($item['type'] == 'constant')
		{
			$id = preg_replace('_[^a-z0-9]_i', '_', strtolower($item['name']));
			if ($item['class'] != '')
				$id = preg_replace('_[^a-z0-9]_i', '_', strtolower($item['class'])) . '-' . $id;

			echo '<tr id="' . $id . '">
	<td class="rpad">' . ($item['class'] != '' ? $item['class'] . '::' : '') . $item['name'] . '</td>
	<td class="rpad">' . $item['value'] . '</td>
	<td class="rpad">
		';
			if ($item['desc'])
				echo FindLinks(nl2br_(htmlspecialchars(trim($item['desc']))));
			else
				echo '<span class="grey">No description available.</span>';
			echo '
	</td>
	<td style="white-space: nowrap;">
		File: <a href="' . GetFileLink($item) . '">' . $item['file'] . '</a>, <nobr>line ' . $item['lineno'] . '</nobr>
	</td>
</tr>
';
		}
		echo '</table>
</p>
';
	}
	else
	{
		echo '<p>No global constants found.</p>';
	}

	ShowFooter();

	$text = ob_get_contents();
	ob_end_clean();

	$filename = $INFO['outpath'] . 'constants.html';
	$fp = fopen($filename, 'w');
	fwrite($fp, $text);
	fclose($fp);
}


// ==============================  WriteFile  ==============================

function WriteFile($file, &$items)
{
	global $INFO;

	ob_start();

	ShowHeader(null, $file . ' contents');

	// File description
	echo '<p class="indent">
';
	$count = 0;
	foreach ($items as $item)
	{
		if ($item['type'] == 'file' && $item['file'] == $file)
		{
			$count++;
			echo FindLinks(nl2br_(htmlspecialchars(trim($item['desc']))));
		}
	}
	if (!$count)
		echo 'No file description available.';
	echo '
</p>
';

	// Functions in file
	$functions = array();
	foreach ($items as $item)
	{
		if ($item['type'] == 'function' && $item['file'] == $file && $item['class'] == '')
		{
			$functions[strtolower($item['name'])] = $item;
		}
	}
	ksort($functions);

	if ($functions)
	{
	echo '<h2>Functions</h2>
<p class="indent">
<table cellspacing="0" cellpadding="0">
';
		foreach ($functions as $item)
		{
			echo '<tr>
	<td class="rpad"><a href="' . GetFunctionLink($item) . '">' . $item['name'] . '</a></td>
	<td>
		';
			if ($item['desc'])
				echo FindLinks(nl2br_(htmlspecialchars(trim($item['desc']))));
			else
				echo '<span class="grey">No description available.</span>';
			echo '
	</td>
</tr>
';
		}
		echo '</table>
</p>
';
	}

	// Classes in file
	$classes = array();
	foreach ($items as $item)
	{
		if ($item['type'] == 'class' && $item['file'] == $file)
		{
			$classes[strtolower($item['name'])] = $item;
		}
	}
	ksort($classes);

	echo '<h2>Classes</h2>
<p class="indent">
';
	if ($classes)
	{
		foreach ($classes as $item)
		{
			echo '<a href="' . GetClassLink($item) . '">' . $item['name'] . '</a><br>';
		}
	}
	else
		echo 'No classes in this file.';
	echo '
</p>
';

	// Constants in file
	$constants = array();
	foreach ($items as $item)
	{
		if ($item['type'] == 'constant' && $item['file'] == $file)
		{
			$constants[$item['name']] = $item;
		}
	}
	#ksort($constants);

	if ($constants)
	{
	echo '<h2>Constants</h2>
<p class="indent">
<table cellspacing="0" cellpadding="0">
';
		foreach ($constants as $item)
		{
			echo '<tr>
	<td class="rpad">' . $item['name'] . '</td>
	<td class="rpad">' . $item['value'] . '</td>
	<td class="rpad">
		';
			if ($item['desc'])
				echo FindLinks(nl2br_(htmlspecialchars(trim($item['desc']))));
			else
				echo '<span class="grey">No description available.</span>';
			echo '
	</td>
	<td>Line ' . $item['lineno'] . '</td>
</tr>
';
		}
		echo '</table>
</p>
';
	}

	ShowFooter();

	$text = ob_get_contents();
	ob_end_clean();

	$filename = $INFO['outpath'] . GetFileLink($file);
	$fp = fopen($filename, 'w');
	fwrite($fp, $text);
	fclose($fp);
}


// ==============================  WriteFunction  ==============================

function WriteFunction(&$item)
{
	global $INFO;

	ob_start();

	$name = $item['name'];
	#if ($item['class']) $name = $item['class'] . '::' . $name;
	// Kaspersky Anti-Virus 2009 can detect a false positive here, this is a work-around:
	if ($item['class']) $name = $item['class'] . '&#x3A;:' . $name;

	ShowHeader($item, $name . ' ' . ($item['class'] ? 'method' : 'function'));

	echo '<p class="indent">
';
	if ($item['desc'])
		echo FindLinks(nl2br_(htmlspecialchars(trim($item['desc']))));
	else
		echo 'No function description available.';
	echo '
</p>
';

	$params = '';
	foreach ($item['raw_params'] as $rp)
	{
		$params .= ($params ? ', ' : '') . '<i>' . htmlspecialchars($rp) . '</i>';
	}
	echo '<p class="funcsig">
';
	if ($item['class'] && $item['access']) echo $item['access'] . ' ';
	if ($item['class'] && $item['static']) echo 'static ';
	echo 'function <b>' . $name . '</b> (' .  $params . ');';
	echo '
</p>
';

	if ($item['params'])
	{
		echo '<h2>Parameters</h2>
<div class="p indent">
';
		foreach ($item['params'] as $param)
		{
			$extra = '';
			if ($param['dir'] != '' && $param['dir'] != 'in')
				$extra .= ($extra ? ', ' : '') . $param['dir'];
			if ($param['optional'])
				$extra .= ($extra ? ', ' : '') . 'optional';
			if ($param['default'] !== null && $param['default'] !== '')
				$extra .= ($extra ? ', ' : '') . 'default: ' . htmlspecialchars($param['default']);
			if ($extra)
				$extra = '&nbsp; <small>&ndash;&nbsp; ' . $extra . '</small>';

			echo '<div class="param">
	<div class="name">' . $param['name'] . $extra . '</div>
	<div class="desc">';
			if ($param['desc'])
				echo FindLinks(nl2br_(htmlspecialchars(trim($param['desc']))));
			echo '</div>
</div>
';
		}
		echo '</div>
';
	}

	if ($item['returns']) echo '<h2>Return value</h2>
<p class="indent">
' . FindLinks(nl2br_(htmlspecialchars(trim($item['returns'])))) . '
</p>
';

	if ($item['longdesc'])
		echo '<h2>Remarks</h2>
<p class="indent">
' . FindLinks(nl2br_(htmlspecialchars(trim($item['longdesc'])))) . '
</p>
';

	if ($item['throws'])
	{
		//$itemThrows = FindLinks(nl2br_(htmlspecialchars(trim($item['throws']))));

		echo '<h2>Exceptions thrown</h2>
<p class="indent">
<table cellspacing="0" cellpadding="0">
';

		$str = trim($item['throws']);
		while (preg_match('_^\s*((?:{{)?[0-9a-z\_]+Exception(?:}})?)(.*?(?:\n|$))?(.*)$_is', $str, $m))
		{
			$exName = $m[1];
			$desc = trim($m[2]);
			$str = $m[3];

			if (strlen($desc))
				$desc = FindLinks(nl2br_(htmlspecialchars($desc)));
			else
				$desc = '<span class="grey">No description available.</span>';

			echo '<tr>';
			echo '<td>' . FindLinks(htmlspecialchars($exName)) . '&nbsp;</td>';
			echo '<td>' . $desc . '</td>';
			echo '</tr>';
		}

		echo '</table>
</p>
';
	}

	if ($item['uses']) echo '<h2>Uses data</h2>
<p class="indent">
' . FindLinks(nl2br_(htmlspecialchars(trim($item['uses'])))) . '
</p>
';

	if ($item['exports']) echo '<h2>Exports data</h2>
<p class="indent">
' . FindLinks(nl2br_(htmlspecialchars(trim($item['exports'])))) . '
</p>
';

	if ($item['privileges'])
		echo '<h2>Privileges required</h2>
<p class="indent">
' . FindLinks(nl2br_(htmlspecialchars(trim($item['privileges'])))) . '
</p>
';

	if ($item['see']) echo '<h2>See also</h2>
<p class="indent">
' . FindLinks(nl2br_(htmlspecialchars(trim($item['see'])))) . '
</p>
';

	if ($item['class'] != '')
		echo '<p>Member of class: <a href="' . GetClassLink($item) . '">' . $item['class'] . '</a></p>
';

	echo '<p>Implementation file: <a href="' . GetFileLink($item) . '">' . $item['file'] . '</a>, line ' . $item['lineno'] . '</p>
';

	ShowFooter();

	$text = ob_get_contents();
	ob_end_clean();

	$filename = $INFO['outpath'] . GetFunctionLink($item);
	$fp = fopen($filename, 'w');
	fwrite($fp, $text);
	fclose($fp);
}


// ==============================  WriteClass  ==============================

function WriteClass(&$item, &$items)
{
	global $INFO;

	ob_start();

	$name = $item['name'];

	ShowHeader($item, $name . ($item['abstract'] ? ' abstract' : '') . ' class');

	echo '<p class="indent">
';
	if ($item['desc'])
		echo FindLinks(nl2br_(htmlspecialchars(trim($item['desc']))));
	else
		echo 'No class description available.';
	echo '
</p>
';

	if ($item['extends']) echo '<h2>Extends class</h2>
<p class="indent">
' . FindLinks(nl2br_(htmlspecialchars(trim($item['extends'])))) . '
</p>
';

	if ($item['longdesc'])
		echo '<h2>Remarks</h2>
<p class="indent">
' . FindLinks(nl2br_(htmlspecialchars(trim($item['longdesc'])))) . '
</p>
';

	$methods = array();
	foreach ($items as $subitem) if ($subitem['type'] == 'function' && $subitem['class'] == $item['name'] && $subitem['access'] == 'public')
	{
		$key = strtolower($subitem['name']);
		if ($subitem['name']{0} == '_') $key = ' ' . $key;
		$methods[$key] = $subitem;
	}
	ksort($methods);
	echo '<h2>Public methods</h2>
<p class="indent">
';
	if ($methods)
	{
		echo '<table cellspacing="0" cellpadding="0">
';
		foreach ($methods as $method)
		{
			$methodName = $method['name'];
			if ($method['name'] == $method['class'] || $method['name'] == '__construct') $methodName = '<i>Class constructor</i>';

			echo '<tr>
	<td>' . ($method['static'] ? 'S' : '&nbsp;&nbsp;') . '</td>
	<td class="rpad"><a href="' . GetFunctionLink($method) . '">' . $methodName . '</a></td>
	<td>
		';
			if ($method['desc'])
				echo FindLinks(nl2br_(htmlspecialchars(trim($method['desc']))));
			else
				echo '<span class="grey">No description available.</span>';
			echo '
	</td>
</tr>
';
		}
		echo '</table>
';
	}
	else
	{
		echo 'No public methods in this class.';
	}
	echo '</p>
';

	$methods = array();
	foreach ($items as $subitem) if ($subitem['type'] == 'function' && $subitem['class'] == $item['name'] && $subitem['access'] == 'protected')
	{
		$key = strtolower($subitem['name']);
		if ($subitem['name']{0} == '_') $key = ' ' . $key;
		$methods[$key] = $subitem;
	}
	ksort($methods);
	if ($methods)
	{
		echo '<h2>Protected methods</h2>
<p class="indent">
<table cellspacing="0" cellpadding="0">
';
		foreach ($methods as $method)
		{
			$methodName = $method['name'];
			if ($method['name'] == $method['class'] || $method['name'] == '__construct') $methodName = '<i>Class constructor</i>';

			echo '<tr>
	<td>' . ($method['static'] ? 'S' : '&nbsp;&nbsp;') . '</td>
	<td class="rpad"><a href="' . GetFunctionLink($method) . '">' . $methodName . '</a></td>
	<td>
		';
			if ($method['desc'])
				echo FindLinks(nl2br_(htmlspecialchars(trim($method['desc']))));
			else
				echo '<span class="grey">No description available.</span>';
			echo '
	</td>
</tr>
';
		}
		echo '</table>
</p>
';
	}

	$methods = array();
	foreach ($items as $subitem) if ($subitem['type'] == 'function' && $subitem['class'] == $item['name'] && $subitem['access'] == 'private')
	{
		$key = strtolower($subitem['name']);
		if ($subitem['name']{0} == '_') $key = ' ' . $key;
		$methods[$key] = $subitem;
	}
	ksort($methods);
	if ($methods)
	{
		echo '<h2>Private methods</h2>
<p class="indent">
<table cellspacing="0" cellpadding="0">
';
		foreach ($methods as $method)
		{
			$methodName = $method['name'];
			if ($method['name'] == $method['class'] || $method['name'] == '__construct') $methodName = '<i>Class constructor</i>';

			echo '<tr>
	<td>' . ($method['static'] ? 'S' : '&nbsp;&nbsp;') . '</td>
	<td class="rpad"><a href="' . GetFunctionLink($method) . '">' . $methodName . '</a></td>
	<td>
		';
			if ($method['desc'])
				echo FindLinks(nl2br_(htmlspecialchars(trim($method['desc']))));
			else
				echo '<span class="grey">No description available.</span>';
			echo '
	</td>
</tr>
';
		}
		echo '</table>
</p>
';
	}

	$fields = array();
	foreach ($items as $subitem) if ($subitem['type'] == 'field' && $subitem['class'] == $item['name'] && $subitem['access'] == 'public')
		$fields[strtolower($subitem['name'])] = $subitem;
	ksort($fields);
	if ($fields)
	{
		echo '<h2>Public fields</h2>
<div class="p indent">
';
		foreach ($fields as $field)
		{
			$extra = '';
			if ($field['default'] !== null && $field['default'] !== '')
				$extra .= ($extra ? ', ' : '') . 'default: ' . htmlspecialchars($field['default']);
			if ($extra)
				$extra = '&nbsp; <small>&ndash;&nbsp; ' . $extra . '</small>';

			echo '<div class="param">
<div class="name">' . $field['name'] . $extra . '</div>
<div class="desc">' . FindLinks(nl2br_(htmlspecialchars(trim($field['desc']))));
			echo '</div>
</div>
';
		}
		echo '</div>
';
	}

	$fields = array();
	foreach ($items as $subitem) if ($subitem['type'] == 'field' && $subitem['class'] == $item['name'] && $subitem['access'] == 'protected')
		$fields[strtolower($subitem['name'])] = $subitem;
	ksort($fields);
	if ($fields)
	{
		echo '<h2>Protected fields</h2>
<div class="p indent">
';
		foreach ($fields as $field)
		{
			$extra = '';
			if ($field['default'] !== null && $field['default'] !== '')
				$extra .= ($extra ? ', ' : '') . 'default: ' . htmlspecialchars($field['default']);
			if ($extra)
				$extra = '&nbsp; <small>&ndash;&nbsp; ' . $extra . '</small>';

			echo '<div class="param">
<div class="name">' . $field['name'] . $extra . '</div>
<div class="desc">' . FindLinks(nl2br_(htmlspecialchars(trim($field['desc']))));
			echo '</div>
</div>
';
		}
		echo '</div>
';
	}

	$fields = array();
	foreach ($items as $subitem) if ($subitem['type'] == 'field' && $subitem['class'] == $item['name'] && $subitem['access'] == 'private')
		$fields[strtolower($subitem['name'])] = $subitem;
	ksort($fields);
	if ($fields)
	{
		echo '<h2>Private fields</h2>
<div class="p indent">
';
		foreach ($fields as $field)
		{
			$extra = '';
			if ($field['default'] !== null && $field['default'] !== '')
				$extra .= ($extra ? ', ' : '') . 'default: ' . htmlspecialchars($field['default']);
			if ($extra)
				$extra = '&nbsp; <small>&ndash;&nbsp; ' . $extra . '</small>';

			echo '<div class="param">
<div class="name">' . $field['name'] . $extra . '</div>
<div class="desc">' . FindLinks(nl2br_(htmlspecialchars(trim($field['desc']))));
			echo '</div>
</div>
';
		}
		echo '</div>
';
	}

	$haveConstants = false;
	foreach ($items as $subitem)
		if ($subitem['type'] == 'constant' && $subitem['class'] == $item['name'])
		{
			$haveConstants = true;
			break;
		}

	if ($haveConstants)
	{
		echo '<h2>Constants</h2>
<p class="indent">
<table cellspacing="0" cellpadding="0">
';
		foreach ($items as $subitem) if ($subitem['type'] == 'constant' && $subitem['class'] == $item['name'])
		{
			$id = preg_replace('_[^a-z0-9]_i', '_', strtolower($subitem['name']));

			echo '<tr id="' . $id . '">
	<td class="rpad">' . $subitem['name'] . '</td>
	<td class="rpad">' . $subitem['value'] . '</td>
	<td class="rpad">
			';
			if ($subitem['desc'])
				echo FindLinks(nl2br_(htmlspecialchars(trim($subitem['desc']))));
			else
				echo '<span class="grey">No description available.</span>';
			echo '
	</td>
	<td style="white-space: nowrap;">
		File: <a href="' . GetFileLink($subitem) . '">' . $subitem['file'] . '</a>, <nobr>line ' . $subitem['lineno'] . '</nobr>
	</td>
</tr>
';
		}
		echo '</table>
</p>
';
	}

	echo '<p>Implementation file: <a href="' . GetFileLink($item) . '">' . $item['file'] . '</a>, line ' . $item['lineno'] . '</p>
';

	ShowFooter();

	$text = ob_get_contents();
	ob_end_clean();

	$filename = $INFO['outpath'] . GetClassLink($item);
	$fp = fopen($filename, 'w');
	fwrite($fp, $text);
	fclose($fp);
}


// ==============================  ShowHeader  ==============================

function ShowHeader($item, $title)
{
	global $INFO;

	echo '<html>
<head>
<title>' . $title . ' - ' . $INFO['title'] . '</title>
<link rel="stylesheet" href="coderef.css" type="text/css" />
</head>

<body>
<div class="headarea">
<div class="title"><a href="index.html">' . $INFO['title'] . '</a></div>
<h1>' . $title;
	if (is_array($item) && $item['lang'] == 'js') echo ' <small>[JavaScript]</small>';
	echo '</h1>
</div>
';
}


// ==============================  ShowFooter  ==============================

function ShowFooter()
{
	global $INFO;

	echo '<p class="footline">
' . $INFO['footline'] . '</p>

</body>
</html>
';
}


// ==============================  FindLinks  ==============================

function FindLinksCmpFn($a, $b)
{
	return strlen($a['name']) < strlen($b['name']);
}

function FindLinksCmpFn_class($a, $b)
{
	return strlen($a['class'] . $a['name']) < strlen($b['class'] . $b['name']);
}

function FindLinksRegSafe($str)
{
	return preg_replace('/([\/\\\\\\^\\$.\\[\\]|()?*+{}])/', '\\\\$1', $str);
}

function FindLinks($str)
{
	global $globalitems, $globalitems_class;

	$replaced = array();
	foreach ($globalitems_class as $item)
	{
		if (isset($item))
		{
			$name = $item['class'] . '::' . $item['name'];
			if (strpos($str, $name) !== false &&
				!in_array($name, $replaced))
			{
				$str = preg_replace('/(?<=^|[ \t\r\n.,:;()\[\]-])(' . FindLinksRegSafe($name) . ')(?=$|[ \t\r\n.,:;()\[\]-])/', '<a href="' . GetItemLink($item) . '">$1</a>', $str);
				$replaced[] = $name;
			}
		}
	}

	$replaced = array();
	foreach ($globalitems as $item)
	{
		if (isset($item))
		{
			$name = $item['name'];
			if (strpos($str, $name) !== false &&
				!in_array($name, $replaced) &&
				!$item['ambigous'])
			{
				$str = preg_replace(
					'/(?<=^|[ \t\r\n.,:;()\[\]-])(' . FindLinksRegSafe($name) . ')(?=$|[ \t\r\n.,:;()\[\]-])/',
					'<a href="' . GetItemLink($item) . '">$1</a>',
					$str);
				// Special case: Find identifier alone in a {{...}} tag
				$str = preg_replace(
					'/({{' . FindLinksRegSafe($name) . '}})/',
					'<a href="' . GetItemLink($item) . '">$1</a>',
					$str);
				$replaced[] = $name;
			}
		}
	}

	// Do some basic markup
	$str = preg_replace('/{{([\t\r\n 0-9a-z_]+)}}/i', '<code>$1</code>', $str);

	return $str;
}

function nl2br_($str)
{
	$str = preg_replace('_\r?\n[ \t]*\r?\n_', '&par;', $str);
	$str = preg_replace('_[ \t]*\r?\n[ \t]*(?!\(|\* \S)_', ' ', $str);
	$str = str_replace('&par;', "\n\n", $str);
	return nl2br($str);
}

function firstparagraph($str)
{
	return preg_replace('_\r?\n\r?\n.*$_s', '', $str);
}

function WriteCssFile($filename)
{
	$css = 'body
{
	margin: 0;
	padding: 10px;
}
body, p, td
{
	font: 13px Segoe UI, sans-serif;
}

a,
a:visited
{
	color: blue;
	text-decoration: none;
}
a:hover,
a:active
{
	text-decoration: underline;
}

div.headarea
{
	margin: -10px -10px 16px;
	padding: 8px 10px 6px;
	background: #BBDDFF;
	border-bottom: solid 1px #808080;
}
div.headarea div.title
{
	font-size: 0.9em;
}
h1
{
	font-size: 1.4em;
	font-weight: bold;
	margin: 3px 0 2px;
	padding: 0;
}
h1 small
{
	font-size: 11px;
}
h2
{
	font-size: 1em;
	font-weight: bold;
	margin: 0 0 2px;
	padding: 0;
}

p,
div.p
{
	margin: 0 0 12px;
}
p.indent,
div.p.indent
{
	padding-left: 20px;
}

small
{
	font-size: 0.9em;
	color: #606060;
}
div.param div.name small
{
	font-style: normal;
}

table
{
	border-collapse: collapse;
}
td
{
	vertical-align: top;
	border: solid 1px #B0B0B0;
	padding: 2px 8px 2px 4px;
}
td.rpad
{
	padding-right: 15px;
}

p.funcsig
{
	margin: 12px -2px;
	background: #EEEEEE;
	padding: 3px 2px;
}

div.param
{
	margin-bottom: 4px;
}
div.param div.name
{
	font-style: italic;
}
div.param div.desc
{
	margin-left: 20px;
}

.grey
{
	color: #808080;
}

code
{
	font-family: Consolas, Andale Mono, Courier New, monospace;
	/*background: #FFFFDD;*/
}

p.footline
{
	margin-top: 20px;
	border-top: solid 1px #808080;
	padding-top: 4px;
	font-size: 0.9em;
	color: #606060;
}
';

	$fp = fopen($filename, 'w');
	fwrite($fp, $css);
	fclose($fp);
}

?>