<?php
// Encoding: UTF-8 (without BOM) (auto-detect: )
header('Content-Type: text/html; charset=utf-8');

$startTime = microtime(true);

// Retain output until after the session is initialised
ob_start();

echo '<p><big>UNB Components setup script</big></p>';

echo 'Including library and class files...<br>';
#require_once('unb_lib/unb_all.lib.php');
if (!class_exists('UnbEnvironment')) require_once('unb_lib/environment.class.php');
if (!class_exists('UnbDatabase')) require_once('unb_lib/database.class.php');
if (!class_exists('UnbUser')) require_once('unb_lib/user.class.php');
if (!class_exists('UnbMessage')) require_once('unb_lib/message.class.php');
if (!class_exists('UnbTag')) require_once('unb_lib/tag.class.php');
if (!class_exists('UnbSession')) require_once('unb_lib/session.class.php');
echo 'Script execution time: ' . round((microtime(true) - $startTime) * 1000, 1) . ' ms<br>';

UnbConfiguration::SetFilename('../unb_config.inc.php');

$db = UnbDatabase::GetInstance();

echo 'Connecting to database...<br>';
$db->Open();
echo 'Script execution time: ' . round((microtime(true) - $startTime) * 1000, 1) . ' ms<br>';

echo 'Clearing all stored data...<br>';
$db->StartTransaction();
$db->Execute('DELETE FROM "#message"');
$db->Execute('DELETE FROM "#message_rating"');
$db->Execute('DELETE FROM "#message_reference"');
$db->Execute('DELETE FROM "#message_revision"');
$db->Execute('UPDATE "#tag" SET "ParentTag" = NULL');
$db->Execute('DELETE FROM "#tag"');
$db->Execute('DELETE FROM "#message_revision_tag"');
$db->Execute('DELETE FROM "#user"');
$db->Execute('DELETE FROM "#user_config"');
$db->Execute('DELETE FROM "#user_vcard"');
$db->Execute('DELETE FROM "#keylist"');
$db->Commit();

echo 'Starting session...<br>';
UnbSession::EnableSession();

echo 'Creating system role keys...<br>';
$user_system = UnbUser::Create('System key', null, null, UnbUserKey::System);
$user_admin = UnbUser::Create('Administrator key', null, null, UnbUserKey::Administrator);
$user_mod = UnbUser::Create('Moderator key', null, null, UnbUserKey::Moderator);
$user_trusted = UnbUser::Create('Trusted user key', null, null, UnbUserKey::Trusted);
$user_default = UnbUser::Create('Default user key', null, null, UnbUserKey::DefaultUser);

echo 'Logging in as admin user...<br>';
// Setup some things as administrator
UnbSession::SetSessionUser($user_admin, null, true);

echo 'Creating the first user...<br>';
$user_first = UnbUser::Create('First user', 'first', 'first');
$user_first->AddAdditionalKey($user_mod);
$user_first->AddAdditionalKey($user_admin);
$user_first->AddAdditionalKey($user_trusted);

echo 'Logging in as first user...<br>';
// Now work on with a regular user
UnbSession::SetSessionUser($user_first, 'first');

echo 'Setting up first user\'s vcard...<br>';
$user_first->SetVcardEntry('RealName', 'First User', UnbUserVcardVisibility::Public_);

echo 'Logging out...<br>';
// Log out to clear auto-login cookie which may be set in the wrong URL path here
UnbSession::SetSessionUser(null);

$dbStat = $db->GetStatistics();
echo 'Database statistics: ' . $dbStat['queryCount'] . ' queries took ' . round($dbStat['totalTime'] * 1000, 1) . ' ms<br>';

echo 'Script execution time: ' . round((microtime(true) - $startTime) * 1000, 1) . ' ms<br>';

echo '<p><b>Finished</b></p>';

?>