<?php
// UNB Components
// Copyright 2007-2010 by Yves Goergen
// Website: http://unclassified.de/go/unb2
// Licence: GNU General Public License (GPL) Version 3
class UNB
{
}
class UnbDatabaseException extends Exception
{
}
class UnbSecurityException extends Exception
{
}
abstract class UnbEntity
{
protected static function FromRecord($rec)
{
}
}
class UnbDebug
{
public static function Trace($text)
{
$text = date('Y-m-d H:i:s ') . $text . "\n";
file_put_contents(dirname(__FILE__) . '/unb.trace', $text, FILE_APPEND);
}
}
class UnbDatabase
{
const DisplayQueryErrors = true;
const DisplayQueryInfo = false;
private static $instance = null;
private $connectData = null;
private $connection = null;
private $driverName = '';
private $tablePrefix = '';
private $transactionLevel = 0;
private $databaseLocked = false;
private $queryCount = 0;
private $totalTime = 0.0;
private $rowsAffected = 0;
private $constraintError = false;
private $busyError = false;
private function __construct()
{
}
public static function GetInstance()
{
if (!isset(self::$instance))
{
self::$instance = new self();
}
return self::$instance;
}
public function IsOpen()
{
return isset($this->connection);
}
public function GetDriverName()
{
return $this->driverName;
}
public function GetStatistics()
{
return array(
'queryCount' => $this->queryCount,
'totalTime' => $this->totalTime);
}
public function Open($connectData = null)
{
if (isset($connectData))
{
if (!is_array($connectData))
throw new InvalidArgumentException('[#1] Invalid argument type: $connectData');
}
else
{
$entries = UnbConfiguration::Get('database.*');
$connectData = array();
foreach ($entries as $name => $data)
{
$connectData[substr($name, 9)] = $data;
}
}
$this->connectData = $connectData;
if ($connectData['table-prefix'] != null)
$this->tablePrefix = $connectData['table-prefix'];
switch ($this->connectData['driver'])
{
case 'mysql':
$dsn = 'mysql:dbname=' . $this->connectData['dbname'] .
(isset($this->connectData['hostname']) ? ';host=' . $this->connectData['port'] : '') .
(isset($this->connectData['port']) ? ';port=' . $this->connectData['port'] : '') .
(isset($this->connectData['unix-socket']) ? ';unix_socket=' . $this->connectData['unix-socket'] : '');
break;
case 'odbc':
$dsn = 'odbc:' . $this->connectData['dsn'];
break;
case 'pgsql':
$dsn = 'pgsql:host=' . $this->connectData['hostname'] .
(isset($this->connectData['port']) ? ';port=' . $this->connectData['port'] : '') .
';dbname=' . $this->connectData['dbname'];
break;
case 'sqlite':
$dsn = 'sqlite:' . dirname(__FILE__) . '/' . $this->connectData['filename'];
break;
default:
throw new InvalidArgumentException('Unsupported database driver name. Check your configuration.');
}
$this->driverName = $this->connectData['driver'];
$parameters = array();
if ($connectData['persistent'])
$parameters[PDO::ATTR_PERSISTENT] = true;
$this->connection = new PDO($dsn, $this->connectData['username'], $this->connectData['password'], $parameters);
switch ($this->driverName)
{
case 'mysql':
$this->connection->exec('SET sql_mode = \'ANSI\'');
$this->connection->exec('SET storage_engine = \'InnoDB\'');
$this->connection->exec('SET CHARACTER SET \'utf8\'');
$this->connection->exec('SET NAMES \'utf8\'');
break;
case 'pgsql':
if (isset($this->connectData['schema']))
$this->connection->exec('SET search_path TO ' . $this->connection->quote($this->connectData['schema']));
break;
case 'sqlite':
if (isset($this->connectData['synchronous']))
$this->connection->exec('PRAGMA synchronous = ' . $this->connectData['synchronous']);
$this->connection->sqliteCreateFunction('LOWER', 'mb_strtolower', 1);
break;
}
}
public function Close()
{
$this->connection = null;
}
private function ShowQueryInfo($ps, $r, $parameters, $time)
{
if (!$r || self::DisplayQueryInfo)
{
echo '<div class="sql-info"><b>SQL execute:</b> ';
if ($ps instanceof PDOStatement)
{
$sql = htmlspecialchars($ps->queryString, ENT_NOQUOTES);
}
elseif ($ps instanceof PDO)
{
$sql = htmlspecialchars($parameters, ENT_NOQUOTES);
$parameters = null;
}
if ($parameters)
{
foreach ($parameters as $p)
{
if (!isset($p))
$p = 'NULL';
elseif (is_bool($p))
$p = $p ? '1' : '0';
elseif (!is_numeric($p))
$p = '\'' . htmlspecialchars(mb_substr($p, 0, 30), ENT_NOQUOTES) . '\'';
$sql = preg_replace('_\?_', '<i>' . $p . '</i>', $sql, 1);
}
}
echo $sql;
if ($time >= 0)
{
echo ' &ndash; ' . ($time >= 0.005 ? '<b>' : '') . round($time * 1000, 1) . 'ms' . ($time >= 0.005 ? '</b>' : '');
}
echo '</div>';
}
if (!$r)
{
$ei = $ps->errorInfo();
echo '<div class="sql-error"><b>SQL error:</b> [SQLSTATE:' . $ei[0] . ', ' . $ei[1] . '] ' . htmlspecialchars($ei[2], ENT_NOQUOTES) . '</div>';
}
}
private function DetermineErrorCondition($ps)
{
$this->constraintError = false;
$this->busyError = false;
$ei = $ps->errorInfo();
if (substr($ei[0], 0, 2) == '23')
$this->constraintError = true;
switch ($this->driverName)
{
case 'sqlite':
if ($ei[1] == 5)
{
$this->busyError = true;
usleep(mt_rand(2000, 20000));
}
elseif ($ei[1] == 19)
$this->constraintError = true;
break;
}
}
public function IsConstraintError()
{
return $this->constraintError;
}
public function Optimise($extensive = false)
{
$r = true;
switch ($this->driverName)
{
case 'mysql':
if ($extensive)
{
$r &= $this->Query('OPTIMIZE TABLE "#keylist", "#message", "#message_rating", "#message_reference", "#message_revision", "#message_revision_tag", "#tag", "#user", "#user_config", "#user_vcard"');
}
else
{
$r &= $this->Query('ANALYZE TABLE "#keylist", "#message", "#message_rating", "#message_reference", "#message_revision", "#message_revision_tag", "#tag", "#user", "#user_config", "#user_vcard"');
}
break;
case 'pgsql':
if ($extensive)
{
$r &= $this->Execute('VACUUM FULL');
$r &= $this->Execute('REINDEX DATABASE ?', $this->connectData['database']);
}
else
{
$r &= $this->Execute('VACUUM');
$r &= $this->Execute('ANALYZE');
}
break;
case 'sqlite':
if ($extensive)
{
$r &= $this->Execute('VACUUM');
$r &= $this->Execute('ANALYZE');
}
else
{
$r &= $this->Execute('ANALYZE');
}
break;
}
return $r;
}
public function StartTransaction($lock = false)
{
if (!isset($this->connection))
throw new BadMethodCallException('Database not opened');
if ($lock && $this->transactionLevel > 0 && !$this->databaseLocked)
throw new BadMethodCallException('You cannot lock during a transaction that has no lock.');
$r = true;
if ($this->transactionLevel == 0)
{
$start = microtime(true);
do
{
if ($lock)
{
switch ($this->driverName)
{
case 'sqlite':
ignore_user_abort(true);
$ps = $this->connection->prepare('BEGIN EXCLUSIVE TRANSACTION');
$r &= $ps->execute();
break;
default:
$r &= $this->connection->beginTransaction();
}
$this->databaseLocked = true;
}
else
{
$r &= $this->connection->beginTransaction();
}
if (!$r) $this->DetermineErrorCondition($this->connection);
}
while (!$r && $this->busyError);
$time = microtime(true) - $start;
$this->totalTime += $time;
if (self::DisplayQueryErrors)
$this->ShowQueryInfo($this->connection, $r, 'START TRANSACTION', $time);
}
$this->transactionLevel++;
return $r;
}
public function RollBack()
{
if (!isset($this->connection))
throw new BadMethodCallException('Database not opened');
if ($this->transactionLevel == 0)
throw new BadMethodCallException('No transaction is active');
$r = true;
$this->transactionLevel--;
if ($this->transactionLevel == 0)
{
$start = microtime(true);
if ($this->databaseLocked)
{
switch ($this->driverName)
{
case 'sqlite':
$ps = $this->connection->prepare('ROLLBACK');
$r &= $ps->execute();
ignore_user_abort(false);
break;
default:
$r &= $this->connection->rollBack();
}
}
else
{
$r &= $this->connection->rollBack();
}
$time = microtime(true) - $start;
$this->totalTime += $time;
if (self::DisplayQueryErrors)
$this->ShowQueryInfo($this->connection, $r, 'ROLLBACK', $time);
$this->databaseLocked = false;
}
return $r;
}
public function Commit()
{
if (!isset($this->connection))
throw new BadMethodCallException('Database not opened');
if ($this->transactionLevel == 0)
throw new BadMethodCallException('No transaction is active');
$r = true;
$this->transactionLevel--;
if ($this->transactionLevel == 0)
{
$start = microtime(true);
if ($this->databaseLocked)
{
switch ($this->driverName)
{
case 'sqlite':
$ps = $this->connection->prepare('COMMIT');
$r &= $ps->execute();
ignore_user_abort(false);
break;
default:
$r &= $this->connection->commit();
}
}
else
{
$r &= $this->connection->commit();
}
$time = microtime(true) - $start;
$this->totalTime += $time;
if (!$r) $this->DetermineErrorCondition($this->connection);
if (self::DisplayQueryErrors)
$this->ShowQueryInfo($this->connection, $r, 'COMMIT', $time);
$this->databaseLocked = false;
}
return $r;
}
private function ProcessSql($sql)
{
switch ($this->driverName)
{
case 'sqlite':
if (preg_match('_^\s*SELECT\s+.+(\s+FOR\s+UPDATE)$_is', $sql, $m))
{
if (!$this->databaseLocked && self::DisplayQueryErrors)
{
echo '<div style="color: gray; font-size: 0.85em;"><b>SQL execute:</b> ' . htmlspecialchars($sql) . '</div>';
echo '<div style="color: red; font-size: 0.85em;"><b>SQL warning:</b> A SELECT ... FOR UPDATE query was issued but the database was not locked for the transaction.</div>';
}
$sql = substr($sql, 0, -strlen($m[1]));
}
break;
}
return preg_replace('_\s*--\s.*?$_m', '',
preg_replace('_(?<=")#(?=[a-zA-Z])_', $this->tablePrefix, $sql));
}
public function Prepare($sql)
{
if (!is_string($sql))
throw new InvalidArgumentException('Invalid argument type: $sql');
if (!isset($this->connection))
throw new BadMethodCallException('Database not opened');
$sql = $this->ProcessSql($sql);
$ps = $this->connection->prepare($sql);
if (!$ps)
{
$this->DetermineErrorCondition($this->connection);
$this->ShowQueryInfo($this->connection, false, $sql, 0);
throw new UnbDatabaseException('Statement could not be prepared.');
}
return $ps;
}
private function InternalExecute($sql, $parameters)
{
if (!isset($this->connection))
throw new BadMethodCallException('Database not opened');
$start = microtime(true);
if (is_string($sql))
{
$ps = $this->Prepare($sql);
}
elseif ($sql instanceof PDOStatement)
{
$ps = $sql;
}
else
{
throw new InvalidArgumentException('Invalid argument type: $sql');
}
if (!is_array($parameters) && isset($parameters))
{
$parameters = array($parameters);
}
do
{
$r = $ps->execute($parameters);
if (!$r) $this->DetermineErrorCondition($ps);
}
while (!$r && $this->busyError);
$time = microtime(true) - $start;
$this->totalTime += $time;
$this->queryCount++;
if (self::DisplayQueryErrors)
$this->ShowQueryInfo($ps, $r, $parameters, $time);
if (!$r)
throw new UnbDatabaseException('Statement could not be executed.');
return $ps;
}
public function Execute($sql, $parameters = null)
{
$ps = $this->InternalExecute($sql, $parameters);
if (isset($ps))
$this->rowsAffected = $ps->rowCount();
else
$this->rowsAffected = -1;
}
public function Query($sql, $parameters = null)
{
$ps = $this->InternalExecute($sql, $parameters);
$records = $ps->fetchAll(PDO::FETCH_ASSOC);
return $records;
}
public function QueryArray($sql, $parameters = null)
{
$ps = $this->InternalExecute($sql, $parameters);
$records = $ps->fetchAll(PDO::FETCH_NUM);
$array = array();
foreach ($records as $record)
{
$array[] = $record[0];
}
return $array;
}
public function QueryRecord($sql, $parameters = null)
{
$ps = $this->InternalExecute($sql, $parameters);
$record = $ps->fetch(PDO::FETCH_ASSOC);
$ps->closeCursor();
return $record;
}
public function QueryValue($sql, $parameters = null)
{
$ps = $this->InternalExecute($sql, $parameters);
$value = $ps->fetchColumn();
$ps->closeCursor();
if ($value === false) return null;
return $value;
}
public function GetRowsAffected()
{
return $this->rowsAffected;
}
public static function ConvertDateTime($in)
{
if (!isset($in))
{
return null;
}
if (is_int($in))
{
if ($in == 0) return '0000-00-00 00:00:00';
return gmdate('Y-m-d H:i:s', $in);
}
if (is_string($in))
{
if (preg_match('_^(\d{4})-(\d{2})-(\d{2})(?: (\d{2}):(\d{2}):(\d{2})(\.\d+)?)?$_', $in, $m))
{
if (intval($m[1]) == 0) return 0;
return gmmktime(intval($m[4]), intval($m[5]), intval($m[6]), intval($m[2]), intval($m[3]), intval($m[1]));
}
throw new InvalidArgumentException('Unrecognised date/time format');
}
throw new InvalidArgumentException('Invalid argument type: $in');
}
public function AddKeylistKey($tableName, $columnName, $idColumn, $idValue, $key)
{
if (!is_string($tableName))
throw new InvalidArgumentException('Invalid argument type: $tableName');
if (!preg_match('_^#?[0-9A-Z\_a-z]+$_', $tableName))
throw new InvalidArgumentException('Invalid argument value: $tableName');
if (!is_string($columnName))
throw new InvalidArgumentException('Invalid argument type: $columnName');
if (!preg_match('_^[0-9A-Z\_a-z]+$_', $columnName))
throw new InvalidArgumentException('Invalid argument value: $columnName');
if (!is_string($idColumn))
throw new InvalidArgumentException('Invalid argument type: $idColumn');
if (!preg_match('_^[0-9A-Z\_a-z]+$_', $idColumn))
throw new InvalidArgumentException('Invalid argument value: $idColumn');
if ($key instanceof UnbUser)
$key = $key->GetUserId();
if (!is_int($key))
throw new InvalidArgumentException('Invalid argument type: $key');
if (in_array($key, UnbUser::GetNonListKeys()))
throw new InvalidArgumentException('This special key cannot be added to a key list: ' . $key);
$this->StartTransaction(true);
try
{
$keylistId = $this->QueryValue(
'SELECT "' . $columnName . '" FROM "' . $tableName . '" WHERE "' . $idColumn . '" = ? FOR UPDATE',
$idValue);
$newlist = false;
if (!$keylistId)
{
$newlist = true;
$keylistId = $this->QueryValue('SELECT MAX("KeylistId") FROM "#keylist" FOR UPDATE');
if (!isset($keylistId))
$keylistId = 0;
$keylistId++;
}
$this->Execute(
'INSERT INTO "#keylist" ("KeylistId", "UserId") VALUES (?, ?)',
array($keylistId, $key));
if ($newlist)
{
$this->Execute(
'UPDATE "' . $tableName . '" SET "' . $columnName . '" = ? WHERE "' . $idColumn . '" = ?',
array($keylistId, $idValue));
}
$this->Commit();
}
catch (Exception $ex)
{
$this->RollBack();
throw $ex;
}
}
public function RemoveKeylistKey($tableName, $columnName, $idColumn, $idValue, $key)
{
if (!is_string($tableName))
throw new InvalidArgumentException('Invalid argument type: $tableName');
if (!preg_match('_^#?[0-9A-Z\_a-z]+$_', $tableName))
throw new InvalidArgumentException('Invalid argument value: $tableName');
if (!is_string($columnName))
throw new InvalidArgumentException('Invalid argument type: $columnName');
if (!preg_match('_^[0-9A-Z\_a-z]+$_', $columnName))
throw new InvalidArgumentException('Invalid argument value: $columnName');
if (!is_string($idColumn))
throw new InvalidArgumentException('Invalid argument type: $idColumn');
if (!preg_match('_^[0-9A-Z\_a-z]+$_', $idColumn))
throw new InvalidArgumentException('Invalid argument value: $idColumn');
if ($key instanceof UnbUser)
$key = $key->GetUserId();
if (!is_int($key))
throw new InvalidArgumentException('Invalid argument type: $key');
$this->StartTransaction(true);
try
{
$keylistId = $this->QueryValue(
'SELECT "' . $columnName . '" FROM "' . $tableName . '" WHERE "' . $idColumn . '" = ? FOR UPDATE',
$idValue);
if (!isset($keylistId))
{
$this->RollBack();
return true;
}
$this->Execute(
'DELETE FROM "#keylist" WHERE "KeylistId" = ? AND "UserId" = ?',
array($keylistId, $key));
$val = $this->QueryValue(
'SELECT COUNT(*) FROM "#keylist" WHERE "KeylistId" = ? FOR UPDATE',
$keylistId);
if ($val == 0)
{
$this->Execute(
'UPDATE "' . $tableName . '" SET "' . $columnName . '" = NULL WHERE "' . $idColumn . '" = ?',
$idValue);
}
$this->Commit();
}
catch (Exception $ex)
{
$this->RollBack();
throw $ex;
}
}
public function ResetKeylistKeys($tableName, $columnName, $idColumn, $idValue, $keys)
{
if (!is_string($tableName))
throw new InvalidArgumentException('Invalid argument type: $tableName');
if (!preg_match('_^#?[0-9A-Z\_a-z]+$_', $tableName))
throw new InvalidArgumentException('Invalid argument value: $tableName');
if (!is_string($columnName))
throw new InvalidArgumentException('Invalid argument type: $columnName');
if (!preg_match('_^[0-9A-Z\_a-z]+$_', $columnName))
throw new InvalidArgumentException('Invalid argument value: $columnName');
if (!is_string($idColumn))
throw new InvalidArgumentException('Invalid argument type: $idColumn');
if (!preg_match('_^[0-9A-Z\_a-z]+$_', $idColumn))
throw new InvalidArgumentException('Invalid argument value: $idColumn');
if (isset($keys))
{
if (!is_array($keys))
throw new InvalidArgumentException('Invalid argument type: $keys');
if (!count($keys))
$keys = null;
}
if (isset($keys))
{
$invalidKeys = UnbUser::GetNonListKeys();
foreach ($keys as $key)
if (in_array($key, $invalidKeys))
throw new InvalidArgumentException('This special key cannot be added to a key list: ' . $key);
}
$this->StartTransaction(true);
try
{
$keylistId = $this->QueryValue(
'SELECT "' . $columnName . '" FROM "' . $tableName . '" WHERE "' . $idColumn . '" = ? FOR UPDATE',
$idValue);
if (!isset($keylistId) && !isset($keys))
{
$this->RollBack();
return;
}
if (isset($keylistId))
{
$this->Execute('DELETE FROM "#keylist" WHERE "KeylistId" = ?', $keylistId);
if (!isset($keys))
{
$this->Execute(
'UPDATE "' . $tableName . '" SET "' . $columnName . '" = NULL WHERE "' . $idColumn . '" = ?',
$idValue);
}
}
if (isset($keys))
{
$done = array();
foreach ($keys as $key)
{
if (in_array($key, $done)) continue;
$this->AddKeylistKey($tableName, $columnName, $idColumn, $idValue, $key);
$done[] = $key;
}
}
$this->Commit();
}
catch (Exception $ex)
{
$this->RollBack();
throw $ex;
}
}
public static function EscapeLikeWildcards($s)
{
return str_replace(
array('%',   '_',   '\\',   '*', '?'),
array('\\%', '\\_', '\\\\', '%', '_'),
$s);
}
}
class UnbConfiguration
{
private static $settingsFilename = 'unb_config.inc.php';
private static $settings = null;
private static $systemUser = null;
private function __construct()
{
}
public static function SetFilename($filename)
{
if (isset($filename))
{
if (!is_string($filename))
throw new InvalidArgumentException('Invalid argument type: $filename');
$filename = trim($filename);
if (!strlen($filename))
$filename = null;
}
self::$settingsFilename = $filename;
}
private static function ReadFile()
{
if (isset(self::$settingsFilename))
{
if (self::$settingsFilename{0} == '/')
include self::$settingsFilename;
else
include dirname(__FILE__) . '/' . self::$settingsFilename;
if (!is_array($settings))
throw new Exception('[#1] Configuration file is corrupt.');
self::$settings = $settings;
}
}
public static function Initialise($settings)
{
if (!is_array($settings))
throw new InvalidArgumentException('Invalid argument type: $settings');
self::$settings = $settings;
}
public static function Get($name, $default = null)
{
if (!isset(self::$settings))
self::ReadFile();
$entries = array();
if (strpos($name, '*') === false)
{
if (array_key_exists($name, self::$settings))
return self::$settings[$name];
}
else
{
$rsName = preg_replace('_([\\\\^$.\\]|()?+{}\\_])_', '\\\\$1', $name);
$rsName = str_replace('*', '.*', $rsName);
foreach (self::$settings as $myname => $data)
{
if (preg_match('_^' . $rsName . '$_i', $myname))
$entries[$myname] = $data;
}
}
$db = UnbDatabase::GetInstance();
if ($db->IsOpen())
{
if (!isset(self::$systemUser))
self::$systemUser = UnbUser::FindByUserId(UnbUserKey::System);
$entries = array_merge(self::$systemUser->GetSettings($name), $entries);
}
if (strpos($name, '*') === false)
{
if (count($entries) > 0)
return $entries[0];
else
return $default;
}
else
return $entries;
}
public static function Set($name, $data)
{
if (!isset(self::$settings))
self::ReadFile();
if (!isset(self::$systemUser))
self::$systemUser = UnbUser::FindByUserId(UnbUserKey::System);
self::$systemUser->SetSetting($name, $data);
self::$settings[$name] = $data;
}
}
class UnbUserVcardVisibility
{
const Public_ = 0;
const Internal = 1;
const Confidential = 2;
const Private_ = 3;
}
class UnbUserKey
{
const System = 1;
const Administrator = 2;
const Moderator = 3;
const Trusted = 4;
const DefaultUser = 5;
}
class UnbUser extends UnbEntity
{
const KeyLength = 5;
const RequireSecurePassword = false;
private $userId = 0;
private $createdTime = '';
private $lastLoginTime = '';
private $displayName = '';
private $loginName = '';
private $loginPassword = '';
private $additionalKeys = null;
private $db;
private function __construct()
{
$this->db = UnbDatabase::GetInstance();
}
public static function GetNonListKeys()
{
return array(UnbUserKey::System, UnbUserKey::DefaultUser);
}
public static function Create($displayName = null, $loginName = null, $loginPassword = null, $newId = null, $copyDefaultSettings = false)
{
if (isset($newId))
{
if (!is_int($newId))
throw new InvalidArgumentException('Invalid argument type: $newId');
if ($newId <= 0)
throw new InvalidArgumentException('Argument out of range: $newId');
}
else
{
if (self::KeyLength < 2 || self::KeyLength > 9)
throw new InvalidArgumentException('Argument out of range: KeyLength');
}
if (isset($displayName))
{
if (!is_string($displayName))
throw new InvalidArgumentException('Invalid argument type: $displayName');
$displayName = trim($displayName);
if (mb_strlen($displayName) > 255)
throw new LengthException('[#16] Display name is too long');
if (!strlen($displayName))
$displayName = null;
}
if (isset($loginName))
{
if (!is_string($loginName))
throw new InvalidArgumentException('Invalid argument type: $loginName');
$loginName = mb_strtolower(trim($loginName));
if (mb_strlen($loginName) > 255)
throw new LengthException('[#17] Login name is too long');
if (!strlen($loginName))
$loginName = null;
}
if (isset($loginPassword))
{
if (!is_string($loginPassword))
throw new InvalidArgumentException('Invalid argument type: $loginPassword');
$loginPassword = trim($loginPassword);
if (!strlen($loginPassword))
$loginPassword = null;
}
if (isset($loginName) ^ isset($loginPassword))
throw new InvalidArgumentException('[#18] Either both login name and password or none at all must be set.');
if (self::RequireSecurePassword && isset($loginPassword) && !self::IsSecurePassword($loginPassword, $loginName, $err))
throw new UnbSecurityException('[#19] Login password is not considered secure. ' . $err);
$now = time();
$db = UnbDatabase::GetInstance();
$db->StartTransaction(true);
try
{
if (isset($displayName))
{
$val = $db->QueryValue(
'SELECT COUNT(*)
FROM "#user"
WHERE LOWER("DisplayName") = LOWER(?) FOR UPDATE',
$displayName);
if (intval($val) > 0)
{
throw new UnbSecurityException('[#20] Display name is already assigned.');
}
}
if (isset($loginName))
{
$val = $db->QueryValue(
'SELECT COUNT(*)
FROM "#user"
WHERE LOWER("LoginName") = LOWER(?) FOR UPDATE',
$loginName);
if (intval($val) > 0)
{
throw new UnbSecurityException('[#21] Login name is already assigned.');
}
}
if (!isset($newId))
{
$ps = $db->Prepare('SELECT COUNT(*) FROM "#user" WHERE "UserId" = ? FOR UPDATE');
$retries = 20;
$minId = pow(10, self::KeyLength - 1);
$maxId = pow(10, self::KeyLength) - 1;
do
{
$newId = mt_rand($minId, $maxId);
if ($retries == 1)
{
$newId = $minId;
}
$count = $db->QueryValue($ps, $newId);
if ($count > 0 && $retries <= 3)
{
$newId = $db->QueryValue(
'SELECT MIN(UserId + 1) AS FreeId
FROM "#user" u1
WHERE NOT EXISTS (SELECT * FROM "#user" u2 WHERE u2."UserId" = u1."UserId" + 1)
AND "UserId" >= ?',
$newId);
if ($newId <= $maxId)
{
$count = 0;
}
}
$retries--;
}
while ($count > 0 && $retries > 0);
if ($count > 0)
{
throw new Exception('[#22] No free user ID available. Consider raising the key length.');
}
}
$fields = '';
$values = '';
$parameters = array(
$newId,
UnbDatabase::ConvertDateTime($now));
if (isset($displayName))
{
$fields .= ', "DisplayName"';
$values .= ', ?';
$parameters[] = $displayName;
}
if (isset($loginName))
{
$fields .= ', "LoginName"';
$values .= ', ?';
$parameters[] = $loginName;
}
if (isset($loginPassword))
{
$loginPassword = UnbSession::ConvertPassword($loginPassword);
$fields .= ', "LoginPassword"';
$values .= ', ?';
$parameters[] = $loginPassword;
}
$sql = 'INSERT INTO "#user" ("UserId", "CreatedTime"' . $fields . ') VALUES (?, ?' . $values . ')';
$db->Execute($sql, $parameters);
$user = new self();
$user->userId = $newId;
$user->createdTime = $now;
$user->displayName = $displayName;
$user->loginName = $loginName;
$user->loginPassword = $loginPassword;
if ($copyDefaultSettings)
{
$defaultUser = self::FindByUserId(UnbUserKey::DefaultUser);
$defaultSettings = $defaultUser->GetSettings();
$user->SetSetting($defaultSettings);
}
$db->Commit();
return $user;
}
catch (Exception $ex)
{
$db->RollBack();
throw $ex;
}
}
protected static function FromRecord($rec)
{
if (!is_array($rec))
throw new InvalidArgumentException('Invalid argument type: $rec');
if (!count($rec))
throw new InvalidArgumentException('Record array is empty');
$user = new self();
$user->userId = intval($rec['UserId']);
$user->displayName = $rec['DisplayName'];
$user->createdTime = UnbDatabase::ConvertDateTime($rec['CreatedTime']);
$user->lastLoginTime = UnbDatabase::ConvertDateTime($rec['LastLoginTime']);
$user->loginName = $rec['LoginName'];
$user->loginPassword = $rec['LoginPassword'];
return $user;
}
public static function FindByUserId($userId)
{
if (!is_int($userId))
throw new InvalidArgumentException('Invalid argument type: $userId');
if ($userId <= 0)
throw new InvalidArgumentException('Argument out of range: $userId');
$db = UnbDatabase::GetInstance();
$rec = $db->QueryRecord('SELECT * FROM "#user" WHERE "UserId" = ?', $userId);
if (!$rec) return null;
return self::FromRecord($rec);
}
public static function FindByDisplayName($displayName)
{
if (!is_string($displayName))
throw new InvalidArgumentException('Invalid argument type: $displayName');
$displayName = trim($displayName);
if (!strlen($displayName))
throw new InvalidArgumentException('Display name is empty');
if (mb_strlen($displayName) > 255)
throw new LengthException('[#16] Display name is too long');
$db = UnbDatabase::GetInstance();
$rec = $db->QueryRecord(
'SELECT * FROM "#user" WHERE LOWER("DisplayName") = LOWER(?)',
$displayName);
if (!$rec) return null;
return self::FromRecord($rec);
}
public static function FindByLoginName($loginName)
{
if (!is_string($loginName))
throw new InvalidArgumentException('Invalid argument type: $loginName');
$loginName = mb_strtolower(trim($loginName));
if (!strlen($loginName))
throw new InvalidArgumentException('Login name is empty');
if (mb_strlen($loginName) > 255)
throw new LengthException('[#17] Login name is too long');
$db = UnbDatabase::GetInstance();
$rec = $db->QueryRecord(
'SELECT * FROM "#user" WHERE LOWER("LoginName") = LOWER(?)',
$loginName);
if (!$rec) return null;
return self::FromRecord($rec);
}
public static function Find()
{
$db = UnbDatabase::GetInstance();
$recs = $db->Query('SELECT * FROM "#user" WHERE "UserId" > 9 ORDER BY "DisplayName"');
if (!isset($recs)) return null;
$users = array();
foreach ($recs as $rec)
{
$users[] = self::FromRecord($rec);
}
return $users;
}
public static function IsSecurePassword($password, $data, &$err)
{
$err = '';
if (mb_strlen($password) < 6)
{
$err = 'Require at least 6 characters';
return false;
}
if (mb_strtolower($password) == mb_strtolower($data))
{
$err = 'Reject password caseless equal to username';
return false;
}
if ($data != '' && mb_strpos(mb_strtolower($password), mb_strtolower($data)) !== false)
{
$err = 'Reject password caseless containing username';
return false;
}
if (preg_match('/[^!-~]/', $password))
{
$err = 'Reject non-ASCII characters';
return false;
}
if (!(preg_match('/[a-z]/', $password) && preg_match('/[0-9]/', $password) ||
preg_match('/[a-z]/', $password) && preg_match('/[A-Z]/', $password) ||
preg_match('/[A-Z]/', $password) && preg_match('/[0-9]/', $password) ||
preg_match('/[0-9]/', $password) && preg_match('/[!#$%&()*+,\-.\/:;<=>?@\[\\\\\]^_|~]/', $password) ||
preg_match('/[a-z]/', $password) && preg_match('/[!#$%&()*+,\-.\/:;<=>?@\[\\\\\]^_|~]/', $password) ||
preg_match('/[A-Z]/', $password) && preg_match('/[!#$%&()*+,\-.\/:;<=>?@\[\\\\\]^_|~]/', $password)))
{
$err = 'Require at least 2 character classes out of numbers, lowercase, uppercase characters and other symbols';
return false;
}
if (preg_match('/(.)\1\1/', $password))
{
$err = 'Reject the same character 3 times or more';
return false;
}
for ($n = 0; $n <= 7; $n++)
if (preg_match('/' . $n . ($n + 1) . ($n + 2) . '/', $password))
{
$err = 'Reject simple sequences: 012, 123, ...';
return false;
}
for ($n = 0x41; $n <= 0x58; $n++)
if (preg_match('/' . chr($n) . chr($n + 1) . chr($n + 2) . '/', $password))
{
$err = 'Reject simple sequences: ABC, BCD, ...';
return false;
}
for ($n = 0x61; $n <= 0x78; $n++)
if (preg_match('/' . chr($n) . chr($n + 1) . chr($n + 2) . '/', $password))
{
$err = 'Reject simple sequences: abc, bcd, ...';
return false;
}
if (strlen(count_chars($password, 3)) < 4)
{
$err = 'Require at least 4 different characters to be used';
return false;
}
return true;
}
public function GetUserId()
{
return $this->userId;
}
public function GetCreatedTime()
{
return $this->createdTime;
}
public function GetLastLoginTime()
{
return $this->lastLoginTime;
}
public function UpdateLastLoginTime()
{
if (UnbSession::GetSessionUserId() != $this->GetUserId())
throw new UnbSecurityException('You must be logged in as this user to set the last login time.');
$now = time();
$this->db->StartTransaction();
try
{
$this->db->Execute(
'UPDATE "#user" SET "LastLoginTime" = ? WHERE "UserId" = ?',
array(UnbDatabase::ConvertDateTime($now), $this->userId));
$this->db->Commit();
}
catch (Exception $ex)
{
$this->db->RollBack();
throw $ex;
}
$this->lastLoginTime = $now;
}
public function GetDisplayName()
{
return $this->displayName;
}
public function SetDisplayName($displayName)
{
if (!is_string($displayName))
throw new InvalidArgumentException('Invalid argument type: $displayName');
$displayName = trim($displayName);
if (!strlen($displayName))
throw new InvalidArgumentException('Display name is empty');
if (mb_strlen($displayName) > 255)
throw new LengthException('[#16] Display name is too long');
if (UnbSession::GetSessionUserId() != $this->GetUserId())
throw new UnbSecurityException('You must be logged in as this user to set the display name.');
$this->db->StartTransaction(true);
try
{
$val = $this->db->QueryValue(
'SELECT COUNT(*) FROM "#user" WHERE LOWER("DisplayName") = LOWER(?) FOR UPDATE',
$displayName);
if (intval($val) > 0)
{
throw new UnbSecurityException('[#20] Display name is already assigned.');
}
$this->db->Execute(
'UPDATE "#user" SET "DisplayName" = ? WHERE "UserId" = ?',
array($displayName, $this->userId));
$this->db->Commit();
}
catch (Exception $ex)
{
$this->db->RollBack();
throw $ex;
}
$this->displayName = $displayName;
}
public function GetLoginName()
{
if (!UnbSession::IsAdministrator() && UnbSession::GetSessionUserId() != $this->userId)
throw new UnbSecurityException('You must be logged in as this user or need the administrator privilege to get a login name.');
return $this->loginName;
}
public function SetLoginName($loginName)
{
if (isset($loginName))
{
if (!is_string($loginName))
throw new InvalidArgumentException('Invalid argument type: $loginName');
$loginName = mb_strtolower(trim($loginName));
if (mb_strlen($loginName) > 255)
throw new LengthException('[#17] Login name is too long');
if (!strlen($loginName))
$loginName = null;
}
if (!UnbSession::IsAdministrator() && UnbSession::GetSessionUserId() != $this->userId)
throw new UnbSecurityException('You must be logged in as this user or need the administrator privilege to set a login name.');
$this->db->StartTransaction();
try
{
$this->db->Execute(
'UPDATE "#user" SET "LoginName" = ? WHERE "UserId" = ?',
array($loginName, $this->userId));
$this->db->Commit();
}
catch (Exception $ex)
{
$this->db->RollBack();
if ($this->db->IsConstraintError())
throw new UnbSecurityException('[#21] Login name is already assigned.');
throw $ex;
}
$this->loginName = $loginName;
}
public function GetLoginPassword()
{
return $this->loginPassword;
}
public function SetLoginPassword($loginPassword)
{
if (isset($loginPassword))
{
if (!is_string($loginPassword))
throw new InvalidArgumentException('Invalid argument type: $loginPassword');
$loginPassword = trim($loginPassword);
if (!strlen($loginPassword))
$loginPassword = null;
}
if (isset($loginPassword))
{
if (self::RequireSecurePassword && !self::IsSecurePassword($loginPassword, $this->GetLoginName(), $err))
throw new UnbSecurityException('[#19] Login password is not considered secure. ' . $err);
$loginPassword = UnbSession::ConvertPassword($loginPassword);
}
if (!UnbSession::IsAdministrator() && UnbSession::GetSessionUserId() != $this->userId)
throw new UnbSecurityException('You must be logged in as this user or need the administrator privilege to set a login password.');
$this->db->StartTransaction();
try
{
$this->db->Execute(
'UPDATE "#user" SET "LoginPassword" = ? WHERE "UserId" = ?',
array($loginPassword, $this->userId));
$this->db->Commit();
}
catch (Exception $ex)
{
$this->db->RollBack();
throw $ex;
}
$this->loginPassword = $loginPassword;
}
public function IsAdministrator()
{
if ($this->userId == UnbUserKey::Administrator) return true;
$keys = $this->additionalKeys;
if (!isset($keys))
$keys = $this->GetAdditionalKeys();
return in_array(UnbUserKey::Administrator, $keys);
}
public function IsModerator()
{
if ($this->userId == UnbUserKey::Moderator) return true;
$keys = $this->additionalKeys;
if (!isset($keys))
$keys = $this->GetAdditionalKeys();
return in_array(UnbUserKey::Moderator, $keys);
}
public function IsTrusted()
{
if ($this->userId == UnbUserKey::Trusted) return true;
$keys = $this->additionalKeys;
if (!isset($keys))
$keys = $this->GetAdditionalKeys();
return in_array(UnbUserKey::Trusted, $keys);
}
public function GetAdditionalKeys()
{
$keys = $this->db->QueryArray(
'SELECT k."UserId"
FROM "#user" u, "#keylist" k
WHERE u."AdditionalKeylistId" = k."KeylistId" AND u."UserId" = ?
ORDER BY k."UserId"',
$this->userId);
$keys = array_map('intval', $keys);
$this->additionalKeys = $keys;
return $keys;
}
public function AddAdditionalKey($key)
{
if (!UnbSession::IsAdministrator())
throw new UnbSecurityException('You need the administrator privilege to add additional user keys.');
$this->additionalKeys = null;
$this->db->AddKeylistKey('#user', 'AdditionalKeylistId', 'UserId', $this->userId, $key);
}
public function RemoveAdditionalKey($key)
{
if (!UnbSession::IsAdministrator())
throw new UnbSecurityException('You need the administrator privilege to remove additional user keys.');
$this->additionalKeys = null;
$this->db->RemoveKeylistKey('#user', 'AdditionalKeylistId', 'UserId', $this->userId, $key);
}
public function SetAdditionalKeys($keys)
{
if (!UnbSession::IsAdministrator())
throw new UnbSecurityException('You need the administrator privilege to set additional keys.');
$this->additionalKeys = null;
$this->db->ResetKeylistKeys('#user', 'AdditionalKeylistId', 'UserId', $this->userId, $keys);
}
public function Delete()
{
if (!UnbSession::IsAdministrator())
throw new UnbSecurityException('You need the administrator privilege to delete a user.');
$this->additionalKeys = null;
$this->db->StartTransaction();
try
{
$keylistIds = $this->db->QueryArray(
'SELECT "KeylistId"
FROM "#keylist" k1
WHERE
-- KeylistId is referenced as an access-restricting key list
("KeylistId" IN (SELECT "ReadAccessKeylistId" FROM "#message") OR
"KeylistId" IN (SELECT "AlterAccessKeylistId" FROM "#message") OR
"KeylistId" IN (SELECT "ReplyAccessKeylistId" FROM "#message") OR
"KeylistId" IN (SELECT "UseAccessKeylistId" FROM "#tag") OR
"KeylistId" IN (SELECT "ReadAccessKeylistId" FROM "#tag")) AND
-- The key list does not contain any other UserId than that of the to-be-deleted user
NOT EXISTS (
SELECT *
FROM "#keylist" k2
WHERE k2."KeylistId" = k1."KeylistId" AND
k2."UserId" <> ?)',
$this->userId);
$keylistIds = array_map('intval', $keylistIds);
if (count($keylistIds))
{
$this->db->Execute(
'UPDATE "#keylist" k1
SET "UserId" = ?
WHERE "KeylistId" IN (' . join(', ', $keylistIds) . ')',
UnbUserKey::System);
}
$this->db->Execute('DELETE FROM "#user" WHERE "UserId" = ?', $this->userId);
$this->db->Execute(
'UPDATE "#user"
SET "AdditionalKeylistId" = NULL
WHERE "AdditionalKeylistId" IS NOT NULL AND
"AdditionalKeylistId" NOT IN (SELECT "KeylistId" FROM "#keylist")');
$this->db->Commit();
}
catch (Exception $ex)
{
$this->db->RollBack();
throw $ex;
}
}
public function GetVcardEntries($types = null)
{
if (isset($types))
{
if (is_string($types))
$types = array($types);
if (!is_array($types))
throw new InvalidArgumentException('Invalid argument type: $types');
if (!count($types))
throw new InvalidArgumentException('Types array is empty');
}
if (isset($visibility))
{
if (!is_int($visibility))
throw new InvalidArgumentException('Invalid argument type: $visibility');
if ($visibility < 0)
throw new InvalidArgumentException('Argument out of range: $visibility');
}
if (UnbSession::GetSessionUserId() == $this->userId)
$visibility = UnbUserVcardVisibility::Private_;
elseif (UnbSession::IsTrusted() || UnbSession::IsAdministrator())
$visibility = UnbUserVcardVisibility::Confidential;
elseif (UnbSession::IsLoggedIn())
$visibility = UnbUserVcardVisibility::Internal;
else
$visibility = UnbUserVcardVisibility::Public_;
$sql = 'SELECT uv."Type", uv."Data", uv."Visibility"
FROM "#user" u
JOIN "#user_vcard" uv USING ("UserId")
WHERE u."UserId" = ?';
$parameters = array($this->userId);
if (isset($types))
{
$sql .= ' AND (';
for ($i = 0; $i < count($types); $i++)
{
if (mb_strlen($types[$i]) > 255)
throw new LengthException('Type is too long');
$sql .= ($i ? ' OR' : '') . ' uv."Type" = ?';
$parameters[] = $types[$i];
}
$sql .= ')';
}
if (isset($visibility))
{
$sql .= ' AND uv."Visibility" <= ?';
$parameters[] = $visibility;
}
$recs = $this->db->Query($sql, $parameters);
$entries = array();
if (is_array($recs))
{
foreach ($recs as $rec)
{
$entries[$rec['Type']] = array('Data' => $rec['Data'], 'Visibility' => $rec['Visibility']);
}
}
return $entries;
}
public function SetVcardEntry($type, $data, $visibility = null)
{
if (!is_string($type))
throw new InvalidArgumentException('Invalid argument type: $type');
$type = trim($type);
if (!strlen($type))
throw new InvalidArgumentException('Type is empty');
if (mb_strlen($type) > 255)
throw new LengthException('[#23] Type is too long');
if (!is_string($data))
throw new InvalidArgumentException('Invalid argument type: $data');
$data = trim($data);
if (mb_strlen($data) > 65535)
throw new LengthException('[#24] Data is too long');
if (isset($visibility))
{
if (!is_int($visibility))
throw new InvalidArgumentException('Invalid argument type: $visibility');
if ($visibility < 0)
throw new InvalidArgumentException('Argument out of range: $visibility');
}
if (!UnbSession::IsAdministrator() && UnbSession::GetSessionUserId() != $this->userId)
throw new UnbSecurityException('You must be logged in as this user or need the administrator privilege to set a vcard entry.');
$this->db->StartTransaction();
try
{
if (isset($visibility))
{
$this->db->Execute(
'UPDATE "#user_vcard" SET "Data" = ?, "Visibility" = ? WHERE "UserId" = ? AND "Type" = ?',
array($data, $visibility, $this->userId, $type));
}
else
{
$this->db->Execute(
'UPDATE "#user_vcard" SET "Data" = ? WHERE "UserId" = ? AND "Type" = ?',
array($data, $this->userId, $type));
}
if ($this->db->GetRowsAffected() == 0)
{
try
{
if (!isset($visibility)) $visibility = UnbUserVcardVisibility::Public_;
$this->db->Execute(
'INSERT INTO "#user_vcard" ("UserId", "Type", "Data", "Visibility") VALUES (?, ?, ?, ?)',
array($this->userId, $type, $data, $visibility));
}
catch (UnbDatabaseException $ex)
{
if (!$this->db->IsConstraintError()) throw $ex;
}
}
$this->db->Commit();
}
catch (Exception $ex)
{
$this->db->RollBack();
throw $ex;
}
}
public function RemoveVcardEntry($type)
{
if (!is_string($type))
throw new InvalidArgumentException('Invalid argument type: $type');
$type = trim($type);
if (!strlen($type))
throw new InvalidArgumentException('Type is empty');
if (mb_strlen($type) > 255)
throw new LengthException('[#23] Type is too long');
if (!UnbSession::IsAdministrator() && UnbSession::GetSessionUserId() != $this->userId)
throw new UnbSecurityException('You must be logged in as this user or need the administrator privilege to remove a vcard entry.');
$this->db->StartTransaction();
try
{
$this->db->Execute(
'DELETE FROM "#user_vcard" WHERE "UserId" = ? AND "Type" = ?',
array($this->userId, $type));
$this->db->Commit();
}
catch (Exception $ex)
{
$this->db->RollBack();
throw $ex;
}
}
public function GetSettings($names = null)
{
if (isset($names))
{
if (is_string($names))
$names = array($names);
if (!is_array($names))
throw new InvalidArgumentException('Invalid argument type: $names');
if (!count($names))
throw new InvalidArgumentException('Names array is empty');
}
$sql = 'SELECT "Name", "Data" FROM "#user_config" WHERE "UserId" = ?';
$parameters = array($this->userId);
if (isset($names))
{
$sql .= ' AND (';
for ($i = 0; $i < count($names); $i++)
{
if (mb_strlen($names[$i]) > 255)
throw new LengthException('[#25] Name is too long');
if ($i)
$sql .= ' OR ';
if (strpos($names[$i], '*') === false)
{
$sql .= 'LOWER("Name") = LOWER(?)';
$parameters[] = $names[$i];
}
else
{
$sql .= 'LOWER("Name") LIKE LOWER(?)';
$parameters[] = str_replace('*', '%', $names[$i]);
}
}
$sql .= ')';
}
$recs = $this->db->Query($sql, $parameters);
if (!isset($recs)) return null;
$entries = array();
foreach ($recs as $rec)
{
$entries[$rec['Name']] = $rec['Data'];
}
return $entries;
}
public function GetSingleSetting($name)
{
if (!is_string($name))
throw new InvalidArgumentException('Invalid argument type: $name');
$name = trim($name);
if (!strlen($name))
throw new InvalidArgumentException('Name is empty');
$a = $this->GetSettings($name);
return $a[$name];
}
public function SetSetting($name, $data)
{
$pairs = array();
if (is_array($name))
$pairs = $name;
elseif (is_string($name) && (is_string($data) || !isset($data)))
$pairs = array($name => $data);
else
throw new InvalidArgumentException('Invalid argument type.');
$newPairs = array();
foreach ($pairs as $name => $data)
{
if (!is_string($name) ||
!(is_string($data) || !isset($data)))
throw new InvalidArgumentException('Invalid argument types.');
$name = trim($name);
if (!strlen($name))
throw new InvalidArgumentException('Name is empty');
if (mb_strlen($name) > 255)
throw new LengthException('[#25] Name is too long');
$data = trim($data);
if (mb_strlen($data) > 255)
throw new LengthException('[#26] Data is too long');
$newPairs[$name] = $data;
}
$pairs = $newPairs;
$this->db->StartTransaction();
try
{
foreach ($pairs as $name => $data)
{
$this->db->Execute(
'UPDATE "#user_config" SET "Data" = ? WHERE "UserId" = ? AND LOWER("Name") = LOWER(?)',
array($data, $this->userId, $name));
if ($this->db->GetRowsAffected() == 0)
{
$this->db->Execute(
'INSERT INTO "#user_config" ("UserId", "Name", "Data") VALUES (?, LOWER(?), ?)',
array($this->userId, $name, $data));
}
}
$this->db->Commit();
}
catch (Exception $ex)
{
$this->db->RollBack();
throw $ex;
}
}
public function RemoveSetting($name)
{
if (!is_string($name))
throw new InvalidArgumentException('Invalid argument type: $name');
$name = trim($name);
if (!strlen($name))
throw new InvalidArgumentException('Name is empty');
if (mb_strlen($name) > 255)
throw new LengthException('[#25] Name is too long');
$this->db->StartTransaction();
try
{
$this->db->Execute(
'DELETE FROM "#user_config" WHERE "UserId" = ? AND "Name" = ?',
array($this->userId, $name));
$this->db->Commit();
}
catch (Exception $ex)
{
$this->db->RollBack();
throw $ex;
}
}
public function GetMessagesRating()
{
$rec = $this->db->QueryRecord(
'SELECT COUNT(*) AS "Count", SUM(mr."Weight") AS "TotalWeight", SUM(mr."Rating" * mr."Weight") AS "WeightedSum"
FROM "#message_rating" mr
JOIN "#message" m USING ("MessageId")
WHERE m."Owner" = ?',
$this->userId);
if (!$rec) return null;
$totalWeight = max(100.0, floatval($rec['TotalWeight']));
return array(
intval($rec['Count']),
intval($rec['TotalWeight']),
floatval($rec['WeightedSum']) / $totalWeight);
}
}
class UnbTagFlag
{
const NotSelectable = 1;
const Invisible = 2;
const EnforceApproval = 4;
const UserData = 8;
}
class UnbTag extends UnbEntity
{
private $tagId = 0;
private $tagName = '';
private $parentTag = 0;
private $meaning = 0;
private $description = '';
private $flags = 0;
private $tagData = null;
private $db;
private static $tagHierarchicalOrder = null;
private function __construct()
{
$this->db = UnbDatabase::GetInstance();
}
public static function Create($tagName)
{
if (!is_string($tagName))
throw new InvalidArgumentException('Invalid argument type: $tagName');
$tagName = trim($tagName);
if (!strlen($tagName))
throw new InvalidArgumentException('Tag name is empty');
if (mb_strlen($tagName) > 255)
throw new LengthException('[#14] Tag name is too long');
if (!UnbSession::IsModerator())
throw new UnbSecurityException('You need the moderator privilege to create a tag.');
$db = UnbDatabase::GetInstance();
$db->StartTransaction(true);
try
{
$tagId = $db->QueryValue('SELECT MAX("TagId") FROM "#tag" FOR UPDATE');
if (!isset($tagId))
$tagId = 0;
$tagId++;
$db->Execute(
'INSERT INTO "#tag" ("TagId", "TagName") VALUES (?, ?)',
array($tagId, $tagName));
$db->Commit();
}
catch (Exception $ex)
{
$db->RollBack();
throw $ex;
}
$tag = new self();
$tag->tagId = $tagId;
$tag->tagName = $tagName;
return $tag;
}
protected static function FromRecord($rec)
{
if (!is_array($rec))
throw new InvalidArgumentException('Invalid argument type: $rec');
if (!count($rec))
throw new InvalidArgumentException('Record array is empty');
$tag = new self();
$tag->tagId = intval($rec['TagId']);
$tag->tagName = $rec['TagName'];
$tag->parentTag = intval($rec['ParentTag']);
$tag->meaning = intval($rec['Meaning']);
$tag->description = $rec['Description'];
$tag->flags = intval($rec['Flags']);
$tag->tagData = $rec['TagData'];
return $tag;
}
public static function FindByTagId($tagId)
{
if (!is_int($tagId) && !is_array($tagId))
throw new InvalidArgumentException('Invalid argument type: $tagId');
if (is_int($tagId) && $tagId <= 0)
throw new InvalidArgumentException('Argument out of range: $tagId');
if (is_array($tagId) && !count($tagId))
throw new InvalidArgumentException('Argument out of range: $tagId');
$db = UnbDatabase::GetInstance();
if (is_int($tagId))
{
$rec = $db->QueryRecord('SELECT * FROM "#tag" WHERE "TagId" = ?', $tagId);
if (!$rec) return null;
return self::FromRecord($rec);
}
if (is_array($tagId))
{
$tagId = array_map('intval', $tagId);
$rec = $db->Query('SELECT * FROM "#tag" WHERE "TagId" IN (' . join(', ', $tagId) . ')');
if (!$rec) return null;
$a = array();
foreach ($rec as $r)
{
$a[] = self::FromRecord($r);
}
return $a;
}
}
public static function FindByTagName($tagName, $caseless = false, $getAll = false)
{
if (!is_string($tagName))
throw new InvalidArgumentException('Invalid argument type: $tagName');
$tagName = trim($tagName);
if (!strlen($tagName))
throw new InvalidArgumentException('Tag name is empty');
if (mb_strlen($tagName) > 255)
throw new LengthException('[#14] Tag name is too long');
$db = UnbDatabase::GetInstance();
if ($getAll)
{
if ($caseless)
$recs = $db->Query('SELECT * FROM "#tag" WHERE LOWER("TagName") = LOWER(?)', $tagName);
else
$recs = $db->Query('SELECT * FROM "#tag" WHERE "TagName" = ?', $tagName);
if (!$recs) return null;
$tags = array();
foreach ($recs as $rec)
$tags[] = self::FromRecord($rec);
return $tags;
}
else
{
if ($caseless)
$rec = $db->QueryRecord('SELECT * FROM "#tag" WHERE LOWER("TagName") = LOWER(?)', $tagName);
else
$rec = $db->QueryRecord('SELECT * FROM "#tag" WHERE "TagName" = ?', $tagName);
if (!$rec) return null;
return self::FromRecord($rec);
}
}
public static function FindByMeaning($meaning)
{
if (!is_int($meaning))
throw new InvalidArgumentException('Invalid argument type: $meaning');
$tag = array();
$db = UnbDatabase::GetInstance();
$recs = $db->Query('SELECT * FROM "#tag" WHERE "Meaning" = ?', $meaning);
if (!$recs) return $tags;
foreach ($recs as $rec)
$tags[] = self::FromRecord($rec);
return $tags;
}
public static function Find($withReadAccess = false, $withUseAccess = false)
{
$sessionUserId = UnbSession::GetSessionUserId();
if (!isset(self::$tagHierarchicalOrder))
self::UpdateHierarchicalOrder();
$sql = 'SELECT * FROM "#tag" t';
$where = '';
if ($withReadAccess)
{
$where .= ($where ? ' AND ' : ' WHERE ') . ' (
-- Tag has no read access keylist
t."ReadAccessKeylistId" IS NULL
OR EXISTS
-- Tag\'s read access keylist contains personal or additional user key
(SELECT 1
FROM "#keylist" tk
WHERE tk."KeylistId" = t."ReadAccessKeylistId" AND
(tk."UserId" = ? OR
tk."UserId" IN
(SELECT uk."UserId"
FROM "#user" u, "#keylist" uk
WHERE uk."KeylistId" = u."AdditionalKeylistId" AND
u."UserId" = ?))))';
$parameters = array($sessionUserId, $sessionUserId);
}
if ($withUseAccess)
{
$where .= ($where ? ' AND ' : ' WHERE ') . ' (
-- Tag has no use access keylist
t."UseAccessKeylistId" IS NULL
OR EXISTS
-- Tag\'s use access keylist contains personal or additional user key
(SELECT 1
FROM "#keylist" tk
WHERE tk."KeylistId" = t."UseAccessKeylistId" AND
(tk."UserId" = ? OR
tk."UserId" IN
(SELECT uk."UserId"
FROM "#user" u, "#keylist" uk
WHERE uk."KeylistId" = u."AdditionalKeylistId" AND
u."UserId" = ?))))';
$parameters = array($sessionUserId, $sessionUserId);
}
$sql .= $where;
if (count(self::$tagHierarchicalOrder))
{
$sql .= ' ORDER BY CASE "TagId"';
foreach (self::$tagHierarchicalOrder as $index => $tagId)
$sql .= ' WHEN ' . $tagId . ' THEN ' . $index;
$sql .= ' END';
}
$db = UnbDatabase::GetInstance();
$recs = $db->Query($sql, $parameters);
if (!isset($recs)) return null;
$tags = array();
foreach ($recs as $rec)
$tags[] = self::FromRecord($rec);
return $tags;
}
private static function UpdateHierarchicalOrder()
{
self::$tagHierarchicalOrder = array();
$db = UnbDatabase::GetInstance();
$recs = $db->Query('SELECT * FROM "#tag" ORDER BY "TagName"');
if (!$recs) return;
$tags = array();
foreach ($recs as $rec)
$tags[] = self::FromRecord($rec);
self::UpdateHierarchicalOrder_Rec($tags, 0, self::$tagHierarchicalOrder);
}
private static function UpdateHierarchicalOrder_Rec($tags, $parentId, &$order)
{
foreach ($tags as $tag)
{
if ($tag->GetParentTagId() == $parentId)
{
$order[] = $tag->GetTagId();
self::UpdateHierarchicalOrder_Rec($tags, $tag->GetTagId(), $order);
}
}
}
public static function GetHierarchicalOrder()
{
if (!isset(self::$tagHierarchicalOrder))
self::UpdateHierarchicalOrder();
return self::$tagHierarchicalOrder;
}
public function GetTagId()
{
return $this->tagId;
}
public function GetTagName()
{
return $this->tagName;
}
public function SetTagName($tagName)
{
if (!is_string($tagName))
throw new InvalidArgumentException('Invalid argument type: $tagName');
$tagName = trim($tagName);
if (!strlen($tagName))
throw new InvalidArgumentException('Tag name is empty');
if (mb_strlen($tagName) > 255)
throw new LengthException('[#14] Tag name is too long');
if (!UnbSession::IsModerator())
throw new UnbSecurityException('You need the moderator privilege to set a tag name.');
$this->db->StartTransaction();
try
{
$this->db->Execute(
'UPDATE "#tag" SET "TagName" = ? WHERE "TagId" = ?',
array($tagName, $this->tagId));
$this->db->Commit();
}
catch (Exception $ex)
{
$db->RollBack();
throw $ex;
}
$this->tagName = $tagName;
}
public function GetParentTag()
{
if ($this->parentTag == 0) return null;
return self::FindByTagId($this->parentTag);
}
public function GetParentTagId()
{
return $this->parentTag;
}
public function SetParentTag($parentTag)
{
if (isset($parentTag))
{
if ($parentTag instanceof UnbTag)
$parentTag = $parentTag->GetTagId();
if (!is_int($parentTag))
throw new InvalidArgumentException('Invalid argument type: $parentTag');
if ($parentTag < 0)
throw new InvalidArgumentException('Argument out of range: $parentTag');
if ($parentTag == 0)
$parentTag = null;
}
if (!UnbSession::IsModerator())
throw new UnbSecurityException('You need the moderator privilege to set a parent tag.');
$this->db->StartTransaction();
try
{
$this->db->Execute(
'UPDATE "#tag" SET "ParentTag" = ? WHERE "TagId" = ?',
array($parentTag, $this->tagId));
$this->db->Commit();
}
catch (Exception $ex)
{
$db->RollBack();
throw $ex;
}
$this->parentTag = $parentTag;
}
public function GetMeaning()
{
return $this->meaning;
}
public function SetMeaning($meaning)
{
if (isset($meaning))
{
if (!is_int($meaning))
throw new InvalidArgumentException('Invalid argument type: $meaning');
}
if (!UnbSession::IsModerator())
throw new UnbSecurityException('You need the moderator privilege to set a tag meaning.');
$this->db->StartTransaction();
try
{
$this->db->Execute(
'UPDATE "#tag" SET "Meaning" = ? WHERE "TagId" = ?',
array($meaning, $this->tagId));
$this->db->Commit();
}
catch (Exception $ex)
{
$db->RollBack();
throw $ex;
}
$this->meaning = $meaning;
}
public function GetDescription()
{
return $this->description;
}
public function SetDescription($description)
{
if (isset($description))
{
if (!is_string($description))
throw new InvalidArgumentException('Invalid argument type: $description');
$description = trim($description);
if (mb_strlen($description) > 255)
throw new LengthException('[#15] Description is too long');
if (!strlen($description))
$description = null;
}
if (!UnbSession::IsModerator())
throw new UnbSecurityException('You need the moderator privilege to set a tag description.');
$this->db->StartTransaction();
try
{
$this->db->Execute(
'UPDATE "#tag" SET "Description" = ? WHERE "TagId" = ?',
array($description, $this->tagId));
$this->db->Commit();
}
catch (Exception $ex)
{
$db->RollBack();
throw $ex;
}
$this->description = $description;
}
public function GetFlags()
{
return $this->flags;
}
public function SetFlags($flags)
{
if (!is_int($flags))
throw new InvalidArgumentException('Invalid argument type: $flags');
if (!UnbSession::IsModerator())
throw new UnbSecurityException('You need the moderator privilege to set tag flags.');
$this->db->StartTransaction();
try
{
$this->db->Execute(
'UPDATE "#tag" SET "Flags" = ? WHERE "TagId" = ?',
array($flags, $this->tagId));
$this->db->Commit();
}
catch (Exception $ex)
{
$db->RollBack();
throw $ex;
}
$this->flags = $flags;
}
public function GetUseAccessKeys()
{
$keys = $this->db->QueryArray(
'SELECT k."UserId"
FROM "#tag" t, "#keylist" k
WHERE t."UseAccessKeylistId" = k."KeylistId" AND t."TagId" = ?',
$this->tagId);
$keys = array_map('intval', $keys);
return $keys;
}
public function AddUseAccessKey($key)
{
if (!UnbSession::IsAdministrator())
throw new UnbSecurityException('You need the administrator privilege to add use access keys.');
$this->db->AddKeylistKey('#tag', 'UseAccessKeylistId', 'TagId', $this->tagId, $key);
}
public function RemoveUseAccessKey($key)
{
if (!UnbSession::IsAdministrator())
throw new UnbSecurityException('You need the administrator privilege to remove use access keys.');
$this->db->RemoveKeylistKey('#tag', 'UseAccessKeylistId', 'TagId', $this->tagId, $key);
}
public function SetUseAccessKeys($keys)
{
if (!UnbSession::IsAdministrator())
throw new UnbSecurityException('You need the administrator privilege to set use access keys.');
$this->db->ResetKeylistKeys('#tag', 'UseAccessKeylistId', 'TagId', $this->tagId, $keys);
}
public function GetReadAccessKeys()
{
$keys = $this->db->QueryArray(
'SELECT k."UserId"
FROM "#tag" t, "#keylist" k
WHERE t."ReadAccessKeylistId" = k."KeylistId" AND t."TagId" = ?',
$this->tagId);
$keys = array_map('intval', $keys);
return $keys;
}
public function AddReadAccessKey($key)
{
if (!UnbSession::IsAdministrator())
throw new UnbSecurityException('You need the administrator privilege to add read access keys.');
$this->db->AddKeylistKey('#tag', 'ReadAccessKeylistId', 'TagId', $this->tagId, $key);
}
public function RemoveReadAccessKey($key)
{
if (!UnbSession::IsAdministrator())
throw new UnbSecurityException('You need the administrator privilege to remove read access keys.');
$this->db->RemoveKeylistKey('#tag', 'ReadAccessKeylistId', 'TagId', $this->tagId, $key);
}
public function SetReadAccessKeys($keys)
{
if (!UnbSession::IsAdministrator())
throw new UnbSecurityException('You need the administrator privilege to set read access keys.');
$this->db->ResetKeylistKeys('#tag', 'ReadAccessKeylistId', 'TagId', $this->tagId, $keys);
}
public function IsUseAllowed()
{
return self::IsUseAllowedById($this->tagId);
}
public static function IsUseAllowedById($tagId)
{
if ($tagId instanceof UnbTag)
$tagId = $tagId->GetTagId();
if (!is_int($tagId))
throw new InvalidArgumentException('Invalid argument type: $tagId');
$sessionUserId = UnbSession::GetSessionUserId();
$sql = 'SELECT
(EXISTS
-- Tag has no use access keylist
(SELECT 1
FROM "#tag" t
WHERE t."TagId" = ? AND
t."UseAccessKeylistId" IS NULL)
OR EXISTS
-- Tag\'s use access keylist contains personal or additional user key
(SELECT 1
FROM "#tag" t, "#keylist" tk
WHERE tk."KeylistId" = t."UseAccessKeylistId" AND
t."TagId" = ? AND
(tk."UserId" = ? OR
tk."UserId" IN
(SELECT uk."UserId"
FROM "#user" u, "#keylist" uk
WHERE uk."KeylistId" = u."AdditionalKeylistId" AND
u."UserId" = ?))))';
$parameters = array($tagId, $tagId, $sessionUserId, $sessionUserId);
$db = UnbDatabase::GetInstance();
$val = $db->QueryValue($sql, $parameters);
$val = $val ? true : false;
return $val;
}
public function GetTagData()
{
return $this->tagData;
}
public function Delete()
{
if (!UnbSession::IsModerator())
throw new UnbSecurityException('You need the moderator privilege to delete a tag.');
$this->db->StartTransaction();
try
{
$this->db->Execute('DELETE FROM "#tag" WHERE "TagId" = ?', $this->tagId);
$this->db->Commit();
}
catch (Exception $ex)
{
$this->db->RollBack();
throw $ex;
}
}
}
class UnbMessageModerationFlag
{
const Locked = 1;
const Hidden = 2;
const EnforceApproval = 4;
}
class UnbMessage extends UnbEntity
{
private $messageId = 0;
private $createdTime = 0;
private $ownerUserId = 0;
private $primaryReference = 0;
private $entryPoint = false;
private $pageName = '';
private $moderationFlags = 0;
private $lastBeginEdit = 0;
private $searchOptimisedContent = '';
private $searchOptimisedSubject = '';
private $searchRevision = 0;
private $ownerUser = null;
private $db;
private static $readAccessCache = array();
private static $alterAccessCache = array();
private static $replyAccessCache = array();
private function __construct()
{
$this->db = UnbDatabase::GetInstance();
}
public static function Create($subject, $content, $data = null, $dataFileName = null, $writeToFile = null, $entryPoint = false, $primaryRef = null)
{
$primaryRefId = 0;
if (isset($primaryRef))
{
$primaryRefId = $primaryRef;
if ($primaryRefId instanceof UnbMessage)
$primaryRefId = $primaryRefId->GetMessageId();
if (!is_int($primaryRefId))
throw new InvalidArgumentException('Invalid argument type: $primaryRef');
if ($primaryRefId <= 0)
throw new InvalidArgumentException('Argument out of range: $primaryRef');
}
if (!UnbSession::IsLoggedIn())
throw new UnbSecurityException('You must be logged in to create a message.');
$entryPoint = $entryPoint ? true : false;
$now = time();
$db = UnbDatabase::GetInstance();
$db->StartTransaction(true);
try
{
$messageId = $db->QueryValue('SELECT MAX("MessageId") FROM "#message" FOR UPDATE');
if (!isset($messageId))
$messageId = 0;
$messageId++;
$sqlCols = '"MessageId", "CreatedTime", "Owner", "EntryPoint"';
$sqlValues = '?, ?, ?, ?';
$sqlParams = array($messageId, UnbDatabase::ConvertDateTime($now), UnbSession::GetSessionUserId(), $entryPoint ? 1 : 0);
if ($primaryRefId > 0)
{
$sqlCols .= ', "PrimaryReference"';
$sqlValues .= ', ?';
$sqlParams[] = $primaryRefId;
}
$db->Execute(
'INSERT INTO "#message" (' . $sqlCols . ') VALUES (' . $sqlValues . ')',
$sqlParams);
$rev = UnbMessageRevision::Create(
$messageId,
$subject,
$content,
null,
$data,
$dataFileName,
$writeToFile);
$db->Execute(
'UPDATE "#message" SET "SearchRevision" = ? WHERE "MessageId" = ?',
array($rev->GetRevisionNumber(), $messageId));
$db->Commit();
}
catch (Exception $ex)
{
$db->RollBack();
throw $ex;
}
$message = new self();
$message->messageId = $messageId;
$message->createdTime = $now;
$message->ownerUserId = UnbSession::GetSessionUserId();
$message->ownerUser = UnbSession::GetSessionUser();
$message->entryPoint = $entryPoint;
$message->searchRevision = $rev->GetRevisionNumber();
return $message;
}
protected static function FromRecord($rec)
{
if (!is_array($rec))
throw new InvalidArgumentException('Invalid argument type: $rec');
if (!count($rec))
throw new InvalidArgumentException('Record array is empty');
$message = new self();
$message->messageId = intval($rec['MessageId']);
$message->createdTime = UnbDatabase::ConvertDateTime($rec['CreatedTime']);
$message->ownerUserId = intval($rec['Owner']);
$message->primaryReference = intval($rec['PrimaryReference']);
$message->entryPoint = intval($rec['EntryPoint']) ? true : false;
$message->pageName = $rec['PageName'];
$message->moderationFlags = intval($rec['ModerationFlags']);
$message->lastBeginEdit = UnbDatabase::ConvertDateTime($rec['LastBeginEdit']);
$message->searchOptimisedContent = $rec['SearchOptimisedContent'];
$message->searchOptimisedSubject = $rec['SearchOptimisedSubject'];
$message->searchRevision = intval($rec['SearchRevision']);
return $message;
}
public static function FindByMessageId($messageId)
{
if (!is_int($messageId))
throw new InvalidArgumentException('Invalid argument type: $messageId');
if ($messageId <= 0)
throw new InvalidArgumentException('Argument out of range: $messageId');
$db = UnbDatabase::GetInstance();
$rec = $db->QueryRecord(
'SELECT "MessageId", "CreatedTime", "Owner", "PrimaryReference", "EntryPoint", "PageName", "ModerationFlags", "LastBeginEdit", "SearchRevision"
FROM "#message"
WHERE "MessageId" = ?',
$messageId);
if (!$rec) return null;
if (!UnbSession::IsAdministrator() &&
UnbSession::GetSessionUserId() != intval($rec['Owner']) &&
!self::IsReadAllowedById($messageId))
{
return null;
}
if ((intval($rec['ModerationFlags']) & UnbMessageModerationFlag::Hidden) &&
UnbSession::GetSessionUserId() != intval($rec['Owner']) &&
!UnbSession::IsModerator() &&
!UnbSession::IsAdministrator())
{
return null;
}
return self::FromRecord($rec);
}
public static function FindByPageName($pageName)
{
if (!is_string($pageName))
throw new InvalidArgumentException('Invalid argument type: $pageName');
$pageName = trim($pageName);
if (!strlen($pageName))
throw new InvalidArgumentException('Page name is empty');
if (mb_strlen($pageName) > 255)
throw new LengthException('[#2] Page name is too long');
$db = UnbDatabase::GetInstance();
$rec = $db->QueryRecord(
'SELECT "MessageId", "CreatedTime", "Owner", "PrimaryReference", "EntryPoint", "PageName", "ModerationFlags", "LastBeginEdit", "SearchRevision"
FROM "#message"
WHERE "PageName" = ?',
$pageName);
if (!$rec) return null;
if (!UnbSession::IsAdministrator() &&
UnbSession::GetSessionUserId() != intval($rec['Owner']) &&
!self::IsReadAllowedById(intval($rec['MessageId'])))
{
return null;
}
if ((intval($rec['ModerationFlags']) & UnbMessageModerationFlag::Hidden) &&
UnbSession::GetSessionUserId() != intval($rec['Owner']) &&
!UnbSession::IsModerator() &&
!UnbSession::IsAdministrator())
{
return null;
}
return self::FromRecord($rec);
}
public function GetMessageId()
{
return $this->messageId;
}
public function GetCreatedTime()
{
return $this->createdTime;
}
public function GetOwnerUserId()
{
return $this->ownerUserId;
}
public function GetOwnerUser()
{
if (isset($this->ownerUser))
return $this->ownerUser;
$this->ownerUser = UnbUser::FindByUserId($this->ownerUserId);
return $this->ownerUser;
}
public function SetOwner($owner)
{
$ownerId = $owner;
if ($ownerId instanceof UnbUser)
$ownerId = $ownerId->GetUserId();
if (!is_int($ownerId))
throw new InvalidArgumentException('Invalid argument type: $owner');
if ($ownerId <= 0)
throw new InvalidArgumentException('Argument out of range: $owner');
if (!UnbSession::IsAdministrator())
throw new UnbSecurityException('You need the administrator privilege to set the message owner.');
$this->db->StartTransaction();
try
{
$this->db->Execute(
'UPDATE "#message" SET "Owner" = ? WHERE "MessageId" = ?',
array($ownerId, $this->messageId));
$this->db->Commit();
}
catch (Exception $ex)
{
$this->db->RollBack();
throw $ex;
}
$this->ownerUserId = $ownerId;
$this->ownerUser = $owner instanceof UnbUser ? $owner : null;
self::ClearAccessCache();
}
public function GetPrimaryReference()
{
return $this->primaryReference;
}
public function IsEntryPoint()
{
return $this->entryPoint;
}
public function SetEntryPoint($entryPoint)
{
$entryPoint = $entryPoint ? true : false;
if (!UnbSession::IsModerator())
throw new UnbSecurityException('You need the moderator privilege to set a message entry point.');
$this->db->StartTransaction();
try
{
$this->db->Execute(
'UPDATE "#message" SET "EntryPoint" = ? WHERE "MessageId" = ?',
array($entryPoint ? 1 : 0, $this->messageId));
$this->db->Commit();
}
catch (Exception $ex)
{
$this->db->RollBack();
throw $ex;
}
$this->entryPoint = $entryPoint;
}
public function GetPageName()
{
return $this->pageName;
}
public function SetPageName($pageName)
{
if (isset($pageName))
{
if (!is_string($pageName))
throw new InvalidArgumentException('Invalid argument type: $pageName');
$pageName = trim($pageName);
if (mb_strlen($pageName) > 255)
throw new LengthException('[#2] Page name is too long');
if (!strlen($pageName))
$pageName = null;
}
if (!UnbSession::IsModerator())
throw new UnbSecurityException('You need the moderator privilege to set a page name.');
$this->db->StartTransaction();
try
{
$this->db->Execute(
'UPDATE "#message" SET "PageName" = ? WHERE "MessageId" = ?',
array($pageName, $this->messageId));
$this->db->Commit();
}
catch (Exception $ex)
{
$this->db->RollBack();
if ($this->db->IsConstraintError())
throw new UnbSecurityException('Page name is already assigned.');
throw $ex;
}
$this->pageName = $pageName;
}
public function GetReadAccessKeys()
{
$keys = $this->db->QueryArray(
'SELECT k."UserId"
FROM "#message" m, "#keylist" k
WHERE m."ReadAccessKeylistId" = k."KeylistId" AND m."MessageId" = ?',
$this->messageId);
$keys = array_map('intval', $keys);
return $keys;
}
public function GetEffectiveReadAccessKeys()
{
$keylistIds = $this->db->QueryArray(
'SELECT "ReadAccessKeylistId" AS "KeylistId"
FROM "#message"
WHERE "MessageId" = ? AND
"ReadAccessKeylistId" IS NOT NULL
UNION
SELECT t."ReadAccessKeylistId" AS "KeylistId"
FROM "#message" m
JOIN "#message_revision_tag" mrt ON
mrt."MessageId" = m."MessageId" AND
mrt."RevisionNumber" = m."SearchRevision"
JOIN "#tag" t USING ("TagId")
WHERE m."MessageId" = ? AND
t."ReadAccessKeylistId" IS NOT NULL',
array($this->messageId, $this->messageId));
$keylistIds = array_map('intval', $keylistIds);
if (!count($keylistIds)) return null;
$keysArray = array();
foreach ($keylistIds as $keylistId)
{
$keys = $this->db->QueryArray(
'SELECT "UserId" FROM "#keylist" WHERE "KeylistId" = ?',
$keylistId);
$keys = array_map('intval', $keys);
$keysArray[] = $keys;
}
$intersectKeys = array();
foreach ($keysArray[0] as $keyOfFirst)
{
$inAllOthers = true;
for ($n = 1; $n < count($keysArray); $n++)
{
if (!in_array($keyOfFirst, $keysArray[$n]))
{
$inAllOthers = false;
break;
}
}
if ($inAllOthers)
$intersectKeys[] = $keyOfFirst;
}
return $intersectKeys;
}
public function AddReadAccessKey($key)
{
if (UnbSession::GetSessionUserId() != $this->ownerUserId && !UnbSession::IsAdministrator())
throw new UnbSecurityException('You must be logged in as this user or need the administrator privilege to add read access keys.');
self::ClearAccessCache();
$this->db->AddKeylistKey('#message', 'ReadAccessKeylistId', 'MessageId', $this->messageId, $key);
}
public function RemoveReadAccessKey($key)
{
if (UnbSession::GetSessionUserId() != $this->ownerUserId && !UnbSession::IsAdministrator())
throw new UnbSecurityException('You must be logged in as this user or need the administrator privilege to remove read access keys.');
self::ClearAccessCache();
$this->db->RemoveKeylistKey('#message', 'ReadAccessKeylistId', 'MessageId', $this->messageId, $key);
}
public function SetReadAccessKeys($keys)
{
if (UnbSession::GetSessionUserId() != $this->ownerUserId && !UnbSession::IsAdministrator())
throw new UnbSecurityException('You must be logged in as this user or need the administrator privilege to set read access keys.');
$this->db->ResetKeylistKeys('#message', 'ReadAccessKeylistId', 'MessageId', $this->messageId, $keys);
}
public function GetAlterAccessKeys()
{
$keys = $this->db->QueryArray(
'SELECT k."UserId"
FROM "#message" m, "#keylist" k
WHERE m."AlterAccessKeylistId" = k."KeylistId" AND m."MessageId" = ?',
$this->messageId);
$keys = array_map('intval', $keys);
return $keys;
}
public function GetAllAlterAccessKeys()
{
$keylistIds = $this->db->QueryArray(
'SELECT "AlterAccessKeylistId" AS "KeylistId"
FROM "#message"
WHERE "MessageId" = ? AND
"AlterAccessKeylistId" IS NOT NULL
UNION
SELECT t."UseAccessKeylistId" AS "KeylistId"
FROM "#message" m
JOIN "#message_revision_tag" mrt ON
mrt."MessageId" = m."MessageId" AND
mrt."RevisionNumber" = m."SearchRevision"
JOIN "#tag" t USING ("TagId")
WHERE m."MessageId" = ? AND
t."UseAccessKeylistId" IS NOT NULL',
array($this->messageId, $this->messageId));
$keylistIds = array_map('intval', $keylistIds);
if (!count($keylistIds)) return null;
$keysArray = array();
foreach ($keylistIds as $keylistId)
{
$keys = $this->db->QueryArray(
'SELECT "UserId" FROM "#keylist" WHERE "KeylistId" = ?',
$keylistId);
$keys = array_map('intval', $keys);
$keysArray[] = $keys;
}
$intersectKeys = array();
foreach ($keysArray[0] as $keyOfFirst)
{
$inAllOthers = true;
for ($n = 1; $n < count($keysArray); $n++)
{
if (!in_array($keyOfFirst, $keysArray[$n]))
{
$inAllOthers = false;
break;
}
}
if ($inAllOthers)
$intersectKeys[] = $keyOfFirst;
}
return $intersectKeys;
}
public function AddAlterAccessKey($key)
{
if (UnbSession::GetSessionUserId() != $this->ownerUserId && !UnbSession::IsAdministrator())
throw new UnbSecurityException('You must be logged in as this user or need the administrator privilege to add alter access keys.');
self::ClearAccessCache();
$this->db->AddKeylistKey('#message', 'AlterAccessKeylistId', 'MessageId', $this->messageId, $key);
}
public function RemoveAlterAccessKey($key)
{
if (UnbSession::GetSessionUserId() != $this->ownerUserId && !UnbSession::IsAdministrator())
throw new UnbSecurityException('You must be logged in as this user or need the administrator privilege to remove alter access keys.');
self::ClearAccessCache();
$this->db->RemoveKeylistKey('#message', 'AlterAccessKeylistId', 'MessageId', $this->messageId, $key);
}
public function SetAlterAccessKeys($keys)
{
if (UnbSession::GetSessionUserId() != $this->ownerUserId && !UnbSession::IsAdministrator())
throw new UnbSecurityException('You must be logged in as this user or need the administrator privilege to set alter access keys.');
$this->db->ResetKeylistKeys('#message', 'AlterAccessKeylistId', 'MessageId', $this->messageId, $keys);
}
public function GetReplyAccessKeys()
{
$keys = $this->db->QueryArray(
'SELECT k."UserId"
FROM "#message" m, "#keylist" k
WHERE m."ReplyAccessKeylistId" = k."KeylistId" AND m."MessageId" = ?',
$this->messageId);
$keys = array_map('intval', $keys);
return $keys;
}
public function AddReplyAccessKey($key)
{
if (UnbSession::GetSessionUserId() != $this->ownerUserId && !UnbSession::IsAdministrator())
throw new UnbSecurityException('You must be logged in as this user or need the administrator privilege to add reply access keys.');
self::ClearAccessCache();
$this->db->AddKeylistKey('#message', 'ReplyAccessKeylistId', 'MessageId', $this->messageId, $key);
}
public function RemoveReplyAccessKey($key)
{
if (UnbSession::GetSessionUserId() != $this->ownerUserId && !UnbSession::IsAdministrator())
throw new UnbSecurityException('You must be logged in as this user or need the administrator privilege to remove reply access keys.');
self::ClearAccessCache();
$this->db->RemoveKeylistKey('#message', 'ReplyAccessKeylistId', 'MessageId', $this->messageId, $key);
}
public function SetReplyAccessKeys($keys)
{
if (UnbSession::GetSessionUserId() != $this->ownerUserId && !UnbSession::IsAdministrator())
throw new UnbSecurityException('You must be logged in as this user or need the administrator privilege to set reply access keys.');
$this->db->ResetKeylistKeys('#message', 'ReplyAccessKeylistId', 'MessageId', $this->messageId, $keys);
}
public function GetModerationFlags()
{
return $this->moderationFlags;
}
public function SetModerationFlags($moderationFlags)
{
if (!is_int($moderationFlags))
throw new InvalidArgumentException('Invalid argument type: $moderationFlags');
if ($moderationFlags < 0)
throw new InvalidArgumentException('Argument out of range: $moderationFlags');
if (!UnbSession::IsLoggedIn())
throw new UnbSecurityException('You need the moderator privilege to set moderation flags.');
$this->db->StartTransaction();
try
{
$this->db->Execute(
'UPDATE "#message" SET "ModerationFlags" = ? WHERE "MessageId" = ?',
array($moderationFlags, $this->messageId));
$this->db->Commit();
}
catch (Exception $ex)
{
$this->db->RollBack();
throw $ex;
}
$this->moderationFlags = $moderationFlags;
}
public function GetLastBeginEdit()
{
return $this->lastBeginEdit;
}
public function SetLastBeginEdit($editing)
{
$lastBeginEdit = $editing ? time() : 0;
$this->db->StartTransaction();
try
{
$this->db->Execute(
'UPDATE "#message" SET "LastBeginEdit" = ? WHERE "MessageId" = ?',
array(UnbDatabase::ConvertDateTime($lastBeginEdit), $this->messageId));
$this->db->Commit();
}
catch (Exception $ex)
{
$this->db->RollBack();
throw $ex;
}
$this->lastBeginEdit = $lastBeginEdit;
}
public function GetSearchOptimisedContent()
{
return $this->searchOptimisedContent;
}
public function SetSearchOptimisedContent($content)
{
if (isset($content))
{
if (!is_string($content))
throw new InvalidArgumentException('Invalid argument type: $content');
$content = trim($content);
if (mb_strlen($content) > 16777215)
throw new LengthException('[#3] Content is too long');
if (!strlen($content))
$content = null;
}
$this->db->StartTransaction();
try
{
$this->db->Execute(
'UPDATE "#message" SET "SearchOptimisedContent" = ? WHERE "MessageId" = ?',
array($content, $this->messageId));
$this->db->Commit();
}
catch (Exception $ex)
{
$this->db->RollBack();
throw $ex;
}
$this->searchOptimisedContent = $content;
}
public function GetSearchOptimisedSubject()
{
return $this->searchOptimisedSubject;
}
public function SetSearchOptimisedSubject($subject)
{
if (isset($subject))
{
if (!is_string($subject))
throw new InvalidArgumentException('Invalid argument type: $subject');
$subject = trim($subject);
if (mb_strlen($subject) > 255)
throw new LengthException('[#4] Subject is too long');
if (!strlen($subject))
$subject = null;
}
$this->db->StartTransaction();
try
{
$this->db->Execute(
'UPDATE "#message" SET "SearchOptimisedSubject" = ? WHERE "MessageId" = ?',
array($subject, $this->messageId));
$this->db->Commit();
}
catch (Exception $ex)
{
$this->db->RollBack();
throw $ex;
}
$this->searchOptimisedSubject = $subject;
}
public function GetSearchRevisionNumber()
{
return $this->searchRevision;
}
public function GetSearchRevision($contentLevel = null, $allowFallback = false)
{
return $this->GetRevision($this->searchRevision, $contentLevel, $allowFallback);
}
public function SetSearchRevisionNumber($revisionNumber)
{
if (isset($revisionNumber))
{
if (!is_int($revisionNumber))
throw new InvalidArgumentException('Invalid argument type: $revisionNumber');
if ($revisionNumber <= 0)
throw new InvalidArgumentException('Argument out of range: $revisionNumber');
}
$this->db->StartTransaction();
try
{
$this->db->Execute(
'UPDATE "#message" SET "SearchRevision" = ? WHERE "MessageId" = ?',
array($revisionNumber, $this->messageId));
$this->db->Commit();
}
catch (Exception $ex)
{
$this->db->RollBack();
throw $ex;
}
$this->searchRevision = intval($revisionNumber);
}
public function Delete()
{
if (!UnbSession::IsModerator())
throw new UnbSecurityException('You need the moderator privilege to delete a message.');
$this->db->StartTransaction();
try
{
$this->db->ResetKeylistKeys('#message', 'ReadAccessKeylistId', 'MessageId', $this->messageId, null);
$this->db->ResetKeylistKeys('#message', 'AlterAccessKeylistId', 'MessageId', $this->messageId, null);
$this->db->ResetKeylistKeys('#message', 'ReplyAccessKeylistId', 'MessageId', $this->messageId, null);
$this->db->Execute('DELETE FROM "#message" WHERE "MessageId" = ?', $this->messageId);
$this->db->Commit();
}
catch (Exception $ex)
{
$this->db->RollBack();
throw $ex;
}
}
public static function UpdateSearchFields($messageId)
{
if ($messageId instanceof UnbMessage)
$messageId = $messageId->GetMessageId();
if (!is_int($messageId))
throw new InvalidArgumentException('Invalid argument type: $messageId');
if ($messageId <= 0)
throw new InvalidArgumentException('Argument out of range: $messageId');
$db = UnbDatabase::GetInstance();
$db->StartTransaction(true);
try
{
$revisionNumber = $db->QueryValue(
'SELECT MAX("RevisionNumber")
FROM "#message_revision"
WHERE "MessageId" = ? AND "ModerationState" = ' . UnbMessageRevisionModerationState::Approved . '
FOR UPDATE',
$messageId);
if (!isset($revisionNumber))
$revisionNumber = $db->QueryValue(
'SELECT MAX("RevisionNumber")
FROM "#message_revision"
WHERE "MessageId" = ?
FOR UPDATE',
$messageId);
$rec = $db->QueryRecord(
'SELECT "Subject", "Content", "ModerationState"
FROM "#message_revision"
WHERE "MessageId" = ? AND "RevisionNumber" = ?',
array($messageId, $revisionNumber));
$content = null;
$subject = null;
if (isset($rec) && $rec['ModerationState'] == UnbMessageRevisionModerationState::Approved)
{
}
$db->Execute(
'UPDATE "#message"
SET "SearchOptimisedContent" = ?, "SearchOptimisedSubject" = ?, "SearchRevision" = ?
WHERE "MessageId" = ?',
array($content, $subject, $revisionNumber, $messageId));
$db->Commit();
}
catch (Exception $ex)
{
$db->RollBack();
throw $ex;
}
}
public function GetMoreReferences()
{
$sql = 'SELECT mr."Reference", MIN(mrev."CreatedTime") AS ct
FROM "#message_reference" mr,
"#message" m
JOIN "#message_revision" mrev USING ("MessageId")
WHERE mr."Reference" = m."MessageId" AND
mr."MessageId" = ?
GROUP BY mr."Reference"
ORDER BY ct';
$parameters = $this->messageId;
$refs = $this->db->QueryArray($sql, $parameters);
$refs = array_map('intval', $refs);
return $refs;
}
public function AddReference($reference)
{
if ($reference instanceof UnbMessage)
$reference = $reference->GetMessageId();
if (!is_int($reference))
throw new InvalidArgumentException('Invalid argument type: $reference');
if ($reference <= 0)
throw new InvalidArgumentException('Argument out of range: $reference');
if (UnbSession::GetSessionUserId() != $this->ownerUserId && !UnbSession::IsModerator())
throw new UnbSecurityException('You must be logged in as this user or need the moderator privilege to add a reference.');
$this->db->StartTransaction();
try
{
try
{
$this->db->Execute(
'INSERT INTO "#message_reference" ("MessageId", "Reference") VALUES (?, ?)',
array($this->messageId, $reference));
}
catch (UnbDatabaseException $ex)
{
if (!$this->db->IsConstraintError()) throw $ex;
}
$this->db->Commit();
}
catch (Exception $ex)
{
$this->db->RollBack();
throw $ex;
}
}
public function RemoveReference($reference)
{
if ($reference instanceof UnbMessage)
$reference = $reference->GetMessageId();
if (!is_int($reference))
throw new InvalidArgumentException('Invalid argument type: $reference');
if ($reference <= 0)
throw new InvalidArgumentException('Argument out of range: $reference');
if (UnbSession::GetSessionUserId() != $this->ownerUserId && !UnbSession::IsModerator())
throw new UnbSecurityException('You must be logged in as this user or need the moderator privilege to remove a reference.');
$this->db->StartTransaction();
try
{
$this->db->Execute(
'DELETE FROM "#message_reference" WHERE "MessageId" = ? AND "Reference" = ?',
array($this->messageId, $reference));
$this->db->Commit();
}
catch (Exception $ex)
{
$this->db->RollBack();
throw $ex;
}
}
public function GetRefDEP()
{
if ($this->entryPoint) return $this->messageId;
if (!$this->primaryReference) return $this->messageId;
$messageId = $this->primaryReference;
$seenMessageIds = array();
while (true)
{
$seenMessageIds[] = $messageId;
$rec = $this->db->QueryRecord(
'SELECT "EntryPoint", "PrimaryReference" FROM "#message" WHERE "MessageId" = ?',
$messageId);
if ($rec['EntryPoint']) return $messageId;
$primaryReference = intval($rec['PrimaryReference']);
if (!$primaryReference) return $messageId;
$messageId = $primaryReference;
if (in_array($messageId, $seenMessageIds))
throw new UnbDatabaseException('Recursive primary message references detected, starting at ' . $this->messageId);
}
}
public function GetLatestReply()
{
$latestRecord = array();
$latestTime = '';
$refIds = array($this->messageId);
while (true)
{
$recs = $this->db->Query('SELECT * FROM "#message" WHERE "PrimaryReference" IN (' . join(', ', $refIds) . ')');
if (!$recs) break;
$refIds = array();
foreach ($recs as $rec)
{
if ($rec['CreatedTime'] > $latestTime)
{
$latestTime = $rec['CreatedTime'];
$latestRecord = $rec;
}
$refIds[] = intval($rec['MessageId']);
}
}
if ($latestRecord)
return self::FromRecord($latestRecord);
return null;
}
public function CountReplies()
{
$count = 0;
$refIds = array($this->messageId);
while (true)
{
$recs = $this->db->Query('SELECT "MessageId" FROM "#message" WHERE "PrimaryReference" IN (' . join(', ', $refIds) . ')');
if (!$recs) break;
$refIds = array();
foreach ($recs as $rec)
{
$count++;
$refIds[] = intval($rec['MessageId']);
}
}
return $count;
}
public function AddRevision($subject, $content, $summary = '')
{
return UnbMessageRevision::Create($this->messageId, $subject, $content, $summary);
}
public function GetRevision($revisionNumber, $contentLevel = null, $allowFallback = false)
{
return UnbMessageRevision::FindByMessageId($this->messageId, $revisionNumber, $contentLevel, $allowFallback);
}
public function GetLatestRevision($contentLevel = null, $allowFallback = false)
{
return UnbMessageRevision::FindLatestByMessageId($this->messageId, $contentLevel, $allowFallback);
}
public function GetAllRevisions()
{
return UnbMessageRevision::FindAllByMessageId($this->messageId);
}
public function GetRating()
{
$rec = $this->db->QueryRecord(
'SELECT COUNT(*) AS "Count", SUM("Weight") AS "TotalWeight", SUM("Rating" * "Weight") AS "WeightedSum"
FROM "#message_rating"
WHERE "MessageId" = ?',
$this->messageId);
if (!$rec) return null;
$totalWeight = max(100.0, floatval($rec['TotalWeight']));
return array(
intval($rec['Count']),
intval($rec['TotalWeight']),
floatval($rec['WeightedSum']) / $totalWeight);
}
public function GetMyRating()
{
if (!UnbSession::IsLoggedIn())
throw new UnbSecurityException('You must be logged in to get your rating.');
$r = $this->db->QueryValue(
'SELECT "Rating" FROM "#message_rating" WHERE "UserId" = ? AND "MessageId" = ?',
array(UnbSession::GetSessionUserId(), $this->messageId));
if (!isset($r)) return null;
return intval($r);
}
public function SetRating($rating)
{
if (!is_int($rating))
throw new InvalidArgumentException('Invalid argument type: $rating');
if (abs($rating) > 10)
throw new InvalidArgumentException('Argument out of range: $rating');
if (!UnbSession::IsLoggedIn())
throw new UnbSecurityException('You must be logged in to rate a message.');
if (UnbSession::GetSessionUserId() == $this->ownerUserId)
throw new UnbSecurityException('You cannot rate your own messages.');
$userRating = UnbSession::GetSessionUser()->GetMessagesRating();
if (!isset($userRating)) return false;
$myWeight = $userRating[2] + 10;
$this->db->StartTransaction();
try
{
$this->db->Execute(
'UPDATE "#message_rating" SET "Rating" = ?, "Weight" = ? WHERE "UserId" = ? AND "MessageId" = ?',
array($rating, $myWeight, UnbSession::GetSessionUserId(), $this->messageId));
if ($this->db->GetRowsAffected() == 0)
{
try
{
$this->db->Execute(
'INSERT INTO "#message_rating" ("UserId", "MessageId", "Rating", "Weight") VALUES (?, ?, ?, ?)',
array(UnbSession::GetSessionUserId(), $this->messageId, $rating, $myWeight));
}
catch (UnbDatabaseException $ex)
{
if (!$this->db->IsConstraintError()) throw $ex;
}
}
$this->db->Commit();
}
catch (Exception $ex)
{
$this->db->RollBack();
throw $ex;
}
}
public function IsReadAllowed()
{
return self::IsReadAllowedById($this->messageId);
}
public static function IsReadAllowedById($messageId)
{
if ($messageId instanceof UnbMessage)
$messageId = $messageId->GetMessageId();
if (!is_int($messageId))
throw new InvalidArgumentException('Invalid argument type: $messageId');
if (array_key_exists($messageId, self::$readAccessCache))
return self::$readAccessCache[$messageId];
$sessionUserId = UnbSession::GetSessionUserId();
$sql = 'SELECT
(EXISTS
-- Message has no read access keylist
(SELECT 1
FROM "#message" m
WHERE m."MessageId" = ? AND
m."ReadAccessKeylistId" IS NULL)
OR EXISTS
-- Message\'s read access keylist contains personal or additional user key
(SELECT 1
FROM "#message" m, "#keylist" mk
WHERE mk."KeylistId" = m."ReadAccessKeylistId" AND
m."MessageId" = ? AND
(mk."UserId" = ? OR
mk."UserId" IN
(SELECT uk."UserId"
FROM "#user" u, "#keylist" uk
WHERE uk."KeylistId" = u."AdditionalKeylistId" AND
u."UserId" = ?))))
AND NOT EXISTS
-- There must not be any tag that denies access
(SELECT "Allowed"
FROM
(SELECT
EXISTS
-- Tag\'s read access keylist contains personal or additional user key
(SELECT 1
FROM "#keylist" tk
WHERE tk."KeylistId" = t."ReadAccessKeylistId" AND
(tk."UserId" = ? OR
tk."UserId" IN
(SELECT uk."UserId"
FROM "#user" u, "#keylist" uk
WHERE uk."KeylistId" = u."AdditionalKeylistId" AND
u."UserId" = ?)))
AS "Allowed"
FROM "#message" m
JOIN "#message_revision_tag" mrt ON
mrt."MessageId" = m."MessageId" AND
mrt."RevisionNumber" = m."SearchRevision"
JOIN "#tag" t USING ("TagId")
WHERE m."MessageId" = ? AND
-- Only regard tags with read access keylist (only they can fail the access test)
t."ReadAccessKeylistId" IS NOT NULL) AS foo
WHERE NOT "Allowed")';
$parameters = array($messageId, $messageId, $sessionUserId, $sessionUserId, $sessionUserId, $sessionUserId, $messageId);
$db = UnbDatabase::GetInstance();
$val = $db->QueryValue($sql, $parameters);
$val = $val ? true : false;
self::$readAccessCache[$messageId] = $val;
return $val;
}
public static function SetReadAllowedIds($messageIds)
{
foreach ($messageIds as $messageId)
{
if (!array_key_exists($messageId, self::$readAccessCache))
self::$readAccessCache[$messageId] = true;
}
}
public function IsAlterAllowed()
{
return self::IsAlterAllowedById($this->messageId);
}
public static function IsAlterAllowedById($messageId)
{
if ($messageId instanceof UnbMessage)
$messageId = $messageId->GetMessageId();
if (!is_int($messageId))
throw new InvalidArgumentException('Invalid argument type: $messageId');
if (array_key_exists($messageId, self::$alterAccessCache))
return self::$alterAccessCache[$messageId];
$sessionUserId = UnbSession::GetSessionUserId();
if (!$sessionUserId) return false;
$sql = 'SELECT
(EXISTS
-- Message has no alter access keylist
(SELECT 1
FROM "#message" m
WHERE m."MessageId" = ? AND
m."AlterAccessKeylistId" IS NULL)
OR EXISTS
-- Message\'s alter access keylist contains personal or additional user key
(SELECT 1
FROM "#message" m, "#keylist" mk
WHERE mk."KeylistId" = m."AlterAccessKeylistId" AND
m."MessageId" = ? AND
(mk."UserId" = ? OR
mk."UserId" IN
(SELECT uk."UserId"
FROM "#user" u, "#keylist" uk
WHERE uk."KeylistId" = u."AdditionalKeylistId" AND
u."UserId" = ?))))
AND NOT EXISTS
-- There must not be any tag that denies access
(SELECT "Allowed"
FROM
(SELECT
EXISTS
-- Tag\'s use access keylist contains personal or additional user key
(SELECT 1
FROM "#keylist" tk
WHERE tk."KeylistId" = t."UseAccessKeylistId" AND
(tk."UserId" = ? OR
tk."UserId" IN
(SELECT uk."UserId"
FROM "#user" u, "#keylist" uk
WHERE uk."KeylistId" = u."AdditionalKeylistId" AND
u."UserId" = ?)))
AS "Allowed"
FROM "#message" m
JOIN "#message_revision_tag" mrt ON
mrt."MessageId" = m."MessageId" AND
mrt."RevisionNumber" = m."SearchRevision"
JOIN "#tag" t USING ("TagId")
WHERE m."MessageId" = ? AND
-- Only regard tags with use access keylist (only they can fail the access test)
t."UseAccessKeylistId" IS NOT NULL) AS foo
WHERE NOT "Allowed")';
$parameters = array($messageId, $messageId, $sessionUserId, $sessionUserId, $sessionUserId, $sessionUserId, $messageId);
$db = UnbDatabase::GetInstance();
$val = $db->QueryValue($sql, $parameters);
$val = $val ? true : false;
self::$alterAccessCache[$messageId] = $val;
return $val;
}
public function IsReplyAllowed()
{
return self::IsReplyAllowedById($this->messageId);
}
public static function IsReplyAllowedById($messageId)
{
if ($messageId instanceof UnbMessage)
$messageId = $messageId->GetMessageId();
if (!is_int($messageId))
throw new InvalidArgumentException('Invalid argument type: $messageId');
if (array_key_exists($messageId, self::$replyAccessCache))
return self::$replyAccessCache[$messageId];
$sessionUserId = UnbSession::GetSessionUserId();
$sql = 'SELECT
(EXISTS
-- Message has no reply access keylist
(SELECT 1
FROM "#message" m
WHERE m."MessageId" = ? AND
m."ReplyAccessKeylistId" IS NULL)
OR EXISTS
-- Message\'s reply access keylist contains personal or additional user key
(SELECT 1
FROM "#message" m, "#keylist" mk
WHERE mk."KeylistId" = m."ReplyAccessKeylistId" AND
m."MessageId" = ? AND
(mk."UserId" = ? OR
mk."UserId" IN
(SELECT uk."UserId"
FROM "#user" u, "#keylist" uk
WHERE uk."KeylistId" = u."AdditionalKeylistId" AND
u."UserId" = ?))))';
$parameters = array($messageId, $messageId, $sessionUserId, $sessionUserId);
$db = UnbDatabase::GetInstance();
$val = $db->QueryValue($sql, $parameters);
$val = $val ? true : false;
self::$replyAccessCache[$messageId] = $val;
return $val;
}
public static function ClearAccessCache()
{
self::$readAccessCache = array();
self::$alterAccessCache = array();
self::$replyAccessCache = array();
}
}
class UnbMessageRevisionModerationState
{
const WaitingForApproval = 0;
const Approved = 1;
const Locked = 2;
}
class UnbMessageRevisionContentLevel
{
const NoContent = 0;
const Subject = 1;
const Content = 2;
const HtmlContent = 3;
}
class UnbMessageRevision extends UnbEntity
{
private $messageId = 0;
private $revisionNumber = 0;
private $createdTime = 0;
private $author = 0;
private $subject = '';
private $content = '';
private $htmlContent = '';
private $summary = '';
private $moderationState = 0;
private $hasData = false;
private $authorUser = null;
private $contentLevel = 0;
private $db;
private function __construct()
{
$this->db = UnbDatabase::GetInstance();
}
public static function Create($messageId, $subject, $content, $summary = null, $data = null, $dataFileName = null, $writeToFile = null)
{
if ($messageId instanceof UnbMessage)
$messageId = $messageId->GetMessageId();
if (!is_int($messageId))
throw new InvalidArgumentException('Invalid argument type: $messageId');
if ($messageId <= 0)
throw new InvalidArgumentException('Argument out of range: $messageId');
if (isset($subject))
{
if (!is_string($subject))
throw new InvalidArgumentException('Invalid argument type: $subject');
$subject = trim($subject);
if (mb_strlen($subject) > 255)
throw new LengthException('[#4] Subject is too long');
if (!strlen($subject)) $subject = null;
}
if (isset($content))
{
if (!is_string($content))
throw new InvalidArgumentException('Invalid argument type: $content');
$content = str_replace("\r", '', $content);
$content = preg_replace('_[ \t]+(?=\n)_', '', $content);
$content = rtrim($content);
if (mb_strlen($content) > 16777215)
throw new LengthException('[#3] Content is too long');
if (!strlen(trim($content))) $content = null;
}
if (isset($summary))
{
if (!is_string($summary))
throw new InvalidArgumentException('Invalid argument type: $summary');
$summary = trim($summary);
if (mb_strlen($summary) > 255)
throw new LengthException('[#5] Summary is too long');
if (!strlen($summary)) $summary = null;
}
if (isset($data))
{
if (!is_string($data))
throw new InvalidArgumentException('Invalid argument type: $data');
}
if (isset($dataFileName))
{
if (!is_string($dataFileName))
throw new InvalidArgumentException('Invalid argument type: $dataFileName');
if (isset($data))
throw new InvalidArgumentException('You cannot set $data and $dataFileName at the same time.');
if (!strlen($dataFileName)) $dataFileName = null;
if (isset($dataFileName) && !file_exists($dataFileName))
throw new InvalidArgumentException('The specified file does not exist.');
}
if (!UnbSession::IsLoggedIn())
throw new UnbSecurityException('You must be logged in to create a message revision.');
$now = time();
if (!UnbMessage::IsReadAllowedById($messageId))
throw new UnbSecurityException('You are not allowed to read this message.');
if (!UnbMessage::IsAlterAllowedById($messageId))
throw new UnbSecurityException('You are not allowed to alter this message.');
$db = UnbDatabase::GetInstance();
$db->StartTransaction(true);
try
{
$revisionNumber = $db->QueryValue(
'SELECT MAX("RevisionNumber") FROM "#message_revision" WHERE "MessageId" = ? FOR UPDATE',
$messageId);
if (!isset($revisionNumber))
$revisionNumber = 0;
$revisionNumber++;
$fields = '';
$values = '';
$parameters = array(
$messageId,
$revisionNumber,
UnbDatabase::ConvertDateTime($now),
UnbSession::GetSessionUserId(),
$subject,
$content,
$summary);
$autoApprove = false;
if (UnbSession::IsModerator())
$autoApprove = true;
if (UnbSession::IsTrusted() && UnbConfiguration::Get('moderation.approve-from-trusted', false))
$autoApprove = true;
if ($autoApprove)
{
$fields .= ', "ModerationState"';
$values .= ', ?';
$parameters[] = UnbMessageRevisionModerationState::Approved;
}
if (isset($data) || isset($dataFileName))
{
$fields .= ', "HasData"';
$values .= ', ?';
$parameters[] = 1;
if (!isset($writeToFile))
{
$dbMaxSize = UnbConfiguration::Get('attachment.storage.database-max-size', -1);
if ($dbMaxSize == -1)
{
$writeToFile = false;
}
else
{
if (is_string($data))
$dataLength = strlen($data);
else
$dataLength = filesize($dataFileName);
$writeToFile = $dataLength > $dbMaxSize;
}
}
if ($writeToFile)
{
$newFilename = UnbConfiguration::Get('path.file-uploads', '.') . '/' . $messageId . '_' . $revisionNumber;
if (isset($dataFileName))
{
if (is_uploaded_file($dataFileName))
{
if (!move_uploaded_file($dataFileName, $newFilename))
throw new Exception('[#6] The uploaded file could not be moved to its new location.');
}
else
{
if (!copy($dataFileName, $newFilename))
throw new Exception('[#6] The input file could not be copied to its new location.');
}
}
else
{
if (file_put_contents($newFilename, $data) === false)
throw new Exception('[#7] The data could not be written to the local file.');
}
}
else
{
if (is_string($dataFileName))
{
$data = file_get_contents($dataFileName);
if ($data === false)
throw new Exception('[#8] The data could not be read from the input file.');
}
$fields .= ', "Data"';
$values .= ', ?';
$parameters[] = $data;
}
}
$sql = 'INSERT INTO "#message_revision" ("MessageId", "RevisionNumber", "CreatedTime", "Author", "Subject", "Content", "Summary"' . $fields . ') VALUES (?, ?, ?, ?, ?, ?, ?' . $values . ')';
$db->Execute($sql, $parameters);
UnbMessage::UpdateSearchFields($messageId);
$db->Commit();
}
catch (Exception $ex)
{
$db->RollBack();
throw $ex;
}
$messageRevision = new self();
$messageRevision->messageId = $messageId;
$messageRevision->revisionNumber = $revisionNumber;
$messageRevision->createdTime = $now;
$messageRevision->author = UnbSession::GetSessionUserId();
$messageRevision->subject = $subject;
$messageRevision->content = $content;
$messageRevision->summary = $summary;
$messageRevision->hasData = isset($data) || isset($dataFileName);
return $messageRevision;
}
public function CreateCopy($subject, $content, $summary, $data, $dataFileName, $writeToFile)
{
if ($this->contentLevel < UnbMessageRevisionContentLevel::Content)
throw new UnbSecurityException('This revision was not retrieved with an appropriate content level to get the content.');
if ($data === true)
{
$data = $this->GetFileData(2);
$dataFileName = $this->GetFileDataPath();
}
$db = UnbDatabase::GetInstance();
$db->StartTransaction(true);
try
{
$newRevision = self::Create(
$this->messageId,
($subject === true ? $this->subject : $subject),
($content === true ? $this->content : $content),
$summary,
$data,
$dataFileName,
$writeToFile);
$sql = 'INSERT INTO "#message_revision_tag"
("MessageId", "RevisionNumber", "TagId", "TagData")
SELECT "MessageId", ? "RevisionNumber", "TagId", "TagData"
FROM "#message_revision_tag"
WHERE "MessageId" = ? AND "RevisionNumber" = ?';
$parameters = array(
$newRevision->GetRevisionNumber(),
$this->messageId,
$this->revisionNumber);
$db->Execute($sql, $parameters);
$db->Commit();
}
catch (Exception $ex)
{
$db->RollBack();
throw $ex;
}
return $newRevision;
}
protected static function FromRecord($rec)
{
if (!is_array($rec))
throw new InvalidArgumentException('Invalid argument type: $rec');
if (!count($rec))
throw new InvalidArgumentException('Record array is empty');
$messageRevision = new self();
$messageRevision->messageId = intval($rec['MessageId']);
$messageRevision->revisionNumber = intval($rec['RevisionNumber']);
$messageRevision->createdTime = UnbDatabase::ConvertDateTime($rec['CreatedTime']);
$messageRevision->author = intval($rec['Author']);
$messageRevision->subject = $rec['Subject'];
$messageRevision->content = $rec['Content'];
$messageRevision->htmlContent = $rec['HtmlContent'];
$messageRevision->summary = $rec['Summary'];
$messageRevision->moderationState = intval($rec['ModerationState']);
$messageRevision->hasData = $rec['HasData'] && $rec['HasData'] !== 'FALSE';
return $messageRevision;
}
private static function ReadInternal($messageId, $revisionNumber, $contentLevel, $allowFallback)
{
if ($messageId instanceof UnbMessage)
$messageId = $messageId->GetMessageId();
if (!is_int($messageId))
throw new InvalidArgumentException('Invalid argument type: $messageId');
if ($messageId <= 0)
throw new InvalidArgumentException('Argument out of range: $messageId');
if (!isset($contentLevel))
$contentLevel = UnbMessageRevisionContentLevel::Content;
if (!is_int($contentLevel))
throw new InvalidArgumentException('Invalid argument type: $contentLevel');
if ($contentLevel < 0)
throw new InvalidArgumentException('Argument out of range: $contentLevel');
$contentFields = '';
if ($contentLevel == UnbMessageRevisionContentLevel::Subject)
$contentFields .= 'mr."Subject", mr."Summary", ';
elseif ($contentLevel == UnbMessageRevisionContentLevel::Content)
$contentFields .= 'mr."Subject", mr."Summary", mr."Content", ';
elseif ($contentLevel == UnbMessageRevisionContentLevel::HtmlContent)
$contentFields .= 'mr."Subject", mr."Summary", mr."Content", mr."HtmlContent", ';
$sessionUser = UnbSession::GetSessionUser();
$sessionUserId = UnbSession::GetSessionUserId();
if ($allowFallback)
{
$maxContentLevel = $contentLevel;
$contentLevel = UnbMessageRevisionContentLevel::NoContent;
}
$isReadAllowed = UnbMessage::IsReadAllowedById($messageId);
$isMsgAccessException = UnbSession::IsAdministrator() && $contentLevel <= UnbMessageRevisionContentLevel::Subject;
$isMsgModException = UnbSession::IsModerator() ||
$contentLevel == UnbMessageRevisionContentLevel::NoContent ||
UnbSession::IsAdministrator() && $contentLevel <= UnbMessageRevisionContentLevel::Subject;
$sql = 'SELECT mr."MessageId", mr."RevisionNumber", mr."CreatedTime", mr."Author", ' . $contentFields . 'mr."ModerationState", mr."HasData", m."ModerationFlags"
FROM "#message" m
JOIN "#message_revision" mr USING ("MessageId")
WHERE mr."MessageId" = ? AND
(? OR
m."Owner" = ? AND ?) AND
(m."ModerationFlags" & ' . (UnbMessageModerationFlag::Locked | UnbMessageModerationFlag::Hidden) . ' = 0 OR
? OR
m."Owner" = ?) AND
mr."RevisionNumber" = ';
$parameters = array(
$messageId,
$isReadAllowed || $isMsgAccessException,
$sessionUserId,
$contentLevel <= UnbMessageRevisionContentLevel::Subject,
$isMsgModException,
$sessionUserId);
$isRevModException = $isMsgModException;
if (isset($revisionNumber))
{
$sql .= '? AND
(mr."ModerationState" = ' . UnbMessageRevisionModerationState::Approved . ' OR
? OR
mr."Author" = ?)';
$parameters[] = $revisionNumber;
$parameters[] = $isRevModException;
$parameters[] = $sessionUserId;
}
else
{
$sql .= '(SELECT MAX(mr2."RevisionNumber")
FROM "#message_revision" mr2
WHERE mr2."MessageId" = mr."MessageId" AND
(mr2."ModerationState" = ' . UnbMessageRevisionModerationState::Approved . ' OR
? OR
mr2."Author" = ?))';
$parameters[] = $isRevModException;
$parameters[] = $sessionUserId;
}
$db = UnbDatabase::GetInstance();
$rec = $db->QueryRecord($sql, $parameters);
if (!$rec) return null;
if ($allowFallback)
{
$possibleContentLevel = UnbMessageRevisionContentLevel::HtmlContent;
if (!$isReadAllowed)
{
if (UnbSession::IsAdministrator() || intval($rec['Owner']) == $sessionUserId)
{
$possibleContentLevel = min(UnbMessageRevisionContentLevel::Subject, $possibleContentLevel);
}
else
{
$possibleContentLevel = UnbMessageRevisionContentLevel::NoContent;
}
}
if (intval($rec['ModerationFlags']) & (UnbMessageModerationFlag::Locked | UnbMessageModerationFlag::Hidden) ||
intval($rec['ModerationState']) != UnbMessageRevisionModerationState::Approved)
{
if (UnbSession::IsModerator() || intval($rec['Author']) == $sessionUserId)
{
}
elseif (UnbSession::IsAdministrator())
{
$possibleContentLevel = min(UnbMessageRevisionContentLevel::Subject, $possibleContentLevel);
}
else
{
$possibleContentLevel = UnbMessageRevisionContentLevel::NoContent;
}
}
$contentLevel = min($possibleContentLevel, $maxContentLevel);
if ($contentLevel < UnbMessageRevisionContentLevel::Subject)
$rec['Subject'] = $rec['Summary'] = null;
if ($contentLevel < UnbMessageRevisionContentLevel::Content)
$rec['Content'] = null;
if ($contentLevel < UnbMessageRevisionContentLevel::HtmlContent)
$rec['HtmlContent'] = null;
}
$rev = self::FromRecord($rec);
$rev->contentLevel = $contentLevel;
return $rev;
}
public static function FindByMessageId($messageId, $revisionNumber, $contentLevel = null, $allowFallback = false)
{
if (!is_int($revisionNumber))
throw new InvalidArgumentException('Invalid argument type: $revisionNumber');
if ($revisionNumber <= 0)
throw new InvalidArgumentException('Argument out of range: $revisionNumber');
return self::ReadInternal($messageId, $revisionNumber, $contentLevel, $allowFallback);
}
public static function FindLatestByMessageId($messageId, $contentLevel = null, $allowFallback = false)
{
return self::ReadInternal($messageId, null, $contentLevel, $allowFallback);
}
public static function FirstCreatedTime($messageId)
{
if ($messageId instanceof UnbMessage)
$messageId = $messageId->GetMessageId();
if (!is_int($messageId))
throw new InvalidArgumentException('Invalid argument type: $messageId');
if ($messageId <= 0)
throw new InvalidArgumentException('Argument out of range: $messageId');
$isReadAllowed = UnbMessage::IsReadAllowedById($messageId);
$isMsgAccessException = UnbSession::IsAdministrator();
$isMsgModException = UnbSession::IsModerator() ||
UnbSession::IsAdministrator();
$sql = 'SELECT mr."CreatedTime", m."Owner"
FROM "#message" m
JOIN "#message_revision" mr USING ("MessageId")
WHERE mr."MessageId" = ? AND
(? OR
m."Owner" = ?) AND
(m."ModerationFlags" & ' . UnbMessageModerationFlag::Hidden . ' = 0 OR
? OR
m."Owner" = ?) AND
mr."RevisionNumber" =
(SELECT MIN(mr2."RevisionNumber")
FROM "#message_revision" mr2
WHERE mr2."MessageId" = mr."MessageId")';
$parameters = array(
$messageId,
$isReadAllowed || $isMsgAccessException,
UnbSession::GetSessionUserId(),
$isMsgModException,
UnbSession::GetSessionUserId());
$db = UnbDatabase::GetInstance();
$rec = $db->QueryRecord($sql, $parameters);
if (!isset($rec)) return null;
return UnbDatabase::ConvertDateTime($rec['CreatedTime']);
}
public static function FindAllByMessageId($messageId)
{
if ($messageId instanceof UnbMessage)
$messageId = $messageId->GetMessageId();
if (!is_int($messageId))
throw new InvalidArgumentException('Invalid argument type: $messageId');
if ($messageId <= 0)
throw new InvalidArgumentException('Argument out of range: $messageId');
$isReadAllowed = UnbMessage::IsReadAllowedById($messageId);
$isMsgAccessException = UnbSession::IsAdministrator();
$sql = 'SELECT mr."MessageId", mr."RevisionNumber", mr."CreatedTime", mr."Author", mr."Summary", mr."ModerationState"
FROM "#message" m
JOIN "#message_revision" mr USING ("MessageId")
WHERE mr."MessageId" = ? AND
(? OR
m."Owner" = ?)
ORDER BY mr."RevisionNumber"';
$parameters = array(
$messageId,
$isReadAllowed || $isMsgAccessException,
UnbSession::GetSessionUserId());
$db = UnbDatabase::GetInstance();
$recs = $db->Query($sql, $parameters);
if (!isset($recs)) return null;
$revs = array();
foreach ($recs as $rec)
{
$contentLevel = UnbMessageRevisionContentLevel::Subject;
if ($rec['ModerationState'] != UnbMessageRevisionModerationState::Approved &&
!UnbSession::IsModerator() &&
UnbSession::GetSessionUserId() != intval($rec['Author']))
{
$rec['Summary'] = null;
$contentLevel = UnbMessageRevisionContentLevel::NoContent;
}
$rev = self::FromRecord($rec);
$rev->contentLevel = $contentLevel;
$revs[] = $rev;
}
return $revs;
}
public function GetContentLevel()
{
return $this->contentLevel;
}
public function GetMessageId()
{
return $this->messageId;
}
public function GetRevisionNumber()
{
return $this->revisionNumber;
}
public function GetCreatedTime()
{
return $this->createdTime;
}
public function GetAuthorUserId()
{
return $this->author;
}
public function GetAuthorUser()
{
if (isset($this->authorUser))
return $this->authorUser;
$this->authorUser = UnbUser::FindByUserId($this->author);
return $this->authorUser;
}
public function GetSubject()
{
if ($this->contentLevel < UnbMessageRevisionContentLevel::Subject)
throw new UnbSecurityException('This revision was not retrieved with an appropriate content level to get the subject.');
return $this->subject;
}
public function GetContent()
{
if ($this->contentLevel < UnbMessageRevisionContentLevel::Content)
throw new UnbSecurityException('This revision was not retrieved with an appropriate content level to get the content.');
return $this->content;
}
public function GetHtmlContent($autoGenerate = false, $markup = null)
{
if ($this->contentLevel < UnbMessageRevisionContentLevel::HtmlContent)
throw new UnbSecurityException('This revision was not retrieved with an appropriate content level to get the HTML content.');
if (!isset($this->htmlContent) && $autoGenerate)
{
if ($this->GetContent() !== null)
{
$html = $markup->Convert($this->GetContent());
}
else
{
$html = '';
}
try
{
$this->SetHtmlContent($html);
}
catch (Exception $ex)
{
}
}
return $this->htmlContent;
}
public function SetHtmlContent($htmlContent)
{
if (isset($htmlContent))
{
if (!is_string($htmlContent))
throw new InvalidArgumentException('Invalid argument type: $htmlContent');
$htmlContent = trim($htmlContent);
if (mb_strlen($htmlContent) > 16777215)
throw new LengthException('[#3] HTML content is too long');
if (!strlen($htmlContent))
$htmlContent = null;
}
$this->db->StartTransaction();
try
{
$this->db->Execute(
'UPDATE "#message_revision" SET "HtmlContent" = ? WHERE "MessageId" = ? AND "RevisionNumber" = ?',
array($htmlContent, $this->messageId, $this->revisionNumber));
$this->db->Commit();
}
catch (Exception $ex)
{
$db->RollBack();
throw $ex;
}
$this->htmlContent = $htmlContent;
}
public function GetSummary()
{
if ($this->contentLevel < UnbMessageRevisionContentLevel::Subject)
throw new UnbSecurityException('This revision was not retrieved with an appropriate content level to get the summary.');
return $this->summary;
}
public function HasFileData()
{
return $this->hasData;
}
public function GetFileDataLength($searchMode = 0)
{
if (!$this->hasData) return 0;
if ($searchMode == 0)
{
$val = $this->db->QueryValue(
'SELECT LENGTH("Data") FROM "#message_revision"
WHERE "MessageId" = ? AND "RevisionNumber" = ?',
array($this->messageId, $this->revisionNumber));
}
if (!isset($val))
{
$filename = UnbConfiguration::Get('path.file-uploads', '.') . '/' . $this->messageId . '_' . $this->revisionNumber;
if (file_exists($filename))
{
$val = filesize($filename);
}
else
{
if ($searchMode == 1)
{
$val = $this->db->QueryValue(
'SELECT LENGTH("Data") FROM "#message_revision"
WHERE "MessageId" = ? AND "RevisionNumber" = ?',
array($this->messageId, $this->revisionNumber));
}
if (!isset($val))
throw new Exception('[#9] Attached data not found.');
}
}
return $val;
}
public function GetFileDataPath()
{
if (!$this->hasData) return null;
$filename = UnbConfiguration::Get('path.file-uploads', '.') . '/' . $this->messageId . '_' . $this->revisionNumber;
if (file_exists($filename))
{
return $filename;
}
return null;
}
public function GetFileData($searchMode = 0)
{
if ($this->contentLevel < UnbMessageRevisionContentLevel::Content)
throw new UnbSecurityException('This revision was not retrieved with an appropriate content level to get the file data.');
if (!$this->hasData)
{
if ($searchMode == 3) return false;
return null;
}
if ($searchMode == 0 || $searchMode == 2)
{
$val = $this->db->QueryValue(
'SELECT "Data" FROM "#message_revision"
WHERE "MessageId" = ? AND "RevisionNumber" = ?',
array($this->messageId, $this->revisionNumber));
}
if (!isset($val))
{
if ($searchMode == 2) return null;
$filename = UnbConfiguration::Get('path.file-uploads', '.') . '/' . $this->messageId . '_' . $this->revisionNumber;
if (file_exists($filename))
{
if ($searchMode == 3)
return readfile($filename) !== false;
$val = file_get_contents($filename);
}
else
{
if ($searchMode == 3)
return false;
if ($searchMode == 1)
{
$val = $this->db->QueryValue(
'SELECT "Data" FROM "#message_revision"
WHERE "MessageId" = ? AND "RevisionNumber" = ?',
array($this->messageId, $this->revisionNumber));
}
if (!isset($val))
throw new Exception('[#9] Attached data not found.');
}
}
return $val;
}
public function GetModerationState()
{
return $this->moderationState;
}
public function SetModerationState($moderationState)
{
if (!is_int($moderationState))
throw new InvalidArgumentException('Invalid argument type: $moderationState');
if ($moderationState < 0)
throw new InvalidArgumentException('Argument out of range: $moderationState');
if (!UnbSession::IsModerator())
throw new UnbSecurityException('You need the moderator privilege to set the moderation state.');
$this->db->StartTransaction(true);
try
{
$this->db->Execute(
'UPDATE "#message_revision" SET "ModerationState" = ? WHERE "MessageId" = ? AND "RevisionNumber" = ?',
array($moderationState, $this->messageId, $this->revisionNumber));
UnbMessage::UpdateSearchFields($this->messageId);
$this->db->Commit();
}
catch (Exception $ex)
{
$db->RollBack();
throw $ex;
}
$this->moderationState = $moderationState;
}
public function Delete()
{
if (!UnbSession::IsModerator())
throw new UnbSecurityException('You need the moderator privilege to delete a message revision.');
$this->db->StartTransaction();
try
{
$this->db->Execute(
'DELETE FROM "#message_revision" WHERE "MessageId" = ? AND "RevisionNumber" = ?',
array($this->messageId, $this->revisionNumber));
$dataFilename = $this->GetFileDataPath();
if (file_exists($dataFilename))
if (!unlink($dataFilename))
throw new Exception('[#10] The attached file could not be deleted.');
$this->db->Commit();
}
catch (Exception $ex)
{
$this->db->RollBack();
throw $ex;
}
}
public function GetTags($onlyVisible = true, $names = null)
{
if (isset($names))
{
if (!is_array($names))
throw new InvalidArgumentException('Invalid argument type: $names');
}
$sql = 'SELECT t."TagId" AS "TagId", t.*, mrt."TagData"
FROM "#message_revision_tag" mrt
JOIN "#tag" t USING ("TagId")
WHERE mrt."MessageId" = ? AND mrt."RevisionNumber" = ?';
$parameters = array($this->messageId, $this->revisionNumber);
if ($onlyVisible)
$sql .= ' AND NOT (t.Flags & ' . UnbTagFlag::Invisible . ')';
if (isset($names))
{
$sql .= ' AND (';
$count = 0;
foreach ($names as $name)
{
if (!is_string($name))
throw new InvalidArgumentException('Invalid argument type: $names');
if ($count++)
$sql .= ' OR ';
$sql .= 't.TagName = ?';
$parameters[] = $name;
}
$sql .= ')';
}
$order = UnbTag::GetHierarchicalOrder();
if (count($order))
{
$sql .= ' ORDER BY CASE t."TagId"';
foreach ($order as $index => $tagId)
$sql .= ' WHEN ' . $tagId . ' THEN ' . $index;
$sql .= ' END';
}
$recs = $this->db->Query($sql, $parameters);
$tags = array();
foreach ($recs as $rec)
{
if (isset($names))
{
$tag = UnbTag::FromRecord($rec);
$tags[$tag->GetTagName()] = $tag;
}
else
{
$tags[] = UnbTag::FromRecord($rec);
}
}
return $tags;
}
public function GetTagIds($withData = true)
{
$recs = $this->db->Query(
'SELECT "TagId", "TagData" FROM "#message_revision_tag" WHERE "MessageId" = ? AND "RevisionNumber" = ?',
array($this->messageId, $this->revisionNumber));
$tags = array();
foreach ($recs as $rec)
{
$rec['TagId'] = intval($rec['TagId']);
if ($withData && $rec['TagData'] != null)
{
$tags[] = $rec;
}
else
{
$tags[] = $rec['TagId'];
}
}
return $tags;
}
public function AddTag($tagId, $tagData = null)
{
if ($tagId instanceof UnbTag)
$tagId = $tagId->GetTagId();
if (!is_int($tagId))
throw new InvalidArgumentException('Invalid argument type: $tagId');
if ($tagId <= 0)
throw new InvalidArgumentException('Argument out of range: $tagId');
if (isset($tagData))
{
if (!is_string($tagData))
throw new InvalidArgumentException('Invalid argument type: $tagData');
$tagData = trim($tagData);
if (mb_strlen($tagData) > 255)
throw new LengthException('[#11] Tag data is too long');
}
if (UnbSession::GetSessionUserId() != $this->author && !UnbSession::IsModerator())
throw new UnbSecurityException('You must be logged in as this user or need the moderator privilege to add a tag.');
if (!UnbTag::IsUseAllowedById($tagId))
throw new UnbSecurityException('You are not allowed to use this tag.');
UnbMessage::ClearAccessCache();
$this->db->StartTransaction();
try
{
$this->db->Execute(
'UPDATE "#message_revision_tag"
SET "TagData" = ?
WHERE "MessageId" = ? AND "RevisionNumber" = ? AND "TagId" = ?',
array($tagData, $this->messageId, $this->revisionNumber, $tagId));
if ($this->db->GetRowsAffected() == 0)
{
try
{
$this->db->Execute(
'INSERT INTO "#message_revision_tag" ("MessageId", "RevisionNumber", "TagId", "TagData") VALUES (?, ?, ?, ?)',
array($this->messageId, $this->revisionNumber, $tagId, $tagData));
}
catch (UnbDatabaseException $ex)
{
if (!$this->db->IsConstraintError()) throw $ex;
}
}
$this->db->Commit();
}
catch (Exception $ex)
{
$this->db->RollBack();
throw $ex;
}
}
public function RemoveTag($tagId)
{
if ($tagId instanceof UnbTag)
$tagId = $tagId->GetTagId();
if (!is_int($tagId))
throw new InvalidArgumentException('Invalid argument type: $tagId');
if ($tagId <= 0)
throw new InvalidArgumentException('Argument out of range: $tagId');
if (UnbSession::GetSessionUserId() != $this->author && !UnbSession::IsModerator())
throw new UnbSecurityException('You must be logged in as this user or need the moderator privilege to remove a tag.');
UnbMessage::ClearAccessCache();
$this->db->StartTransaction();
try
{
$this->db->Execute(
'DELETE FROM "#message_revision_tag" WHERE "MessageId" = ? AND "RevisionNumber" = ? AND "TagId" = ?',
array($this->messageId, $this->revisionNumber, $tagId));
$this->db->Commit();
}
catch (Exception $ex)
{
$this->db->RollBack();
throw $ex;
}
}
}
class UnbSession
{
const SessionName = '';
const SessionIpNetMask = 0xFFFFFF00;
const SessionExpireTimeout = 60;
const CookieDomain = '';
const CookiePath = '';
const SessionArrayVar = 'UNB';
const DefaultEncryptionKey = 'UNB2_Key_1234567890';
private static $sessionUser = null;
private function __construct()
{
}
private static function InitialiseSession()
{
$_SESSION[self::SessionArrayVar] = array();
$_SESSION[self::SessionArrayVar]['IpAddress'] = ip2long($_SERVER['REMOTE_ADDR']);
$_SESSION[self::SessionArrayVar]['HeartBeat'] = time();
}
private static function GetCookieDomain()
{
$cookieDomain = trim(self::CookieDomain);
if (!strlen($cookieDomain))
{
$cookieDomain = '';
}
if (strlen($cookieDomain) && $cookieDomain{0} != '.')
$cookieDomain = '.' . $cookieDomain;
return $cookieDomain;
}
private static function GetCookiePath()
{
$cookiePath = trim(self::CookiePath);
if (!strlen($cookiePath))
{
$cookiePath = dirname($_SERVER['PHP_SELF']);
}
if (substr($cookiePath, -1) != '/')
$cookiePath .= '/';
return $cookiePath;
}
private static function StartPhpSession($sessionId = null)
{
if (!session_id() || isset($sessionId))
{
$sessionname = self::SessionName;
if (!is_string($sessionname) || trim($sessionname) === '')
{
$sessionname = 'unb_session_' . substr(md5(dirname(__FILE__)), 0, 16);
}
$sid = $_REQUEST[$sessionname];
if (!preg_match('/[a-f0-9-,]{0,42}/i', $sid))
throw new InvalidArgumentException('Invalid session ID value');
ini_set('url_rewriter.tags', '');
ini_set('session.use_trans_sid', '0');
ini_set('session.cookie_domain', self::GetCookieDomain());
ini_set('session.cookie_path', self::GetCookiePath());
session_name($sessionname);
if (isset($sessionId))
session_id($sessionId);
session_start();
@header('Pragma: ');
@header('Cache-Control: cache');
}
}
public static function EnableSession($sessionId = null)
{
if (!is_string(self::SessionArrayVar) || trim(self::SessionArrayVar) === '')
throw new InvalidArgumentException('Invalid argument type: SessionArrayVar');
self::StartPhpSession($sessionId);
if (!isset($_SESSION[self::SessionArrayVar]))
{
self::InitialiseSession();
}
$ip = ip2long($_SERVER['REMOTE_ADDR']);
if ($ip & self::SessionIpNetMask != $_SESSION[self::SessionArrayVar]['IpAddress'] & self::SessionIpNetMask)
{
self::InitialiseSession();
}
if (self::SessionExpireTimeout > 0 &&
$_SESSION[self::SessionArrayVar]['HeartBeat'] < time() - self::SessionExpireTimeout * 60)
{
self::InitialiseSession();
}
$_SESSION[self::SessionArrayVar]['HeartBeat'] = time();
if (isset($_SESSION[self::SessionArrayVar]['SessionUserId']))
{
if (!is_int($_SESSION[self::SessionArrayVar]['SessionUserId']))
{
if (is_numeric($_SESSION[self::SessionArrayVar]['SessionUserId']))
{
$_SESSION[self::SessionArrayVar]['SessionUserId'] = intval($_SESSION[self::SessionArrayVar]['SessionUserId']);
}
else
{
$_SESSION[self::SessionArrayVar]['SessionUserId'] = null;
}
}
if ($_SESSION[self::SessionArrayVar]['SessionUserId'] > 0)
{
try
{
self::$sessionUser = UnbUser::FindByUserId($_SESSION[self::SessionArrayVar]['SessionUserId']);
}
catch (Exception $ex)
{
self::$sessionUser = null;
}
}
if (!isset(self::$sessionUser))
{
$_SESSION[self::SessionArrayVar]['SessionUserId'] = null;
}
}
if (!isset(self::$sessionUser))
{
list($userId, $pCookie) = explode(' ', $_COOKIE['unb_autologin']);
if (isset($userId))
{
$userId = intval($userId);
try
{
$user = UnbUser::FindByUserId($userId);
if (isset($user))
{
$pStored = self::Decrypt($user->GetSingleSetting('autologin.password'));
if ($pCookie !== $pStored)
throw new Exception();
self::SetSessionUser($user, null, true);
}
}
catch (Exception $ex)
{
}
}
}
}
public static function DeleteSession()
{
StartPhpSession();
$_SESSION[self::SessionArrayVar] = null;
$_SESSION = array();
session_destroy();
}
public static function SetVar($name, $value = null)
{
$_SESSION[self::SessionArrayVar]['Var'][$name] = $value;
}
public static function GetVar($name, $unset = false)
{
$value = $_SESSION[self::SessionArrayVar]['Var'][$name];
if ($unset)
$_SESSION[self::SessionArrayVar]['Var'][$name] = null;
return $value;
}
public static function SetCookie($name, $value = null, $expire = -1)
{
if (!is_string($name))
throw new InvalidArgumentException('Invalid argument type: $name');
$name = trim($name);
if (!strlen($name))
throw new InvalidArgumentException('Name is empty');
if (!isset($value))
$value = '';
if (!is_string($value))
throw new InvalidArgumentException('Invalid argument type: $value');
$value = trim($value);
if (!is_int($expire))
throw new InvalidArgumentException('Invalid argument type: $expire');
if (!strlen($value))
$expire = 1;
elseif ($expire == -1)
$expire = intval(pow(2, 31) - 1);
setcookie($name, $value, $expire, self::GetCookiePath(), self::GetCookieDomain());
}
public static function GetSessionUser()
{
if (!session_id())
throw new BadMethodCallException('Session not enabled');
return self::$sessionUser;
}
public static function GetSessionUserId()
{
if (!session_id())
throw new BadMethodCallException('Session not enabled');
if (isset(self::$sessionUser)) return self::$sessionUser->GetUserId();
return 0;
}
public static function IsAdministrator()
{
if (!session_id())
throw new BadMethodCallException('Session not enabled');
if (!isset(self::$sessionUser)) return false;
return self::$sessionUser->IsAdministrator();
}
public static function IsModerator()
{
if (!session_id())
throw new BadMethodCallException('Session not enabled');
if (!isset(self::$sessionUser)) return false;
return self::$sessionUser->IsModerator();
}
public static function IsTrusted()
{
if (!session_id())
throw new BadMethodCallException('Session not enabled');
if (!isset(self::$sessionUser)) return false;
return self::$sessionUser->IsTrusted();
}
public static function IsLoggedIn()
{
if (!session_id())
throw new BadMethodCallException('Session not enabled');
return isset(self::$sessionUser);
}
public static function SetSessionUser($sessionUser, $password = null, $overridePassword = false)
{
if (!session_id())
throw new BadMethodCallException('Session not enabled');
if (isset($sessionUser))
{
if (!($sessionUser instanceof UnbUser))
throw new InvalidArgumentException('Invalid argument type: $sessionUser');
if ((!isset($password) || $password === '') && self::IsAdministrator() ||
$overridePassword)
{
}
else
{
if (!is_string($password))
throw new InvalidArgumentException('Invalid argument type: $password');
$password = trim($password);
$sessionUserPassword = $sessionUser->GetLoginPassword();
if (!isset($sessionUserPassword))
throw new UnbSecurityException('[#12] You cannot log in without a password.');
if (!self::ValidatePassword($password, $sessionUserPassword))
throw new UnbSecurityException('[#13] The login password you provided was rejected.');
}
}
$_SESSION[self::SessionArrayVar]['SessionUserId'] = isset($sessionUser) ? $sessionUser->GetUserId() : 0;
self::$sessionUser = $sessionUser;
if (isset($sessionUser))
$sessionUser->UpdateLastLoginTime();
}
public static function ConvertPassword($password)
{
$salt = '';
for ($i = 0; $i < 4; $i++) $salt .= chr(mt_rand(0, 255));
return base64_encode($salt . hash('sha256', $salt . $password, true));
}
private static function ValidatePassword($password, $hash)
{
$hash = base64_decode($hash);
$salt = substr($hash, 0, 4);
return $hash == $salt . hash('sha256', $salt . $password, true);
}
public static function ValidateSessionUserPassword($password)
{
if (isset($password))
{
if (!is_string($password))
throw new InvalidArgumentException('Invalid argument type: $password');
$password = trim($password);
if (!strlen($password))
$password = null;
}
if (!self::IsLoggedIn())
throw new BadMethodCallException('Not logged in');
$sessionUserPassword = self::GetSessionUser()->GetLoginPassword();
if (!isset($sessionUserPassword) && !isset($password))
return true;
return self::ValidatePassword($password, $sessionUserPassword);
}
public static function GeneratePassword($length = 12)
{
$p = '';
while ($length--)
$p .= chr(mt_rand(0x21, 0x7E));
return $p;
}
public static function Encrypt($data)
{
if (!isset($data)) return null;
$td = mcrypt_module_open(MCRYPT_TWOFISH, '', MCRYPT_MODE_ECB, '');
$iv = mcrypt_create_iv(mcrypt_enc_get_iv_size($td), MCRYPT_RAND);
$key = UnbConfiguration::Get('encryption.key');
if (!isset($key) || !strlen($key))
$key = self::DefaultEncryptionKey;
$key = substr($key, 0, mcrypt_enc_get_key_size($td));
mcrypt_generic_init($td, $key, $iv);
$encrypted_data = mcrypt_generic($td, $data);
mcrypt_generic_deinit($td);
mcrypt_module_close($td);
return base64_encode($encrypted_data);
}
public static function Decrypt($data)
{
if (!isset($data)) return null;
$td = mcrypt_module_open(MCRYPT_TWOFISH, '', MCRYPT_MODE_ECB, '');
$iv = mcrypt_create_iv(mcrypt_enc_get_iv_size($td), MCRYPT_RAND);
$key = UnbConfiguration::Get('encryption.key');
if (!isset($key) || !strlen($key))
$key = self::DefaultEncryptionKey;
$key = substr($key, 0, mcrypt_enc_get_key_size($td));
mcrypt_generic_init($td, $key, $iv);
$decrypted_data = mdecrypt_generic($td, base64_decode($data));
mcrypt_generic_deinit($td);
mcrypt_module_close($td);
return rtrim($decrypted_data, "\0");
}
public static function GetUrlKey()
{
if (!session_id())
throw new BadMethodCallException('Session not enabled');
return sprintf('%08X', crc32(session_id()));
}
public static function CheckUrlKey($key)
{
if (!session_id())
throw new BadMethodCallException('Session not enabled');
return $key == sprintf('%08X', crc32(session_id()));
}
}
class UnbEnvironment
{
private static $browserName = null;
private static $browserVersion = null;
private static $browserClass = null;
private static $isBrowser = null;
private static $searchTerms = null;
private function __construct()
{
}
public static function SetupEnvironment()
{
self::SetupErrorReporting();
self::SetupMultibyte();
self::SetupMagicQuotesRuntime();
self::SetupMagicQuotesGPC();
self::SetupNull();
self::SetupRegisterGlobals();
self::EnableHttpCompression();
}
public static function SetupErrorReporting()
{
error_reporting(E_ALL & ~E_NOTICE);
}
public static function SetupMultibyte()
{
mb_internal_encoding('UTF-8');
}
public static function SetupMagicQuotesRuntime()
{
set_magic_quotes_runtime(0);
}
public static function SetupMagicQuotesGPC()
{
if (get_magic_quotes_gpc())
{
self::DeSlashArray($_GET);
self::DeSlashArray($_POST);
self::DeSlashArray($_REQUEST);
self::DeSlashArray($_COOKIE);
self::DeSlashArray($_FILES);
}
}
public static function SetupNull()
{
self::DeNulArray($_GET);
self::DeNulArray($_POST);
self::DeNulArray($_REQUEST);
self::DeNulArray($_COOKIE);
self::DeNulArray($_FILES);
}
public static function SetupRegisterGlobals()
{
if (ini_get('register_globals'))
{
if (sizeof($_SESSION)) foreach (array_keys($_SESSION) as $key) unset($GLOBALS[$key]);
if (sizeof($_GET)) foreach (array_keys($_GET) as $key) unset($GLOBALS[$key]);
if (sizeof($_POST)) foreach (array_keys($_POST) as $key) unset($GLOBALS[$key]);
if (sizeof($_COOKIE)) foreach (array_keys($_COOKIE) as $key) unset($GLOBALS[$key]);
if (sizeof($_SERVER)) foreach (array_keys($_SERVER) as $key) unset($GLOBALS[$key]);
if (is_array($_GET['GLOBALS'])) foreach (array_keys($_GET['GLOBALS']) as $key) unset($GLOBALS[$key]);
if (is_array($_POST['GLOBALS'])) foreach (array_keys($_POST['GLOBALS']) as $key) unset($GLOBALS[$key]);
}
}
public static function EnableHttpCompression()
{
if (!ini_get('zlib.output_compression') && !ob_get_level())
ob_start('ob_gzhandler');
if (!ob_get_level())
ob_start();
}
private static function DeSlashArray(&$a)
{
foreach ($a as $key => $value)
{
if (strpos($key, '\\') !== false)
{
unset($a[$key]);
continue;
}
if (is_string($value))
{
$a[$key] = stripslashes($value);
}
elseif (is_array($value))
{
self::DeSlashArray($a[$key]);
}
}
}
private static function DeNulArray(&$a)
{
foreach ($a as $key => $value)
{
if (strpos($key, "\x00") !== false)
{
unset($a[$key]);
continue;
}
if (is_string($value))
{
$a[$key] = str_replace(array("\x00", "\r"), array('', ''), $value);
}
elseif (is_array($value))
{
self::DeNulArray($a[$key]);
}
}
}
public static function GetReferer()
{
return isset($_SERVER['HTTP_REFERER']) ? $_SERVER['HTTP_REFERER'] : '';
}
public static function IsRefererMyDomain()
{
$referer = (isset($_SERVER['HTTP_REFERER']) ? $_SERVER['HTTP_REFERER'] : '');
$ref_mydomain = false;
if (preg_match('_^https?://(.*?)/_i', $referer, $m))
{
$referer = $m[1];
$me = $_SERVER['SERVER_NAME'];
if (!strcasecmp(substr($me, 0, 4), 'www.')) $me = substr($me, 4);
if (!strcasecmp(substr($referer, 0, 4), 'www.')) $referer = substr($referer, 4);
if (!strcasecmp($me, $referer)) $ref_mydomain = true;
}
return $ref_mydomain;
}
public static function GetRemoteAddress()
{
return $_SERVER['REMOTE_ADDR'];
}
public static function GetClientLanguage()
{
$languageCodes = array();
$priorities = array();
$acceptLanguage = $_SERVER['HTTP_ACCEPT_LANGUAGE'];
while (preg_match('/^\s*([a-z]{2})(?:[-_]([a-z]{2}))?(?:;\s*q\s*=\s*(\d+(?:\.\d+)?))?(?:\s*,\s*(.*))?$/i', $acceptLanguage, $m))
{
$acceptLanguage = $m[4];
$languageCodes[] = strtolower($m[1]) . ($m[2] ? '_' . strtoupper($m[2]) : '');
$priorities[] = $m[3] ? intval(floatval($m[3]) * 100) : 100;
}
array_multisort($priorities, SORT_DESC, SORT_NUMERIC, $languageCodes);
return $languageCodes;
}
private static function DetectBrowser()
{
$names = array(
'Slurp' => 'search',
'ZyBorg' => 'search',
'Ask Jeeves' => 'search',
'Opera' => 'opera',
'iCab' => 'icab',
'MSIE' => 'ie',
'Firebird' => 'fx',
'Firefox' => 'fx',
'Minefield' => 'fx',
'Thunderbird' => 'fx',
'Galeon' => 'galeon',
'Chrome' => 'chrome',
'Safari' => 'safari',
'Konqueror' => 'konq',
'Camino' => 'cam',
'Gecko' => 'moz',
'Netscape' => 'nn',
'Lynx' => 'text',
'ELinks' => 'text',
'Links' => 'text',
'w3m' => 'text',
'Googlebot' => 'search',
'Mediapartners-Google' => 'search',
'msnbot' => 'search',
'ConveraMultiMediaCrawler' => 'search',
'Spider' => 'search',
'Scooter' => 'search',
'WebCrawler' => 'search',
'crawler' => 'search',
'WiseCrawler' => 'search',
'LinkWalker' => 'search',
'Nutch' => 'search',
'robot' => 'bot',
'Indy Library' => 'bot',
'Jetbot' => 'bot',
'psbot' => 'bot',
'ia_archiver' => 'bot',
'Python-urllib' => 'bot',
'Gigabot' => 'bot',
'Firefly' => 'bot',
'ichiro' => 'bot',
'lwp-trivial' => 'bot',
'LWP::Simple' => 'bot',
'bot' => 'bot',
'Wget' => 'download',
'DA 4' => 'download',
'NetAnts' => 'download',
'Mass Downloader' => 'download',
'GetRight' => 'download',
'PHP' => 'bot',
'Mozilla' => 'nn'
);
$raw_ua = $_SERVER['HTTP_USER_AGENT'];
foreach ($names as $ua => $name)
{
if (stristr($raw_ua, $ua))
{
self::$browserName = $name;
if ($ua == 'Gecko') $ua = 'rv:';
elseif ($ua == 'Netscape') $ua .= '[0-9]?[ /]';
else $ua .= '[ /]';
if (preg_match('~' . $ua . '([0-9.]+)~', $raw_ua, $m)) self::$browserVersion = $m[1];
break;
}
}
self::$isBrowser = true;
if (self::$browserName == 'search' ||
self::$browserName == 'bot' ||
self::$browserName == 'download' ||
self::$browserName == 'php')
{
self::$isBrowser = false;
}
$classes = array(
'ie' => array('ie'),
'opera' => array('opera'),
'gecko' => array('moz', 'fx', 'cam'),
'webkit' => array('chrome', 'konq', 'safari')
);
foreach ($classes as $c => $browsers)
{
if (in_array(self::$browserName, $browsers))
{
self::$browserClass = $c;
break;
}
}
}
public static function GetBrowserName()
{
if (!isset(self::$browserName))
self::DetectBrowser();
return self::$browserName;
}
public static function GetBrowserVersion()
{
if (!isset(self::$browserVersion))
self::DetectBrowser();
return self::$browserVersion;
}
public static function GetBrowserClass()
{
if (!isset(self::$browserClass))
self::DetectBrowser();
return self::$browserClass;
}
public static function IsBrowser()
{
if (!isset(self::$isBrowser))
self::DetectBrowser();
return self::$isBrowser;
}
public static function BestXHtml()
{
$accept = explode(',', $_SERVER['HTTP_ACCEPT']);
foreach ($accept as $h => $v) { $e = explode(';', $v); $accept[$h] = $e[0]; }
if (in_array('application/xhtml+xml', $accept)) return 'application/xhtml+xml';
elseif (in_array('text/xml', $accept)) return 'text/xml';
elseif (in_array('text/html', $accept)) return 'text/html';
return 'text/html';
}
public static function UseXml()
{
return strstr(self::BestXHtml(), 'xml') != false;
}
public static function GetClientSystemName()
{
$raw_ua = $_SERVER['HTTP_USER_AGENT'];
if (stristr($raw_ua, 'Windows 95')) return 'win95';
elseif (stristr($raw_ua, 'Win95')) return 'win95';
elseif (stristr($raw_ua, 'Windows 98')) return 'win98';
elseif (stristr($raw_ua, 'Win98')) return 'win95';
elseif (stristr($raw_ua, 'Windows ME')) return 'winme';
elseif (stristr($raw_ua, 'Windows NT 4')) return 'winnt';
elseif (stristr($raw_ua, 'Windows NT 5.0')) return 'win2k';
elseif (stristr($raw_ua, 'Windows NT 5.1')) return 'winxp';
elseif (stristr($raw_ua, 'Windows NT 5.2')) return 'win2k3';
elseif (stristr($raw_ua, 'Windows NT 6.0')) return 'winvista';
elseif (stristr($raw_ua, 'Windows NT 6.1')) return 'win7';
elseif (stristr($raw_ua, 'WinNT')) return 'winnt';
elseif (stristr($raw_ua, 'Windows-NT')) return 'winnt';
elseif (stristr($raw_ua, 'Windows NT')) return 'winnt';
elseif (stristr($raw_ua, 'Win')) return 'win';
elseif (stristr($raw_ua, 'Linux')) return 'linux';
elseif (stristr($raw_ua, 'Unix')) return 'unix';
elseif (stristr($raw_ua, 'Mac')) return 'mac';
elseif (stristr($raw_ua, 'SunOS')) return 'sun';
return '';
}
public static function GetClientSystemClass()
{
$systemName = self::GetClientSystemName();
if (strpos($systemName, 'win') !== false) return 'win';
if (strpos($systemName, 'mac') !== false) return 'mac';
if (strpos($systemName, 'linux') !== false) return 'unix';
if (strpos($systemName, 'unix') !== false) return 'unix';
if (strpos($systemName, 'sun') !== false) return 'unix';
return '';
}
public static function GetSearchTerms()
{
if (!isset(self::$searchTerms))
{
self::$searchTerms = array();
$url = parse_url(self::GetReferer());
if (preg_match('_google\.[a-z]{2,3}(?:\.[a-z]{2,3})?$_i', $url['host']) && $url['path'] == '/search')
{
parse_str($url['query'], $arr);
$q = $arr['q'];
if (get_magic_quotes_gpc()) $q = stripslashes($q);
$q = str_replace("\x00", '', $q);
$q = explode(' ', $q);
foreach ($q as $a)
{
$a = trim($a);
if ($a != '') $searchTerms[] = $a;
}
}
}
return self::$searchTerms;
}
public static function GetServerSystemType()
{
if (substr(strtoupper(PHP_OS), 0, 3) == 'WIN') return 'win';
if (substr(strtoupper(PHP_OS), 0, 5) == 'LINUX') return 'linux';
return '';
}
}
class UnbSearch
{
private function __construct()
{
}
private static function ConstructSqlQuery($searchParameters)
{
$sessionUserId = UnbSession::GetSessionUserId();
$isModerator = UnbSession::IsModerator();
$isAdministrator = UnbSession::IsAdministrator();
if (is_int($searchParameters['owner-ids']))
$searchParameters['owner-ids'] = array($searchParameters['owner-ids']);
$getTagData = intval($searchParameters['get-tag-data']);
$parameters = array();
$sql = '';
if ($getTagData > 0)
$sql .= 'SELECT mrtGTD."TagData" AS "TagData", COUNT(*) AS "Count"';
else
$sql .= 'SELECT m."MessageId"';
$sql .= ' FROM "#message" m';
$sortTagId = intval($searchParameters['sort-tagid']);
if ($sortTagId > 0)
{
$sql .= ' JOIN "#message_revision_tag" mrt
ON mrt."MessageId" = m."MessageId" AND mrt."RevisionNumber" = m."SearchRevision" AND mrt."TagId" = ?';
$parameters[] = $sortTagId;
}
if (is_array($searchParameters['tag-value']))
{
foreach ($searchParameters['tag-value'] as $tagId => $value)
{
if (!is_numeric($tagId)) continue;
$sql .= ' JOIN "#message_revision_tag" mrTV' . $tagId . '
ON mrTV' . $tagId . '."MessageId" = m."MessageId" AND
mrTV' . $tagId . '."RevisionNumber" = m."SearchRevision" AND
mrTV' . $tagId . '."TagId" = ?';
$parameters[] = $tagId;
}
}
if ($getTagData > 0)
{
$sql .= ' JOIN "#message_revision_tag" mrtGTD ON mrtGTD."MessageId" = m."MessageId" AND mrtGTD."RevisionNumber" = m."SearchRevision" AND mrtGTD."TagId" = ?';
$parameters[] = $getTagData;
}
$sql .= '
WHERE
-- Grant all access to administrators (so that they can alter the access key lists)
(? OR
-- Message has no read access keylist
(m."ReadAccessKeylistId" IS NULL
OR EXISTS
-- Message\'s read access keylist contains personal or additional user key
(SELECT 1
FROM "#message" m2, "#keylist" m2k
WHERE m2k."KeylistId" = m2."ReadAccessKeylistId" AND
m2."MessageId" = m."MessageId" AND
(m2k."UserId" = ? OR
m2k."UserId" IN
(SELECT uk."UserId"
FROM "#user" u, "#keylist" uk
WHERE uk."KeylistId" = u."AdditionalKeylistId" AND
u."UserId" = ?))))';
$parameters[] = $isAdministrator;
$parameters[] = $sessionUserId;
$parameters[] = $sessionUserId;
$dbDriver = UnbDatabase::GetInstance()->GetDriverName();
if ($dbDriver != 'pgsql')
{
$sql .= '
AND NOT EXISTS
-- There must not be any tag that denies access
(SELECT
EXISTS
-- Tag\'s read access keylist contains personal or additional user key
(SELECT 1
FROM "#keylist" tk
WHERE tk."KeylistId" = t."ReadAccessKeylistId" AND
(tk."UserId" = ? OR
tk."UserId" IN
(SELECT uk."UserId"
FROM "#user" u, "#keylist" uk
WHERE uk."KeylistId" = u."AdditionalKeylistId" AND
u."UserId" = ?)))
AS "Allowed"
FROM "#message_revision_tag" mrt
JOIN "#tag" t USING ("TagId")
WHERE mrt."MessageId" = m."MessageId" AND
mrt."RevisionNumber" = m."SearchRevision" AND
t."ReadAccessKeylistId" IS NOT NULL   -- Only regard tags with read access keylist (only they can fail the access test)';
if ($dbDriver == 'sqlite')
$sql .= '
GROUP BY 1';
$sql .= '
HAVING NOT "Allowed")';
}
else
{
$sql .= '
AND NOT EXISTS
-- There must not be any tag that denies access
(SELECT "Allowed"
FROM
(SELECT
EXISTS
-- Tag\'s read access keylist contains personal or additional user key
(SELECT 1
FROM "#keylist" tk
WHERE tk."KeylistId" = t."ReadAccessKeylistId" AND
(tk."UserId" = ? OR
tk."UserId" IN
(SELECT uk."UserId"
FROM "#user" u, "#keylist" uk
WHERE uk."KeylistId" = u."AdditionalKeylistId" AND
u."UserId" = ?)))
AS "Allowed"
FROM "#message" m2
JOIN "#message_revision_tag" mrt
ON mrt."MessageId" = m2."MessageId" AND mrt."RevisionNumber" = m2."SearchRevision"
JOIN "#tag" t USING ("TagId")
WHERE m2."MessageId" = m."MessageId" AND
-- Only regard tags with read access keylist (only they can fail the access test)
t."ReadAccessKeylistId" IS NOT NULL) AS foo
WHERE NOT "Allowed")';
}
$parameters[] = $sessionUserId;
$parameters[] = $sessionUserId;
$sql .= ')';
if (!$isModerator && !$isAdministrator)
{
$sql .= '
AND (NOT (m."ModerationFlags" & ' . UnbMessageModerationFlag::Hidden . ') OR
m."Owner" = ?)';
$parameters[] = $sessionUserId;
}
if (is_array($searchParameters['message-ids']) && count($searchParameters['message-ids']))
{
$messageIds = array_map('intval', $searchParameters['message-ids']);
$sql .= '
AND m."MessageId" IN (' . join(', ', $messageIds) . ')';
}
if (is_array($searchParameters['owner-ids']) && count($searchParameters['owner-ids']))
{
$ownerIds = array_map('intval', $searchParameters['owner-ids']);
$sql .= '
AND m."Owner" ' .
($searchParameters['owner-ids-inverse'] ? 'NOT ' : '') .
'IN (' . join(', ', $ownerIds) . ')';
}
if (is_bool($searchParameters['entrypoint']))
{
if ($searchParameters['entrypoint'] === true)
$sql .= '
AND m."EntryPoint"';
elseif ($searchParameters['entrypoint'] === false)
$sql .= '
AND NOT m."EntryPoint"';
}
if (is_int($searchParameters['primary-reference']))
{
if ($searchParameters['primary-reference'] > 0)
{
$sql .= '
AND m."PrimaryReference" = ?';
$parameters[] = $searchParameters['primary-reference'];
}
else
{
$sql .= '
AND m."PrimaryReference" IS NULL';
}
}
if (is_array($searchParameters['tags-set']) && count($searchParameters['tags-set']))
{
foreach ($searchParameters['tags-set'] as $tagId)
{
$sql .= '
AND ' . ($tagId < 0 ? 'NOT ' : '') . 'EXISTS
(SELECT 1
FROM "#message_revision_tag" mrt
WHERE mrt."MessageId" = m."MessageId" AND
mrt."RevisionNumber" = m."SearchRevision" AND
mrt."TagId" = ?)';
$parameters[] = abs($tagId);
}
}
if (is_array($searchParameters['tag-value']))
{
foreach ($searchParameters['tag-value'] as $tagId => $value)
{
if (!is_numeric($tagId)) continue;
$sql .= ' AND mrTV' . $tagId . '."TagData" ';
if (is_array($value))
{
$sql .= 'BETWEEN ? AND ?';
$parameters[] = $value[0];
$parameters[] = $value[1];
}
elseif (strpos($value, '*') !== false || strpos($value, '?') !== false)
{
$sql .= 'LIKE ? ESCAPE ?';
$parameters[] = UnbDatabase::EscapeLikeWildcards($value);
$parameters[] = '\\';
}
else
{
$sql .= '= ?';
$parameters[] = $value;
}
}
}
if ($searchParameters['createdtime-notbefore'])
{
$sql .= '
AND "CreatedTime" >= ?';
$time = $searchParameters['createdtime-notbefore'];
if (is_int($time))
$time = UnbDatabase::ConvertDateTime($time);
$parameters[] = $time;
}
if ($searchParameters['createdtime-notafter'])
{
$sql .= '
AND "CreatedTime" <= ?';
$time = $searchParameters['createdtime-notafter'];
if (is_int($time))
$time = UnbDatabase::ConvertDateTime($time);
$parameters[] = $time;
}
if ($getTagData > 0)
{
$sql .= ' GROUP BY mrtGTD."TagData"';
}
if ($sortTagId > 0)
{
$sql .= ' ORDER BY mrt."TagData"';
if ($searchParameters['sort-reverse'])
$sql .= ' DESC';
}
elseif ($searchParameters['sort-createdtime'])
{
$sql .= ' ORDER BY "CreatedTime"';
if ($searchParameters['sort-reverse'])
$sql .= ' DESC';
}
if (preg_match('_^\d+(\s*,\s*\d+)?$_', $searchParameters['limit']))
{
$sql .= ' LIMIT ' . $searchParameters['limit'];
}
return array($sql, $parameters);
}
public static function Query($query = null)
{
$searchParameters = array();
if (is_array($query))
$searchParameters = $query;
list($sql, $parameters) = self::ConstructSqlQuery($searchParameters);
$db = UnbDatabase::GetInstance();
$messageIds = $db->QueryArray($sql, $parameters);
if (!isset($messageIds)) return null;
$messageIds = array_map('intval', $messageIds);
UnbMessage::SetReadAllowedIds($messageIds);
return $messageIds;
}
public static function GetAllReplies($messageId)
{
$out = array();
self::GetAllReplies_rec(0, $messageId, array(), $out);
return $out;
}
private static function GetAllReplies_rec($primaryRef, $messageId, $seenMessageIds, &$out)
{
if (!is_int($messageId))
throw new InvalidArgumentException('Invalid argument type: $messageId');
if ($messageId <= 0)
throw new InvalidArgumentException('Argument out of range: $messageId');
if ($primaryRef)
$out[$messageId] = $primaryRef;
if (in_array($messageId, $seenMessageIds))
{
return;
}
$seenMessageIds[] = $messageId;
$searchParameters = array();
$searchParameters['primary-reference'] = $messageId;
$messageIds = UnbSearch::Query($searchParameters);
if ($messageIds)
{
foreach ($messageIds as $replyMessageId)
{
self::GetAllReplies_rec($messageId, $replyMessageId, $seenMessageIds, $out);
}
}
}
public static function GetAllTagValues($tagId, $query = null)
{
$searchParameters = array();
if (is_array($query))
$searchParameters = $query;
$searchParameters['get-tag-data'] = $tagId;
$searchParameters['sort-tagid'] = $tagId;
list($sql, $parameters) = self::ConstructSqlQuery($searchParameters);
$db = UnbDatabase::GetInstance();
$records = $db->Query($sql, $parameters);
if (!isset($records)) return null;
$a = array();
foreach ($records as $rec)
$a[$rec['TagData']] = intval($rec['Count']);
return $a;
}
}
class UnbMarkup
{
const STYLE_LEFT   = 0;
const STYLE_RIGHT  = 1;
const STYLE_CENTRE = 2;
const STYLE_BOX    = 3;
const STYLE_OT     = 4;
const STYLE_PAR    = 5;
const STYLE_EM     = 6;
const STYLE_STRONG = 7;
const STYLE_MONO   = 8;
const STYLE_COLOUR = 9;
const STYLE_BACKC  = 10;
const STYLE_SIZE   = 11;
const STYLE_BIG    = 12;
const STYLE_SMALL  = 13;
const STYLE_ULINE  = 14;
const STYLE_STRIKE = 15;
const STYLE_SUB    = 16;
const STYLE_SUP    = 17;
const STYLE_RAW    = 18;
const STYLE_ID_MAX = 18;
private $dst = '';
private $actualStyle = array();
private $desiredStyle = array();
private $styleStack = array();
private $listType = '';
private $quoteLevel = 0;
private $tableLine = false;
private $tableColumns = 0;
private $maxTableColumns = 0;
private $smileyPath = '';
private $smileyImages = array();
private $smileyRegexps = '';
private $smileyStarts = '';
private $smileyStartsRegexpCC = '';
public function __construct()
{
$this->smileyPath = 'img/smileys/';
$this->smileyImages = array(
':@'  => array('file' => 'angry.png',       'width' => 11, 'height' => 11),
':-@' => array('file' => 'angry.png',       'width' => 11, 'height' => 11),
'|('  => array('file' => 'annoyed.png',     'width' => 11, 'height' => 11),
'|-(' => array('file' => 'annoyed.png',     'width' => 11, 'height' => 11),
':$'  => array('file' => 'blush.png',       'width' => 11, 'height' => 11),
':-$' => array('file' => 'blush.png',       'width' => 11, 'height' => 11),
'B)'  => array('file' => 'cool.png',        'width' => 11, 'height' => 11),
'B-)' => array('file' => 'cool.png',        'width' => 11, 'height' => 11),
';('  => array('file' => 'cry.png',         'width' => 11, 'height' => 11),
';-(' => array('file' => 'cry.png',         'width' => 11, 'height' => 11),
":'(" => array('file' => 'cry.png',         'width' => 11, 'height' => 11),
':D'  => array('file' => 'grin.png',        'width' => 11, 'height' => 11),
':-D' => array('file' => 'grin.png',        'width' => 11, 'height' => 11),
'^^'  => array('file' => 'happy.png',       'width' => 11, 'height' => 11),
'^_^' => array('file' => 'happy.png',       'width' => 11, 'height' => 11),
'?('  => array('file' => 'huh.png',         'width' => 11, 'height' => 11),
'?-(' => array('file' => 'huh.png',         'width' => 11, 'height' => 11),
':|'  => array('file' => 'indifferent.png', 'width' => 11, 'height' => 11),
':-|' => array('file' => 'indifferent.png', 'width' => 11, 'height' => 11),
':p'  => array('file' => 'razz.png',        'width' => 11, 'height' => 11),
':-p' => array('file' => 'razz.png',        'width' => 11, 'height' => 11),
'8)'  => array('file' => 'rolleyes.png',    'width' => 11, 'height' => 11),
'8-)' => array('file' => 'rolleyes.png',    'width' => 11, 'height' => 11),
':('  => array('file' => 'sad.png',         'width' => 11, 'height' => 11),
':-(' => array('file' => 'sad.png',         'width' => 11, 'height' => 11),
':O'  => array('file' => 'scared.png',      'width' => 11, 'height' => 11),
':-O' => array('file' => 'scared.png',      'width' => 11, 'height' => 11),
':/'  => array('file' => 'sceptic.png',     'width' => 11, 'height' => 11),
':-/' => array('file' => 'sceptic.png',     'width' => 11, 'height' => 11),
'8('  => array('file' => 'shocked.png',     'width' => 11, 'height' => 11),
'8-(' => array('file' => 'shocked.png',     'width' => 11, 'height' => 11),
':)'  => array('file' => 'smile.png',       'width' => 11, 'height' => 11),
':-)' => array('file' => 'smile.png',       'width' => 11, 'height' => 11),
';)'  => array('file' => 'wink.png',        'width' => 11, 'height' => 11),
';-)' => array('file' => 'wink.png',        'width' => 11, 'height' => 11),
'|O'  => array('file' => 'yawn.png',        'width' => 11, 'height' => 11),
'|-O' => array('file' => 'yawn.png',        'width' => 11, 'height' => 11),
);
$this->smileyRegexps = '';
$this->smileyStarts = '';
$this->smileyStartsRegexpCC = '';
foreach ($this->smileyImages as $code => $image)
{
if (strpos($this->smileyStarts, $code{0}) === false)
{
$this->smileyStarts .= $code{0};
$this->smileyStartsRegexpCC .=
str_replace(
array('\\',   '-',  '[',  ']',  '_',  '^'),
array('\\\\', '\-', '\[', '\]', '\_', '\^'),
$code{0});
}
$this->smileyRegexps .= ($this->smileyRegexps ? '|' : '') .
str_replace(
array('\\',   '$',  '(',  ')',  '*',  '+',  '.',  '?',  '[',  ']',  '^',  '_',  '|'),
array('\\\\', '\$', '\(', '\)', '\*', '\+', '\.', '\?', '\[', '\]', '\^', '\_', '\|'),
$code);
}
}
public function SetSmileyPath($path)
{
if (substr($path, -1) != '/') $path .= '/';
$this->smileyPath = $path;
}
private function ResetState()
{
$this->dst = '';
$this->actualStyle = array();
$this->desiredStyle = array();
$this->styleStack = array();
$this->listType = '';
$this->quoteLevel = 0;
$this->tableLine = false;
$this->tableColumns = 0;
$this->maxTableColumns = 0;
}
public function Convert($src)
{
$this->ResetState();
$lines = explode("\n", $src);
foreach ($lines as $line)
{
$this->ProcessLine(rtrim($line));
}
$this->ClearStyle(true);
$this->ApplyStyle();
$dst = $this->dst;
$this->ResetState();
return $dst;
}
public function PostProcess($src)
{
return $src;
}
private function ProcessLine($line, $inQuote = false)
{
if ($line{0} === '=' &&
preg_match('_^=(=+) (.*) =\1$_', $line, $m) &&
!$this->desiredStyle[self::STYLE_PAR] &&
!$this->desiredStyle[self::STYLE_RAW])
{
$level = strlen($m[1]);
$caption = trim($m[2]);
$level = min(6, $level);
$this->UpdateList('');
$this->UpdateTable(false);
if (!$inQuote) $this->UpdateBlockQuote(0);
$this->dst .= '<h' . $level . '>';
$this->ProcessInline($caption, true);
$this->ClearStyle();
$this->ApplyStyle();
$this->dst .= '</h' . $level . '>' . "\n";
}
else if (($line{0} === "\t" || $line{0} === ' ') &&
preg_match('_^(?:(\t+)|(  +))_', $line, $m) &&
!$this->desiredStyle[self::STYLE_PAR] &&
!$this->desiredStyle[self::STYLE_RAW])
{
$level = strlen($m[1]);
if (!$level)
$level = intval(round(strlen($m[2]) / 4.0));
if ($this->listType != '')
{
$this->dst .= '<br />' . "\n";
}
else
{
$this->UpdateTable(false);
if (!$inQuote) $this->UpdateBlockQuote(0);
$this->desiredStyle[self::STYLE_PAR] = $level;
$this->ApplyStyle();
}
$this->ProcessInline(ltrim($line));
}
else if ($line{0} === '>' &&
preg_match('_^(>(?: ?>)*)(| (.*))$_', $line, $m) &&
!$this->desiredStyle[self::STYLE_RAW] &&
!$inQuote)
{
$level = strlen(str_replace(' ', '', $m[1]));
$content = $m[3];
if ($level != $this->quoteLevel)
{
if ($this->desiredStyle[self::STYLE_PAR])
{
$this->desiredStyle[self::STYLE_PAR] = false;
$this->ApplyStyle();
}
$this->UpdateList('');
$this->UpdateTable(false);
}
$this->UpdateBlockQuote($level);
$this->ProcessLine($content, true);
}
else if ($line{0} === '-' &&
preg_match('_^----+$_', $line, $m) &&
!$this->desiredStyle[self::STYLE_RAW])
{
$this->UpdateList('');
if (!$inQuote) $this->UpdateBlockQuote(0);
$this->ClearStyle();
$this->ApplyStyle();
if ($this->tableLine && $this->maxTableColumns > 0)
{
$this->dst .= '<tr>' . "\n";
$this->dst .= '<td colspan="' . $this->maxTableColumns . '"><div style="border-bottom: solid 1px black;"></div></td>' . "\n";
$this->dst .= '</tr>' . "\n";
}
else
{
$this->dst .= '<div style="border-bottom: solid 1px black;"></div>' . "\n";
}
}
else if (($line{0} === '#' || $line{0} === '*') &&
preg_match('_^([#*][#* \t]*[ \t])(.*)$_', $line, $m) &&
!$this->desiredStyle[self::STYLE_RAW])
{
$type = str_replace(array("\t", ' '), array('', ''), $m[1]);
$content = $m[2];
$this->UpdateTable(false);
if (!$inQuote) $this->UpdateBlockQuote(0);
if ($this->desiredStyle[self::STYLE_PAR])
{
$this->desiredStyle[self::STYLE_PAR] = false;
$this->ApplyStyle();
}
$this->UpdateList($type);
$this->ProcessInline($content);
}
else if ($line{0} === '|' &&
preg_match('_^\|(.*)\|$_', $line, $m) &&
!$this->desiredStyle[self::STYLE_RAW])
{
$content = $m[1];
$this->UpdateTable(true, $inQuote);
$this->tableColumns = 1;
$this->dst .= '<tr>' . "\n";
$this->dst .= '<td>';
$this->ProcessInline($content);
$this->dst .= '</td>' . "\n";
$this->dst .= '</tr>' . "\n";
if ($this->tableColumns > $this->maxTableColumns)
{
$this->maxTableColumns = $this->tableColumns;
}
$this->tableColumns = 0;
}
else if (!strlen($line) &&
!$this->desiredStyle[self::STYLE_RAW])
{
$this->UpdateList('');
$this->UpdateTable(false);
if (!$inQuote) $this->UpdateBlockQuote(0);
$this->ClearStyle();
$this->ApplyStyle();
}
else
{
$this->UpdateList('');
$this->UpdateTable(false);
if (!$inQuote) $this->UpdateBlockQuote(0);
if (!$this->desiredStyle[self::STYLE_PAR])
{
$this->desiredStyle[self::STYLE_PAR] = true;
}
$this->ProcessInline($line);
if (!$this->actualStyle[self::STYLE_PAR])
{
$this->desiredStyle[self::STYLE_PAR] = false;
}
if ($this->desiredStyle[self::STYLE_RAW])
{
$this->dst .= '<br />';
}
$this->dst .= "\n";
}
}
private function ProcessInline($line, $onlyBasic = false)
{
for ($i = 0; $i < strlen($line); $i++)
{
if (!$this->desiredStyle[self::STYLE_RAW])
{
if (preg_match("_['{}\[|<" . $this->smileyStartsRegexpCC . "0-9]|(?i)https?:|ftp:|www\.|mailto:_", $line, $m, PREG_OFFSET_CAPTURE, $i))
{
$nextIndex = $m[0][1];
if ($nextIndex != $i)
{
$this->ApplyStyle();
$this->dst .= htmlspecialchars(substr($line, $i, $nextIndex - $i));
$i = $nextIndex;
}
}
else
{
$this->ApplyStyle();
$this->dst .= htmlspecialchars(substr($line, $i));
break;
}
}
$unprocessed = false;
if (!$this->desiredStyle[self::STYLE_RAW])
{
if ($line{$i} === "'" &&
preg_match("_\\G'('+)_", $line, $m, 0, $i))
{
$level = strlen($m[1]);
$level = min(3, $level);
if ($level & 1)
$this->desiredStyle[self::STYLE_EM] = !$this->desiredStyle[self::STYLE_EM];
if ($level & 2)
$this->desiredStyle[self::STYLE_STRONG] = !$this->desiredStyle[self::STYLE_STRONG];
$i += strlen($m[1]);
}
else if (!$onlyBasic &&
$line{$i} === '{' &&
preg_match('_\G{{{_', $line, $m, 0, $i))
{
$this->desiredStyle[self::STYLE_RAW] = true;
$i += 2;
}
else if ($line{$i} === '{' &&
!$this->desiredStyle[self::STYLE_MONO] &&
preg_match('_\G{{_', $line, $m, 0, $i))
{
$this->desiredStyle[self::STYLE_MONO] = true;
$i += 1;
}
else if ($line{$i} === '}' &&
$this->desiredStyle[self::STYLE_MONO] &&
preg_match('_\G}}_', $line, $m, 0, $i))
{
$this->desiredStyle[self::STYLE_MONO] = false;
$i += 1;
}
else if ($line{$i} === '[' &&
preg_match('_\G\[\[=(\S+?)\]\]_i', $line, $m, 0, $i))
{
$anchor = trim($m[1]);
$this->dst .= '<a id="' . htmlspecialchars($anchor) . '"></a>';
$i += strlen($m[1]) + 5 - 1;
}
else if (!$onlyBasic &&
$line{$i} === '[' &&
preg_match('_\G\[\[(.*?)\]\]([0-9a-z]*)_i', $line, $m, 0, $i))
{
$wordExtra = $m[2];
$parts = explode(' ', trim($m[1]), 2);
$link = $parts[0];
$caption = trim($parts[1]);
$this->ApplyStyle();
if (preg_match('_^(https?|ftp)://[-.0-9a-z]+/[^?]+\.(bmp|gif|ico|jp2|jpg|jpeg|png|svg)$_i', urldecode($link)))
{
if (!strlen($caption))
$caption = $link;
$this->dst .= '<a href="' . htmlspecialchars($link) . '"><img src="' . htmlspecialchars($link) . '" alt="' . htmlspecialchars($caption) . '" title="' . htmlspecialchars($caption) . '" /></a>';
$i += strlen($m[1]) + 4 - 1;
}
else
{
$resLink = $this->ResolveLink($link, $caption);
if ($resLink)
{
$link = $resLink[0];
$caption = $resLink[1];
$useWordExtra = $resLink[2];
$rawCaption = $resLink[3];
if ($useWordExtra && $wordExtra != '')
{
$caption .= $wordExtra;
}
else
{
$wordExtra = '';
}
$this->dst .= '<a href="' . htmlspecialchars($link) . '">';
if ($rawCaption)
{
$this->dst .= $caption;
}
else
{
$backupActualStyle = $this->actualStyle;
$backupDesiredStyle = $this->desiredStyle;
$backupStyleStack = $this->styleStack;
$this->actualStyle = array();
$this->desiredStyle = array();
$this->styleStack = array();
$this->ProcessInline($caption, true);
$this->ClearStyle();
$this->ApplyStyle();
$this->actualStyle = $backupActualStyle;
$this->desiredStyle = $backupDesiredStyle;
$this->styleStack = $backupStyleStack;
}
$this->dst .= '</a>';
$i += strlen($m[1]) + 4 - 1 + strlen($wordExtra);
}
else
{
$this->dst .= '[';
}
}
}
else if (!$onlyBasic &&
$line{$i} === '|' &&
$this->tableLine /*&&
preg_match('_\G\|_', $line, $m, 0, $i)*/)
{
$this->ClearStyle();
$this->ApplyStyle();
$this->tableColumns++;
$this->ApplyStyle();
$this->dst .= '</td>' . "\n" . '<td>';
}
else if (!$onlyBasic &&
$line{$i} === '<' &&
!$this->desiredStyle[self::STYLE_LEFT] &&
!$this->desiredStyle[self::STYLE_RIGHT] &&
preg_match('_\G(<left>)_i', $line, $m, 0, $i))
{
$this->desiredStyle[self::STYLE_LEFT] = true;
$i += strlen($m[1]) - 1;
}
else if ($line{$i} === '<' &&
$this->desiredStyle[self::STYLE_LEFT] &&
preg_match('_\G(</left>)_i', $line, $m, 0, $i))
{
$this->ClearStyle();
$this->desiredStyle[self::STYLE_LEFT] = false;
$i += strlen($m[1]) - 1;
}
else if (!$onlyBasic &&
$line{$i} === '<' &&
!$this->desiredStyle[self::STYLE_LEFT] &&
!$this->desiredStyle[self::STYLE_RIGHT] &&
preg_match('_\G(<right>)_i', $line, $m, 0, $i))
{
$this->desiredStyle[self::STYLE_RIGHT] = true;
$i += strlen($m[1]) - 1;
}
else if ($line{$i} === '<' &&
$this->desiredStyle[self::STYLE_RIGHT] &&
preg_match('_\G(</right>)_i', $line, $m, 0, $i))
{
$this->ClearStyle();
$this->desiredStyle[self::STYLE_RIGHT] = false;
$i += strlen($m[1]) - 1;
}
else if (!$onlyBasic &&
$line{$i} === '<' &&
!$this->desiredStyle[self::STYLE_CENTRE] &&
preg_match('_\G(<cent(er|re)>)_i', $line, $m, 0, $i))
{
$this->desiredStyle[self::STYLE_CENTRE] = true;
$i += strlen($m[1]) - 1;
}
else if ($line{$i} === '<' &&
$this->desiredStyle[self::STYLE_CENTRE] &&
preg_match('_\G(</cent(er|re)>)_i', $line, $m, 0, $i))
{
$this->ClearStyle();
$this->desiredStyle[self::STYLE_CENTRE] = false;
$i += strlen($m[1]) - 1;
}
else if (!$onlyBasic &&
$line{$i} === '<' &&
!$this->desiredStyle[self::STYLE_BOX] &&
preg_match('_\G(<box>)_i', $line, $m, 0, $i))
{
$this->desiredStyle[self::STYLE_BOX] = true;
$i += strlen($m[1]) - 1;
}
else if ($line{$i} === '<' &&
$this->desiredStyle[self::STYLE_BOX] &&
preg_match('_\G(</box>)_i', $line, $m, 0, $i))
{
$this->ClearStyle();
$this->desiredStyle[self::STYLE_BOX] = false;
$i += strlen($m[1]) - 1;
}
else if (!$onlyBasic &&
$line{$i} === '<' &&
!$this->desiredStyle[self::STYLE_OT] &&
preg_match('_\G(<(?:ot|offtopic)>)_i', $line, $m, 0, $i))
{
$this->desiredStyle[self::STYLE_OT] = true;
$i += strlen($m[1]) - 1;
}
else if ($line{$i} === '<' &&
$this->desiredStyle[self::STYLE_OT] &&
preg_match('_\G(</(?:ot|offtopic)>)_i', $line, $m, 0, $i))
{
$this->ClearStyle();
$this->desiredStyle[self::STYLE_OT] = false;
$i += strlen($m[1]) - 1;
}
else if (!$onlyBasic &&
$line{$i} === '<' &&
!$this->desiredStyle[self::STYLE_COLOUR] &&
preg_match('_\G(<colou?r\s+(#[0-9a-f]+|[a-z]+)>)_i', $line, $m, 0, $i))
{
$colour = $m[2];
$this->desiredStyle[self::STYLE_COLOUR] = $colour;
$i += strlen($m[1]) - 1;
}
else if ($line{$i} === '<' &&
$this->desiredStyle[self::STYLE_COLOUR] &&
preg_match('_\G(</colou?r>)_i', $line, $m, 0, $i))
{
$this->desiredStyle[self::STYLE_COLOUR] = false;
$i += strlen($m[1]) - 1;
}
else if (!$onlyBasic &&
$line{$i} === '<' &&
!$this->desiredStyle[self::STYLE_BACKC] &&
preg_match('_\G(<back(?:colou?r|ground)?\s+(#[0-9a-f]+|[a-z]+)>)_i', $line, $m, 0, $i))
{
$colour = $m[2];
$this->desiredStyle[self::STYLE_BACKC] = $colour;
$i += strlen($m[1]) - 1;
}
else if ($line{$i} === '<' &&
$this->desiredStyle[self::STYLE_BACKC] &&
preg_match('_\G(</back(?:colou?r|ground)?>)_i', $line, $m, 0, $i))
{
$this->desiredStyle[self::STYLE_BACKC] = false;
$i += strlen($m[1]) - 1;
}
else if (!$onlyBasic &&
$line{$i} === '<' &&
!$this->desiredStyle[self::STYLE_SIZE] &&
preg_match('_\G(<size\s+([0-9]+)>)_i', $line, $m, 0, $i))
{
$size = intval($m[2]) / 10.0;
$this->desiredStyle[self::STYLE_SIZE] = $size;
$i += strlen($m[1]) - 1;
}
else if ($line{$i} === '<' &&
$this->desiredStyle[self::STYLE_SIZE] &&
preg_match('_\G(</size>)_i', $line, $m, 0, $i))
{
$this->desiredStyle[self::STYLE_SIZE] = false;
$i += strlen($m[1]) - 1;
}
else if ($line{$i} === '<' &&
!$this->desiredStyle[self::STYLE_BIG] &&
preg_match('_\G(<big>)_i', $line, $m, 0, $i))
{
$this->desiredStyle[self::STYLE_BIG] = true;
$i += strlen($m[1]) - 1;
}
else if ($line{$i} === '<' &&
$this->desiredStyle[self::STYLE_BIG] &&
preg_match('_\G(</big>)_i', $line, $m, 0, $i))
{
$this->desiredStyle[self::STYLE_BIG] = false;
$i += strlen($m[1]) - 1;
}
else if ($line{$i} === '<' &&
!$this->desiredStyle[self::STYLE_SMALL] &&
preg_match('_\G(<small>)_i', $line, $m, 0, $i))
{
$this->desiredStyle[self::STYLE_SMALL] = true;
$i += strlen($m[1]) - 1;
}
else if ($line{$i} === '<' &&
$this->desiredStyle[self::STYLE_SMALL] &&
preg_match('_\G(</small>)_i', $line, $m, 0, $i))
{
$this->desiredStyle[self::STYLE_SMALL] = false;
$i += strlen($m[1]) - 1;
}
else if ($line{$i} === '<' &&
!$this->desiredStyle[self::STYLE_ULINE] &&
preg_match('_\G(<u>)_i', $line, $m, 0, $i))
{
$this->desiredStyle[self::STYLE_ULINE] = true;
$i += strlen($m[1]) - 1;
}
else if ($line{$i} === '<' &&
$this->desiredStyle[self::STYLE_ULINE] &&
preg_match('_\G(</u>)_i', $line, $m, 0, $i))
{
$this->desiredStyle[self::STYLE_ULINE] = false;
$i += strlen($m[1]) - 1;
}
else if ($line{$i} === '<' &&
!$this->desiredStyle[self::STYLE_STRIKE] &&
preg_match('_\G(<s>)_i', $line, $m, 0, $i))
{
$this->desiredStyle[self::STYLE_STRIKE] = true;
$i += strlen($m[1]) - 1;
}
else if ($line{$i} === '<' &&
$this->desiredStyle[self::STYLE_STRIKE] &&
preg_match('_\G(</s>)_i', $line, $m, 0, $i))
{
$this->desiredStyle[self::STYLE_STRIKE] = false;
$i += strlen($m[1]) - 1;
}
else if (!$onlyBasic &&
$line{$i} === '<' &&
!$this->desiredStyle[self::STYLE_SUB] &&
preg_match('_\G(<sub>)_i', $line, $m, 0, $i))
{
$this->desiredStyle[self::STYLE_SUB] = true;
$i += strlen($m[1]) - 1;
}
else if ($line{$i} === '<' &&
$this->desiredStyle[self::STYLE_SUB] &&
preg_match('_\G(</sub>)_i', $line, $m, 0, $i))
{
$this->desiredStyle[self::STYLE_SUB] = false;
$i += strlen($m[1]) - 1;
}
else if (!$onlyBasic &&
$line{$i} === '<' &&
!$this->desiredStyle[self::STYLE_SUP] &&
preg_match('_\G(<sup>)_i', $line, $m, 0, $i))
{
$this->desiredStyle[self::STYLE_SUP] = true;
$i += strlen($m[1]) - 1;
}
else if ($line{$i} === '<' &&
$this->desiredStyle[self::STYLE_SUP] &&
preg_match('_\G(</sup>)_i', $line, $m, 0, $i))
{
$this->desiredStyle[self::STYLE_SUP] = false;
$i += strlen($m[1]) - 1;
}
else if ($line{$i} === '<' &&
preg_match('_\G(<br\s*/?>)_i', $line, $m, 0, $i))
{
$this->ApplyStyle();
$this->dst .= '<br />';
$i += strlen($m[1]) - 1;
}
else if (!$onlyBasic &&
(strpos('hfwHFW', $line{$i}) !== false) &&
preg_match('_\G(?<=[ \t]|^)
(
( (https?|ftp):// ([-$%+,.0-9=a-z\_~]*(:[-$%+,.0-9=a-z\_~]*)?@)? | www\. )
[0-9a-z]([-.0-9a-z]*[0-9a-z])?
(/[-!#$%&()+,./0-9:;=?@a-z\[\]\_~]*)?
)
(?=[ \t!"),.:;?]|$)
_ix', $line, $m, 0, $i))
{
$m[1] = preg_replace('_[!"),.:;?]$_', '', $m[1]);
$link = $m[1];
$caption = htmlspecialchars($link, ENT_NOQUOTES);
$caption = preg_replace('_(/(?!/)|\?|&amp;)_', '\1<wbr />', $caption);
if (!preg_match('_^(https?|ftp):_i', $link))
{
$link = 'http://' . $link;
}
$this->ApplyStyle();
$this->dst .= '<a href="' . htmlspecialchars($link) . '">' . $caption . '</a>';
$i += strlen($m[1]) - 1;
}
else if (!$onlyBasic &&
(strpos('mM', $line{$i}) !== false) &&
preg_match('_\G(?<=[ \t]|^)
(
mailto:
[!#$%&()*+,\-.0-9:;=?a-z\[\]^\_~]+
@
[0-9a-z]([-.0-9a-z]*[0-9a-z])?
(
\?(to|cc|bcc|subject|body)=[!-%\'->@-~]*
(&(to|cc|bcc|subject|body)=[!-%\'->@-~]*)*
)?
)
(?=[ \t!"),.:;?]|$)
_ix', $line, $m, 0, $i))
{
$m[1] = preg_replace('_[!"),.:;?]$_', '', $m[1]);
$link = $m[1];
$caption = htmlspecialchars($link, ENT_NOQUOTES);
$caption = preg_replace('_(mailto:|\.|@|\?|&amp;|%20|%0A)_i', '\1<wbr />', $caption);
$this->ApplyStyle();
$this->dst .= '<a href="' . htmlspecialchars($link) . '">' . $caption . '</a>';
$i += strlen($m[1]) - 1;
}
else if (($code = $this->CheckSmiley($line, $i)))
{
$this->ApplyStyle();
$this->TranslateSmiley($code);
$i += strlen($code) - 1;
}
else if ((strpos('0123456789', $line{$i}) !== false) &&
preg_match('_([0-9]+([., ][0-9]+)* )[^0-9., ]_', $line, $m, 0, $i))
{
$s = htmlspecialchars($m[1], ENT_NOQUOTES);
if (strlen($m[1]) <= 12)
{
$s = str_replace(' ', '&nbsp;', $s);
}
$this->ApplyStyle();
$this->dst .= $s;
$i += strlen($m[1]) - 1;
}
else
{
$unprocessed = true;
}
}
else // if ($this->desiredStyle[self::STYLE_RAW])
{
if ($line{$i} == '}' &&
preg_match('_\G}}}(}*)_', $line, $m, 0, $i))
{
$add = strlen($m[1]);
if ($add == 0)
{
$this->desiredStyle[self::STYLE_RAW] = false;
$i += 2;
}
else
{
$this->ApplyStyle();
$this->dst .= '}}}' . str_repeat('}', $add - 1);
$i += 2 + $add;
}
}
else
{
$unprocessed = true;
}
}
if ($unprocessed)
{
$this->ApplyStyle();
if ($this->desiredStyle[self::STYLE_RAW] &&
$line{$i} == "\t")
{
$this->dst .= '&nbsp; &nbsp; ';
}
else if ($this->desiredStyle[self::STYLE_RAW] &&
$line{$i} == ' ' &&
($i == 0 || $line{$i - 1} == ' ' && substr($this->dst, -1) == ' '))
{
$this->dst .= '&nbsp;';
}
else
{
$this->dst .= htmlspecialchars($line{$i}, ENT_NOQUOTES);
}
}
}
}
private function ClearStyle($all = false)
{
if ($all)
{
$this->desiredStyle = array();
}
else
{
$keepStyle = array();
$keepStyle[self::STYLE_LEFT] = $this->desiredStyle[self::STYLE_LEFT];
$keepStyle[self::STYLE_RIGHT] = $this->desiredStyle[self::STYLE_RIGHT];
$keepStyle[self::STYLE_CENTRE] = $this->desiredStyle[self::STYLE_CENTRE];
$keepStyle[self::STYLE_BOX] = $this->desiredStyle[self::STYLE_BOX];
$keepStyle[self::STYLE_OT] = $this->desiredStyle[self::STYLE_OT];
$keepStyle[self::STYLE_RAW] = $this->desiredStyle[self::STYLE_RAW];
$this->desiredStyle = $keepStyle;
}
}
private function ApplyStyle()
{
$closeFromStackPos = -1;
for ($x = 0; $x < count($this->styleStack); $x++)
{
$styleId = $this->styleStack[$x];
if ($this->actualStyle[$styleId] != $this->desiredStyle[$styleId])
{
$closeFromStackPos = $x;
break;
}
}
if ($closeFromStackPos >= 0)
{
while (count($this->styleStack) > $closeFromStackPos)
{
$styleId = array_pop($this->styleStack);
$data = $this->desiredStyle[$styleId];
$this->dst .= $this->TranslateStyle($styleId, false, $data);
$this->actualStyle[$styleId] = false;
}
}
for ($styleId = 0; $styleId <= self::STYLE_ID_MAX; $styleId++)
{
if ($this->desiredStyle[$styleId] && !$this->actualStyle[$styleId])
{
$data = $this->desiredStyle[$styleId];
$this->dst .= $this->TranslateStyle($styleId, true, $data);
$this->actualStyle[$styleId] = $data;
array_push($this->styleStack, $styleId);
}
}
}
private function TranslateStyle($styleId, $openOrClose, $data = null)
{
switch ($styleId)
{
case self::STYLE_LEFT:
return $openOrClose ? '<div class="left-align">' : '</div>';
case self::STYLE_RIGHT:
return $openOrClose ? '<div class="right-align">' : '</div>';
case self::STYLE_CENTRE:
return $openOrClose ? '<div class="centre-align">' : '</div>';
case self::STYLE_BOX:
return $openOrClose ? '<div class="bordered-box">' : '</div>';
case self::STYLE_OT:
return $openOrClose ? '<div class="off-topic"><div class="caption">Off Topic:</div>' : '</div>';
case self::STYLE_PAR:
if ($openOrClose)
{
if (is_int($data) && $data > 0)
{
return '<p style="margin-left: ' . ($data * 2) . 'em;">';
}
else
{
return '<p>';
}
}
else
{
return '</p>';
}
case self::STYLE_EM:
return $openOrClose ? '<em>' : '</em>';
case self::STYLE_STRONG:
return $openOrClose ? '<strong>' : '</strong>';
case self::STYLE_MONO:
return $openOrClose ? '<tt>' : '</tt>';
case self::STYLE_COLOUR:
if ($openOrClose)
{
if (is_string($data) && $data != '')
{
return '<span style="color: ' . htmlspecialchars($data) . ';">';
}
else
{
return '<span>';
}
}
else
{
return '</span>';
}
case self::STYLE_BACKC:
if ($openOrClose)
{
if (is_string($data) && $data != '')
{
return '<span style="background-color: ' . htmlspecialchars($data) . ';">';
}
else
{
return '<span>';
}
}
else
{
return '</span>';
}
case self::STYLE_SIZE:
if ($openOrClose)
{
if (is_float($data) && $data > 0)
{
return '<span style="font-size: ' . $data . 'em;">';
}
else
{
return '<span>';
}
}
else
{
return '</span>';
}
case self::STYLE_BIG:
return $openOrClose ? '<big>' : '</big>';
case self::STYLE_SMALL:
return $openOrClose ? '<small>' : '</small>';
case self::STYLE_ULINE:
return $openOrClose ? '<u>' : '</u>';
case self::STYLE_STRIKE:
return $openOrClose ? '<s>' : '</s>';
case self::STYLE_SUB:
return $openOrClose ? '<sub>' : '</sub>';
case self::STYLE_SUP:
return $openOrClose ? '<sup>' : '</sup>';
case self::STYLE_RAW:
return $openOrClose ? '<code>' : '</code>';
}
}
private function CheckSmiley($line, $i)
{
if (strpos($this->smileyStarts, $line{$i}) !== false &&
preg_match('_\G(?<=[ \t]|^)(' . $this->smileyRegexps . ')(?=[ \t!"),.:;?]|$)_', $line, $m, 0, $i))
{
return $m[1];
}
return false;
}
private function TranslateSmiley($code)
{
$image = $this->smileyImages[$code];
if ($image)
{
$this->dst .= '<img src="' . $this->smileyPath . htmlspecialchars($image['file']) . '" style="width: ' . $image['width'] . 'px; height: ' . $image['height'] . 'px;" alt="' . htmlspecialchars($code) . '" />';
}
else
{
$this->dst .= htmlspecialchars($code, ENT_NOQUOTES);
}
}
private function UpdateList($type)
{
$firstDiff = -2;
for ($i = 0; $i < min(strlen($this->listType), strlen($type)); $i++)
{
if ($this->listType{$i} != $type{$i})
{
$firstDiff = $i;
break;
}
}
if ($firstDiff == -2)
{
if (strlen($this->listType) == strlen($type))
$firstDiff = -1;
else
$firstDiff = $i;
}
if ($type != '' && $firstDiff == -1)
{
$this->ClearStyle();
$this->ApplyStyle();
$this->dst .= '</li>' . "\n" . '<li>';
}
else if ($firstDiff >= 0)
{
$origLastListType = $this->listType;
while (strlen($this->listType) > $firstDiff)
{
$lastMode = substr($this->listType, -1);
if ($lastMode == '*')
{
$this->ClearStyle();
$this->ApplyStyle();
$this->dst .= '</li>' . "\n" . '</ul>' . "\n";
}
else if ($lastMode == '#')
{
$this->ClearStyle();
$this->ApplyStyle();
$this->dst .= '</li>' . "\n" . '</ol>' . "\n";
}
$this->listType = substr($this->listType, 0, -1);
}
if (strlen($type) > 0 && strlen($type) < strlen($origLastListType))
{
$this->ClearStyle();
$this->ApplyStyle();
$this->dst .= '</li>' . "\n" . '<li>';
}
for ($i = $firstDiff; $i < strlen($type); $i++)
{
$newMode = $type{$i};
if ($newMode == '*')
{
$this->dst .= '<ul>' . "\n" . '<li>';
}
else if ($newMode == '#')
{
$this->dst .= '<ol>' . "\n" . '<li>';
}
$this->listType .= $newMode;
}
}
}
private function UpdateBlockQuote($level)
{
if ($level > $this->quoteLevel)
{
$this->dst .= str_repeat('<blockquote>' . "\n", $level - $this->quoteLevel);
$this->quoteLevel = $level;
}
else if ($level < $this->quoteLevel)
{
$this->ClearStyle();
$this->ApplyStyle();
$this->dst .= str_repeat('</blockquote>' . "\n", $this->quoteLevel - $level);
$this->quoteLevel = $level;
}
}
private function UpdateTable($active, $inQuote = false)
{
if ($active && !$this->tableLine)
{
if (!$inQuote) $this->UpdateBlockQuote(0);
if ($this->desiredStyle[self::STYLE_PAR])
{
$this->desiredStyle[self::STYLE_PAR] = false;
$this->ApplyStyle();
}
$this->maxTableColumns = 0;
$this->dst .= '<table>' . "\n";
$this->tableLine = true;
}
else if (!$active && $this->tableLine)
{
$this->dst .= '</table>' . "\n";
$this->maxTableColumns = 0;
$this->tableLine = false;
}
}
private function ResolveLink($link, $caption)
{
if (preg_match('_^url:(.*)$_i', $link, $m))
{
if (!strlen($caption)) $caption = $m[1];
return array($m[1], $caption);
}
if (preg_match('_^file:(.*)$_i', $link, $m))
{
if (!strlen($caption)) $caption = 'File: ' . $m[1];
return array($m[1], $caption);
}
if (preg_match('_^image:(.*)$_i', $link, $m))
{
$link = $m[1];
if (preg_match('_(:([0-9]+)(s?))$_i', $m[1], $m2))
{
$link = substr($link, 0, -strlen($m2[1]));
$size = intval($m2[2]);
$square = $m2[3] != '';
}
$imageFile = $link;
if (!strlen($caption)) $caption = 'Image: ' . $link;
if ($size > 0)
{
$imageFile .= '_' . $size;
}
$html = '<img src="' . $imageFile . '" alt="' . htmlspecialchars($caption) . '" title="' . htmlspecialchars($caption) . '" />';
return array($link, $html, false, true);
}
if (preg_match('_^google:(.*)$_i', $link, $m))
{
$useWordExtra = !strlen($caption);
if (!strlen($caption)) $caption = str_replace('_', ' ', $m[1]);
return array('http://www.google.de/search?&q=' . urlencode(str_replace('_', ' ', $m[1])), $caption, $useWordExtra);
}
if (preg_match('_^wikipedia:(.*)$_i', $link, $m))
{
$useWordExtra = !strlen($caption);
if (!strlen($caption)) $caption = str_replace('_', ' ', $m[1]);
return array('http://de.wikipedia.org/wiki/Spezial:Suche?go=Artikel&search=' . urlencode(str_replace('_', ' ', $m[1])), $caption, $useWordExtra);
}
if (preg_match('_^wikipedia-en:(.*)$_i', $link, $m))
{
$useWordExtra = !strlen($caption);
if (!strlen($caption)) $caption = str_replace('_', ' ', $m[1]);
return array('http://en.wikipedia.org/w/index.php?title=Special%3ASearch&go=Go&search=' . urlencode(str_replace('_', ' ', $m[1])), $caption, $useWordExtra);
}
if (preg_match('_^(https?|ftp|mailto|xmpp):_i', $link))
{
if (!strlen($caption)) $caption = $link;
return array($link, $caption);
}
if (preg_match('_^[-0-9a-z\_]*:_i', $link))
{
return false;
}
$useWordExtra = false;
if (!strlen($caption))
{
$caption = $link;
if ($link{0} != '#')
{
$useWordExtra = true;
}
}
return array($link, $caption, $useWordExtra);
}
}
class UnbFormHandler
{
const VerboseErrors = false;
private $newMessage = null;
private $newMessageRevision = null;
private $newTag = null;
private $newUser = null;
private $requestedActions;
private $skipActions = array();
public function GetNewMessage()
{
return $this->newMessage;
}
public function GetNewMessageRevision()
{
return $this->newMessageRevision;
}
public function GetNewTag()
{
return $this->newTag;
}
public function GetNewUser()
{
return $this->newUser;
}
public function HandleForm()
{
if (!self::ValidateRequest())
return array('error' => 'Error validating request');
$actions = $_REQUEST['action'];
if (!is_array($actions))
$actions = array($actions);
$fields = array();
$fields['actions'] = array();
if (!count($fields)) return null;
$tmpVars = array();
try
{
$lockActions = array(
'create_message',
'alter_message',
'moderate_messagerevision',
'set_message_readaccess',
'set_message_alteraccess',
'set_message_replyaccess',
'create_tag',
'set_tag_readaccess',
'set_tag_useaccess',
'create_user',
'set_user_displayname',
'set_user_additionalkeys',
'login'
);
$lock = count(array_intersect($actions, $lockActions)) > 0;
UnbDatabase::GetInstance()->StartTransaction($lock);
usort($actions, array(__CLASS__, 'SortActions'));
$this->requestedActions = $actions;
foreach ($actions as $action)
{
if (isset($fields['actions'][$action])) continue;
if (in_array($action, $this->skipActions)) continue;
try
{
$this->HandleFormAction($action, $actionFields, $tmpVars);
}
catch (UnbSecurityException $ex)
{
$actionFields['error'] = 'Security error: ' . $ex->getMessage();
if (self::VerboseErrors)
$actionFields['error'] .= "\nStack:\n" . $ex->getTraceAsString();
}
catch (UnbDatabaseException $ex)
{
$actionFields['error'] = 'Database error: ' . $ex->getMessage();
if (self::VerboseErrors)
$actionFields['error'] .= "\nStack:\n" . $ex->getTraceAsString();
}
catch (Exception $ex)
{
$actionFields['error'] = 'General error: ' . $ex->getMessage();
if (self::VerboseErrors)
$actionFields['error'] .= "\nStack:\n" . $ex->getTraceAsString();
}
$fields['actions'][$action] = $actionFields;
if (isset($actionFields['error']))
{
UnbDatabase::GetInstance()->RollBack();
$fields['erroraction'] = $action;
return $fields;
}
}
UnbDatabase::GetInstance()->Commit();
$fields['allsuccess'] = '';
}
catch (UnbDatabaseException $ex)
{
UnbDatabase::GetInstance()->RollBack();
$fields['error'] = 'Database error: ' . $ex->getMessage();
}
return $fields;
}
private static function SortActions($a, $b)
{
$actionsOrderFirst = array(
'login',
'create_message',
'alter_message',
'create_tag',
'create_user',
'set_user_additionalkeys',
);
$actionsOrderLast = array(
'get_messages',
);
if (in_array($a, $actionsOrderFirst))
{
if (in_array($b, $actionsOrderFirst))
{
$na = array_search($a, $actionsOrderFirst);
$nb = array_search($b, $actionsOrderFirst);
if ($na == $nb) return 0;
return $na < $nb ? -1 : 1;
}
return -1;
}
if (in_array($a, $actionsOrderLast))
{
if (in_array($b, $actionsOrderLast))
{
$na = array_search($a, $actionsOrderLast);
$nb = array_search($b, $actionsOrderLast);
if ($na == $nb) return 0;
return $na < $nb ? -1 : 1;
}
return 1;
}
if (in_array($b, $actionsOrderFirst))
{
return 1;
}
if (in_array($b, $actionsOrderLast))
{
return -1;
}
if ($a == $b) return 0;
return $a < $b ? -1 : 1;
}
private function SkipAction($action)
{
if (is_string($action))
$this->skipActions[] = $action;
if (is_array($action))
foreach ($action as $a)
if (is_string($a))
$this->skipActions[] = $a;
}
private function IsActionRequested($action)
{
return in_array($action, $this->requestedActions) && !in_array($action, $this->skipActions);
}
private function HandleFormAction($action, &$actionFields, &$tmpVars)
{
switch ($action)
{
case 'login':
$implicitRegister = $_POST['implicit_register'] ? true : false;
if (isset($_POST['user_loginname']))
$loginName = trim($_POST['user_loginname']);
if (isset($_POST['user_loginpassword']))
$loginPassword = trim($_POST['user_loginpassword']);
$this->SkipAction(array('set_user_loginname', 'set_user_loginpassword'));
if (!strlen($loginName) && !$implicitRegister)
{
$actionFields['error'] = '[#27] No user login name specified';
}
elseif (!strlen($loginPassword) && !UnbSession::IsAdministrator() && !$implicitRegister)
{
$actionFields['error'] = '[#28] No user login password specified';
}
else
{
if (strlen($loginName))
$user = UnbUser::FindByLoginName($loginName);
if (isset($user))
{
UnbSession::SetSessionUser($user, $loginPassword);
$actionFields['success'] = '';
}
else
{
if ($_REQUEST['allow_register'] || $implicitRegister)
{
$this->SkipAction('set_user_displayname');
if (isset($_POST['user_displayname']))
$displayName = trim($_POST['user_displayname']);
if (isset($_POST['user_repeatpassword']))
$repeatPassword = trim($_POST['user_repeatpassword']);
if (isset($displayName) && !strlen($displayName) && !strlen($loginName))
{
$actionFields['error'] = '[#29] No user display name specified';
}
else
{
if (!strlen($displayName) && strlen($loginName))
$displayName = $loginName;
if ($repeatPassword !== $loginPassword)
{
$actionFields['error'] = '[#30] Incorrect password repetition';
}
elseif (UnbConfiguration::Get('captcha.enabled', true) &&
class_exists('UnbCaptcha', false) &&
!UnbCaptcha::CheckWord($_POST['captcha_word']))
{
$actionFields['error'] = '[#31] Incorrect CAPTCHA word';
}
else
{
$user = UnbUser::Create($displayName, $loginName, $loginPassword);
$tmpVars['user'] = $user;
$this->newUser = $tmpVars['user'];
UnbSession::SetSessionUser($tmpVars['user'], null, true);
$actionFields['success'] = '';
if (class_exists('UnbCaptcha', false))
UnbCaptcha::ClearWord();
}
}
}
else
{
$actionFields['error'] = '[#32] Login name is unknown';
}
}
if (isset($actionFields['success']))
{
$p = $user->GetSingleSetting('autologin.password');
if (!isset($p))
{
$p = UnbSession::GeneratePassword();
$user->SetSetting('autologin.password', UnbSession::Encrypt($p));
}
else
{
$p = UnbSession::Decrypt($p);
}
$p = $user->GetUserId() . ' ' . $p;
UnbSession::SetCookie('unb_autologin', $p);
}
}
break;
case 'logout':
UnbSession::SetSessionUser(null);
$actionFields['success'] = '';
UnbSession::SetCookie('unb_autologin');
break;
case 'create_message':
$entryPoint = true;
if ($this->IsActionRequested('set_message_entrypoint'))
{
$entryPoint = $_POST['message_entrypoint'] ? true : false;
$this->SkipAction('set_message_entrypoint');
}
$subject = trim($_POST['message_subject']);
$content = rtrim($_POST['message_content']);
$data = null;
$dataFileName = null;
$writeToFile = null;
if ($_POST['set_messagerevision_filedata'])
{
if (strlen(trim($_FILES['message_filedata']['tmp_name'])))
{
$dataFileName = $_FILES['message_filedata']['tmp_name'];
}
}
$primaryRef = null;
if (intval($_POST['replyto_messageid']) > 0)
$primaryRef = intval($_POST['replyto_messageid']);
if (!strlen($content))
{
$actionFields['error'] = '[#33] No message content specified';
}
else
{
$tmpVars['message'] = UnbMessage::Create($subject, $content, $data, $dataFileName, $writeToFile, $entryPoint, $primaryRef);
$tmpVars['messagerevision'] = $tmpVars['message']->GetSearchRevision(UnbMessageRevisionContentLevel::Content, true);
$actionFields['success'] = '';
$this->newMessage = $tmpVars['message'];
$this->newMessageRevision = $tmpVars['messagerevision'];
}
break;
case 'set_messagerevision_tags':
$rev = $this->GetMessageRevision($tmpVars);
$tagIds = $_POST['message_tagid'];
if (!is_array($tagIds))
{
if (strlen($tagIds))
$tagIds = array($tagIds);
else
$tagIds = array();
}
$tagIds = array_map('intval', $tagIds);
$revTagIds = $rev->GetTagIds(false);
if ($revTagIds)
{
foreach (array_diff($revTagIds, $tagIds) as $tagId)
{
$rev->RemoveTag($tagId);
}
}
foreach ($tagIds as $tagId)
{
$tag = UnbTag::FindByTagId($tagId);
if (!isset($tag)) continue;
if ($tag->GetFlags() & UnbTagFlag::NotSelectable) continue;
$tagData = null;
if ($tag->GetFlags() & UnbTagFlag::UserData &&
strlen(trim($_POST['message_tagdata_' . $tagId])))
{
$tagData = trim($_POST['message_tagdata_' . $tagId]);
}
$rev->AddTag($tagId, $tagData);
}
$actionFields['success'] = '';
break;
case 'set_message_pagename':
$msg = $this->GetMessage($tmpVars);
$msg->SetPageName($_POST['message_pagename']);
$actionFields['success'] = '';
break;
case 'set_message_entrypoint':
$msg = $this->GetMessage($tmpVars);
$msg->SetEntryPoint($_REQUEST['message_entrypoint'] ? true : false);
$actionFields['success'] = '';
break;
case 'set_message_readaccess':
if (!$_POST['message_readaccess_empty'])
{
$keys = $this->GetKeys('message_readaccesslist');
if (is_string($keys))
{
$actionFields['error'] = $keys;
break;
}
}
else
{
$keys = array();
}
$msg = $this->GetMessage($tmpVars);
$msg->SetReadAccessKeys($keys);
$actionFields['success'] = '';
break;
case 'set_message_alteraccess':
if (!$_POST['message_alteraccess_empty'])
{
$keys = $this->GetKeys('message_alteraccesslist');
if (is_string($keys))
{
$actionFields['error'] = $keys;
break;
}
$keys[] = UnbUserKey::Moderator;
}
else
{
$keys = array();
}
$msg = $this->GetMessage($tmpVars);
$msg->SetAlterAccessKeys($keys);
$actionFields['success'] = '';
break;
case 'set_message_replyaccess':
if (!$_POST['message_replyaccess_empty'])
{
$keys = $this->GetKeys('message_replyaccesslist');
if (is_string($keys))
{
$actionFields['error'] = $keys;
break;
}
}
else
{
$keys = array();
}
$msg = $this->GetMessage($tmpVars);
$msg->SetReplyAccessKeys($keys);
$actionFields['success'] = '';
break;
case 'alter_message':
$subject = true;
$content = true;
$summary = null;
$data = true;
$dataFileName = null;
$writeToFile = null;
if ($_POST['message_subject_hash'] &&
md5($_POST['message_subject']) != $_POST['message_subject_hash'])
{
$subject = trim($_POST['message_subject']);
}
if ($_POST['message_content_hash'] &&
md5($_POST['message_content']) != $_POST['message_content_hash'])
{
$content = rtrim($_POST['message_content']);
}
$summary = trim($_POST['message_summary']);
if ($_POST['set_messagerevision_filedata'])
{
$clearData = $_POST['message_cleardata'] ? true : false;
if ($clearData)
{
$data = null;
}
elseif (strlen(trim($_FILES['message_filedata']['tmp_name'])))
{
$dataFileName = $_FILES['message_filedata']['tmp_name'];
}
}
if (is_string($content) && !strlen($content))
{
$actionFields['error'] = '[#33] No message content specified';
}
elseif ($subject !== true ||
$content !== true ||
strlen($summary) > 0 ||
$data !== true)
{
$baseRev = $this->GetMessageRevision($tmpVars);
$tmpVars['messagerevision'] = $baseRev->CreateCopy($subject, $content, $summary, $data, $dataFileName, $writeToFile);
$actionFields['success'] = '';
$this->newMessageRevision = $tmpVars['messagerevision'];
}
break;
case 'delete_message':
$msg = $this->GetMessage($tmpVars);
$msg->Delete();
$actionFields['success'] = '';
break;
case 'moderate_message':
$flags = 0;
if ($_POST['message_locked']) $flags |= UnbMessageModerationFlag::Locked;
if ($_POST['message_hidden']) $flags |= UnbMessageModerationFlag::Hidden;
if ($_POST['message_enforce_approval']) $flags |= UnbMessageModerationFlag::EnforceApproval;
$msg = $this->GetMessage($tmpVars);
$msg->SetModerationFlags($flags);
$actionFields['success'] = '';
break;
case 'rate_message':
$msg = $this->GetMessage($tmpVars);
$msg->SetRating(intval($_REQUEST['message_rating']));
$actionFields['success'] = '';
break;
case 'moderate_messagerevision':
$rev = $this->GetMessageRevision($tmpVars);
$rev->SetModerationState(intval($_REQUEST['message_modstate']));
$actionFields['success'] = '';
break;
case 'create_tag':
$tmpVars['tag'] = UnbTag::Create($_POST['tag_name']);
$actionFields['success'] = '';
$this->newTag = $tmpVars['tag'];
$this->SkipAction('set_tag_name');
break;
case 'set_tag_name':
$tag = $this->GetTag($tmpVars);
$tag->SetTagName($_POST['tag_name']);
$actionFields['success'] = '';
break;
case 'set_tag_description':
$tag = $this->GetTag($tmpVars);
$tag->SetDescription($_POST['tag_description']);
$actionFields['success'] = '';
break;
case 'set_tag_flags':
$flags = 0;
if ($_POST['tag_flag_notselectable'])
$flags |= UnbTagFlag::NotSelectable;
if ($_POST['tag_flag_invisible'])
$flags |= UnbTagFlag::Invisible;
if ($_POST['tag_flag_enforceapproval'])
$flags |= UnbTagFlag::EnforceApproval;
if ($_POST['tag_flag_userdata'])
$flags |= UnbTagFlag::UserData;
$tag = $this->GetTag($tmpVars);
$tag->SetFlags($flags);
$actionFields['success'] = '';
break;
case 'set_tag_parent':
$tag = $this->GetTag($tmpVars);
$tag->SetParentTag(intval($_POST['tag_parentid']));
$actionFields['success'] = '';
break;
case 'set_tag_readaccess':
if (!$_POST['tag_readaccess_empty'])
{
$keys = $this->GetKeys('tag_readaccesslist');
if (is_string($keys))
{
$actionFields['error'] = $keys;
break;
}
}
else
{
$keys = array();
}
$tag = $this->GetTag($tmpVars);
$tag->SetReadAccessKeys($keys);
$actionFields['success'] = '';
break;
case 'set_tag_useaccess':
if (!$_POST['tag_useaccess_empty'])
{
$keys = $this->GetKeys('tag_useaccesslist');
if (is_string($keys))
{
$actionFields['error'] = $keys;
break;
}
}
else
{
$keys = array();
}
$tag = $this->GetTag($tmpVars);
$tag->SetUseAccessKeys($keys);
$actionFields['success'] = '';
break;
case 'delete_tag':
$tag = $this->GetTag($tmpVars);
$tag->Delete();
$actionFields['success'] = '';
break;
case 'create_user':
$displayName = trim($_POST['user_displayname']);
$loginName = trim($_POST['user_loginname']);
$loginPassword = trim($_POST['user_loginpassword']);
$this->SkipAction(array('set_user_loginname', 'set_user_loginpassword', 'set_user_displayname'));
if (!strlen($displayName))
$displayName = $loginName;
$repeatPassword = trim($_POST['user_repeatpassword']);
if ($repeatPassword !== $loginPassword)
{
$actionFields['error'] = '[#30] Incorrect password repetition';
}
else
{
$tmpVars['user'] = UnbUser::Create($displayName, $loginName, $loginPassword);
$actionFields['success'] = '';
$this->newUser = $tmpVars['user'];
}
break;
case 'set_user_additionalkeys':
if (!$_POST['user_additionalkeys_empty'])
{
$keys = $this->GetKeys('user_additionalkeyslist');
if (is_string($keys))
{
$actionFields['error'] = $keys;
break;
}
}
else
{
$keys = array();
}
if ($this->IsActionRequested('set_user_administrator'))
{
if ($_POST['user_is_administrator'])
$keys[] = UnbUserKey::Administrator;
$this->SkipAction('set_user_administrator');
}
if ($this->IsActionRequested('set_user_moderator'))
{
if ($_POST['user_is_moderator'])
$keys[] = UnbUserKey::Moderator;
$this->SkipAction('set_user_moderator');
}
if ($this->IsActionRequested('set_user_trusted'))
{
if ($_POST['user_is_trusted'])
$keys[] = UnbUserKey::Trusted;
$this->SkipAction('set_user_trusted');
}
$user = $this->GetUser($tmpVars);
$user->SetAdditionalKeys($keys);
$actionFields['success'] = '';
break;
case 'set_user_administrator':
$user = $this->GetUser($tmpVars);
if ($_REQUEST['user_is_administrator'])
$user->AddAdditionalKey(UnbUserKey::Administrator);
else
$user->RemoveAdditionalKey(UnbUserKey::Administrator);
$actionFields['success'] = '';
break;
case 'set_user_moderator':
$user = $this->GetUser($tmpVars);
if ($_REQUEST['user_is_moderator'])
$user->AddAdditionalKey(UnbUserKey::Moderator);
else
$user->RemoveAdditionalKey(UnbUserKey::Moderator);
$actionFields['success'] = '';
break;
case 'set_user_trusted':
$user = $this->GetUser($tmpVars);
if ($_REQUEST['user_is_trusted'])
$user->AddAdditionalKey(UnbUserKey::Trusted);
else
$user->RemoveAdditionalKey(UnbUserKey::Trusted);
$actionFields['success'] = '';
break;
case 'set_user_displayname':
$user = $this->GetUser($tmpVars);
$displayName = trim($_POST['user_displayname']);
if ($displayName != $user->GetDisplayName())
{
$user->SetDisplayName($displayName);
$actionFields['success'] = '';
}
break;
case 'set_user_loginname':
$user = $this->GetUser($tmpVars);
$loginName = trim($_POST['user_loginname']);
if ($loginName != $user->GetLoginName())
{
if (!UnbSession::IsAdministrator() &&
!UnbSession::ValidateSessionUserPassword($_POST['user_currentpassword']))
{
$actionFields['error'] = '[#34] Incorrect current password';
}
else
{
$user->SetLoginName($loginName);
$actionFields['success'] = '';
}
}
break;
case 'set_user_loginpassword':
if (!strlen($_POST['user_loginpassword']) || strlen(trim($_POST['user_loginpassword'])))
{
if (trim($_POST['user_loginpassword']) !== trim($_POST['user_repeatpassword']))
{
$actionFields['error'] = '[#30] Incorrect password repetition';
}
elseif (!UnbSession::IsAdministrator() &&
!UnbSession::ValidateSessionUserPassword($_POST['user_currentpassword']))
{
$actionFields['error'] = '[#34] Incorrect current password';
}
else
{
$user = $this->GetUser($tmpVars);
$user->SetLoginPassword(trim($_POST['user_loginpassword']));
$actionFields['success'] = '';
}
}
break;
case 'delete_user':
$user = $this->GetUser($tmpVars);
$user->Delete();
$actionFields['success'] = '';
break;
case 'set_user_vcard':
$user = $this->GetUser($tmpVars);
$vcardTypes = array();
foreach ($_POST as $name => $value)
{
if (preg_match('/^vcard_data_(.+)$/', $name, $m))
$vcardTypes[] = $m[1];
}
foreach ($vcardTypes as $type)
{
if ($_POST['vcard_remove_' . $type])
{
$user->RemoveVcardEntry($type);
}
else
{
$visibility = null;
if (isset($_POST['vcard_visibility_' . $type]))
$visibility = intval($_POST['vcard_visibility_' . $type]);
$user->SetVcardEntry($type, $_POST['vcard_data_' . $type], $visibility);
}
}
if (strlen(trim($_POST['vcard_newtype'])))
{
$visibility = UnbUserVcardVisibility::Public_;
if (isset($_POST['vcard_newvisibility']))
$visibility = intval($_POST['vcard_newvisibility']);
$user->SetVcardEntry($_POST['vcard_newtype'], $_POST['vcard_newdata'], $visibility);
}
$actionFields['success'] = '';
break;
case 'get_messages':
$searchParameters = array();
if (is_numeric($_POST['search_replies_of']))
{
$replyIds = UnbSearch::GetAllReplies(intval($_POST['search_replies_of']));
$searchParameters['message-ids'] = array_keys($replyIds);
if (!count($replyIds))
{
$actionFields['message'] = array();
break;
}
}
if (is_array($_POST['search_message_ids']))
$searchParameters['message-ids'] = array_map('intval', $_POST['search_message_ids']);
if (is_string($_POST['search_message_ids']) && strlen($_POST['search_message_ids']))
$searchParameters['message-ids'] = array_map('intval', explode(',', $_POST['search_message_ids']));
if ($_POST['get_new_message'])
{
if ($this->newMessage)
{
$searchParameters['message-ids'] = array($this->newMessage->GetMessageId());
}
else
{
$actionFields['error'] = 'No new message to get';
break;
}
}
if (is_array($_POST['search_owner_ids']))
$searchParameters['owner-ids'] = array_map('intval', $_POST['search_owner_ids']);
if (is_string($_POST['search_owner_ids']) && strlen($_POST['search_owner_ids']))
$searchParameters['owner-ids'] = array_map('intval', explode(',', $_POST['search_owner_ids']));
if ($_POST['search_entrypoint'])
$searchParameters['entrypoint'] = true;
if (is_array($_POST['search_tag_ids']))
$searchParameters['tags-set'] = array_map('intval', $_POST['search_tag_ids']);
if (is_string($_POST['search_tag_ids']) && strlen($_POST['search_tag_ids']))
$searchParameters['tags-set'] = array_map('intval', explode(',', $_POST['search_tag_ids']));
if (is_numeric($_POST['search_primary_reference']))
$searchParameters['primary-reference'] = intval($_POST['search_primary_reference']);
if (is_numeric($_POST['search_sort_tag_id']))
$searchParameters['sort-tagid'] = intval($_POST['search_sort_tag_id']);
if ($_POST['search_sort_createdtime'])
$searchParameters['sort-createdtime'] = true;
if ($_POST['search_sort_reverse'])
$searchParameters['sort-reverse'] = true;
if (is_string($_POST['search_limit']) && strlen($_POST['search_limit']))
$searchParameters['limit'] = trim($_POST['search_limit']);
$messageIds = UnbSearch::Query($searchParameters);
$contentLevel = UnbMessageRevisionContentLevel::Subject;
if ($_POST['message_contentlevel'] == 1)
{
$contentLevel = UnbMessageRevisionContentLevel::Content;
}
if ($_POST['message_contentlevel'] == 2)
{
$contentLevel = UnbMessageRevisionContentLevel::HtmlContent;
$markup = new UnbMarkup();
}
$actionFields['message'] = array();
if ($messageIds) foreach ($messageIds as $messageId)
{
$msg = UnbMessage::FindByMessageId($messageId);
if (!isset($msg)) continue;
$searchRev = $msg->GetSearchRevision($contentLevel, true);
$latestRev = $msg->GetLatestRevision($contentLevel, true);
if (!isset($searchRev)) continue;
if (!isset($latestRev)) continue;
$msgData = array();
$msgData['message-id'] = $messageId;
$msgData['revision-number'] = $searchRev->GetRevisionNumber();
$msgData['primary-reference'] = $msg->GetPrimaryReference();
if ($msg->IsEntryPoint())
$msgData['entry-point'] = true;
if ($searchRev->GetContentLevel() >= UnbMessageRevisionContentLevel::Subject)
{
$msgData['subject'] = $searchRev->GetSubject();
}
if ($searchRev->GetContentLevel() >= UnbMessageRevisionContentLevel::HtmlContent)
{
$html = $searchRev->GetHtmlContent();
if (!isset($html))
{
if ($searchRev->GetContent() !== null)
{
$html = $markup->Convert($searchRev->GetContent());
try
{
$searchRev->SetHtmlContent($html);
}
catch (Exception $ex) { }
}
else
{
$html = '';
}
}
$html = $markup->PostProcess($html);
$msgData['htmlcontent'] = $html;
}
elseif ($searchRev->GetContentLevel() >= UnbMessageRevisionContentLevel::Content)
{
$msgData['content'] = $searchRev->GetContent();
}
$msgData['owner'] = array(
'id' => $msg->GetOwnerUser()->GetUserId(),
'name' => $msg->GetOwnerUser()->GetDisplayName());
$msgData['created-time'] = array(
'unix' => $msg->GetCreatedTime(),
'db' => UnbDatabase::ConvertDateTime($msg->GetCreatedTime()));
if ($searchRev->GetCreatedTime() > $msgCreatedTime)
$msgData['altered-time'] = array(
'unix' => $searchRev->GetCreatedTime(),
'db' => UnbDatabase::ConvertDateTime($searchRev->GetCreatedTime()));
$latestReply = $msg->GetLatestReply();
if (isset($latestReply))
{
$msgData['replies-count'] = $msg->CountReplies();
$msgData['last-reply-time'] = array(
'unix' => $latestReply->GetCreatedTime(),
'db' => UnbDatabase::ConvertDateTime($latestReply->GetCreatedTime()));
}
$msgData['tag'] = array();
$tags = $searchRev->GetTags(true);
if ($tags) foreach ($tags as $tag)
{
$tagData = array();
$tagData['id'] = $tag->GetTagId();
$tagData['name'] = $tag->GetTagName();
if (($tag->GetFlags() & UnbTagFlag::UserData) && ($tag->GetTagData() != null))
$tagData['data'] = $tag->GetTagData();
$msgData['tag'][] = $tagData;
}
if ($searchRev->HasFileData())
{
$msgData['attach-size'] = $searchRev->GetFileDataLength();
}
$flags = $msg->GetModerationFlags();
$msgData['moderation'] = array(
'hidden' => ($flags & UnbMessageModerationFlag::Hidden) ? true : false,
'locked' => ($flags & UnbMessageModerationFlag::Locked) ? true : false,
'latest-rev-waiting-for-approval' => ($latestRev->GetModerationState() == UnbMessageRevisionModerationState::WaitingForApproval),
'latest-rev-locked' => ($latestRev->GetModerationState() == UnbMessageRevisionModerationState::Locked));
$msgData['is-alter-allowed'] = UnbMessage::IsAlterAllowedById($messageId);
$msgData['is-reply-allowed'] = UnbMessage::IsReplyAllowedById($messageId);
$actionFields['message'][] = $msgData;
}
break;
case 'list_revisions':
break;
case 'list_tags':
break;
case 'list_users':
break;
case 'optimise_database':
if (UnbSession::IsAdministrator())
{
UnbDatabase::GetInstance()->Optimise($_REQUEST['optimise_db_extensive'] ? true : false);
$actionFields['success'] = '';
}
else
$actionFields['error'] = 'Only allowed for administrators';
break;
default:
$actionFields['error'] = 'Unknown action: ' . $action;
}
}
private function GetMessage(&$tmpVars)
{
if ($tmpVars['message'] instanceof UnbMessage)
$msg = $tmpVars['message'];
else
{
$msg = UnbMessage::FindByMessageId(intval($_REQUEST['messageid']));
$tmpVars['message'] = $msg;
}
return $msg;
}
private function GetMessageRevision(&$tmpVars)
{
if ($tmpVars['messagerevision'] instanceof UnbMessageRevision)
$rev = $tmpVars['messagerevision'];
else
{
if ($tmpVars['message'] instanceof UnbMessage)
$msg = $tmpVars['message'];
else
{
$msg = UnbMessage::FindByMessageId(intval($_REQUEST['messageid']));
$tmpVars['message'] = $msg;
}
$revisionNumber = intval($_REQUEST['revisionnumber']);
if ($revisionNumber > 0)
$rev = $msg->GetRevision($revisionNumber, UnbMessageRevisionContentLevel::Content, true);
else
$rev = $msg->GetLatestRevision(UnbMessageRevisionContentLevel::Content, true);
$tmpVars['messagerevision'] = $rev;
}
return $rev;
}
private function GetUser(&$tmpVars)
{
if ($tmpVars['user'] instanceof UnbUser)
$user = $tmpVars['user'];
else
{
$user = UnbUser::FindByUserId(intval($_REQUEST['userid']));
$tmpVars['user'] = $user;
}
return $user;
}
private function GetTag(&$tmpVars)
{
if ($tmpVars['tag'] instanceof UnbTag)
$tag = $tmpVars['tag'];
else
{
$tag = UnbTag::FindByTagId(intval($_REQUEST['tagid']));
$tmpVars['tag'] = $tag;
}
return $tag;
}
private function GetKeys($fieldName)
{
$keys = array();
foreach (explode("\n", $_POST[$fieldName]) as $line)
{
$line = trim($line);
if (!strlen($line)) continue;
if (!is_numeric($line))
{
$user = UnbUser::FindByDisplayName($line);
if (!isset($user))
return '[#32] User name "' . $line . '" is unknown.';
$userId = $user->GetUserId();
}
else
{
$userId = intval($line);
if ($userId == -1 && UnbSession::IsLoggedIn())
$userId = UnbSession::GetSessionUserId();
if ($userId <= 0)
return '[#35] User ID "' . $line . '" is invalid.';
$user = UnbUser::FindByUserId($userId);
if (!isset($user))
return '[#36] User ID "' . $line . '" is unknown.';
}
$keys[] = $userId;
}
return $keys;
}
public static function ValidateRequest()
{
if (strlen(UnbEnvironment::GetReferer()) && !UnbEnvironment::IsRefererMyDomain())
{
return false;
}
if (!UnbSession::CheckUrlKey($_REQUEST['urlkey']))
{
return false;
}
return true;
}
}
class UnbCaptcha
{
private static $config = null;
private function __construct()
{
}
public static function GetConfiguration()
{
if (isset(self::$config)) return self::$config;
$C = array();
$C['imgWidth'] = 172;
$C['imgHeight'] = 47;
$C['backRLo'] = 230;
$C['backRHi'] = 250;
$C['backGLo'] = 230;
$C['backGHi'] = 250;
$C['backBLo'] = 230;
$C['backBHi'] = 250;
$C['fontPath'] = '';
$C['knownFonts'] = array(
'Vera.ttf',
'Vera.ttf',
'Vera.ttf',
'Vera.ttf',
'Vera.ttf',
'VeraBd.ttf',
);
$C['sizeLo'] = 19;
$C['sizeHi'] = 26;
$C['angleLo'] = -9;
$C['angleHi'] = 9;
$C['foreRLo'] = 50;
$C['foreRHi'] = 80;
$C['foreGLo'] = 50;
$C['foreGHi'] = 80;
$C['foreBLo'] = 70;
$C['foreBHi'] = 140;
$C['hLines'] = 12;
$C['hLineOpacLo'] = 70;
$C['hLineOpacHi'] = 100;
$C['vLines'] = 12;
$C['vLineOpacLo'] = 70;
$C['vLineOpacHi'] = 100;
$C['rLines'] = 16;
$C['rLineOpacLo'] = 70;
$C['rLineOpacHi'] = 100;
$C['negative'] = false;
$C['wordsFileName'] = 'captcha_words_en.txt';
$C['symbols'] = '"#%*-:;=^_';
$C['combineLetters'] = array(
'A' => 'ABCDEFGHI KLMNOPQRSTU WX Z"#%*-:;=^_',
'B' => 'ABCDEFGHI KLMNOPQRSTUVWXYZ"#%*-:;=^_',
'C' => 'ABCDEFGH  KLMNOPQRS UVWXYZ #%*    ^ ',
'D' => 'ABCDEFGHI KLMNOPQRSTUVWXYZ"#%* :;=^_',
'E' => 'ABCDEFGH  KLMNOPQRS UVWX    %* :; ^ ',
'F' => 'ABCDEFGH  KLMNOPQRS UVW YZ #        ',
'G' => 'ABCDEFGH  KLMNOPQRSTUVWXYZ #%  :;  _',
'H' => 'ABCDEFGH  KLMNOPQRS UVWXYZ"#%* :;=^_',
'I' => 'A C   G       O Q S    X Z" %       ',
'J' => '  C   G       O           "  *      ',
'K' => 'ABCDEFGHI KLMNOPQRSTUVWXYZ" %* :; ^_',
'L' => 'A                      X   #%      _',
'M' => 'ABCDEFGH  KLMNOPQRS UVWXYZ"#%*-:;=^_',
'N' => 'ABCDEFGH  KLMNOPQRS U  XYZ"#%*-:;=^_',
'O' => 'ABCDEFGHI KLMNOPQRSTUVWXYZ"#%*-:;=^_',
'P' => 'ABCDEFGH  KLMNOPQRSTUVWXYZ"#%*    ^ ',
'Q' => 'ABCDEFGHI KLMNOPQRSTUVWXYZ"#%*-  =^ ',
'R' => 'ABCDEFGH  KLMNOPQRSTUVWXYZ"#%* :;=^ ',
'S' => 'ABCDEFGHI KLMNOPQRSTUVWXYZ #%* :;=^_',
'T' => 'A C   G   K   O Q S UVW             ',
'U' => 'ABCDEFGH  KLMNOPQRS U WXYZ"#%*-:;= _',
'V' => ' BCDEFGH  K MNOPQRST   XYZ"#%* :;=^ ',
'W' => ' BCDEFGH  KLMNOPQRS U WXYZ"#%*-:;=^_',
'X' => 'AB DEF HI KLMN P RSTUVW  Z"#%*   =^ ',
'Y' => ' B DEF H  KLMN P RSTUVW  Z"#%*    ^ ',
'Z' => 'ABCDEFGH  KLMNOPQRS UVWXYZ"#%*    ^_',
'"' => 'ABCDEFGHI KLMNOPQRS UVWXY           ',
'#' => 'ABCDEFGH  KLMNOPQRS UVWXY           ',
'%' => 'ABCDEFGHI KLMNOPQRS UVWXYZ          ',
'*' => 'ABCDEFGHI KLMNOPQRS UVWXYZ          ',
'-' => 'ABCDEFGH  KLMNOPQR  U W             ',
':' => 'ABCDEFGH  KLMNOPQRS UVW             ',
';' => 'ABCDEFGH  KLMNOPQRS UVW             ',
'=' => 'ABCDEFGHI KLMNOPQRS UVWX            ',
'^' => 'ABCDEFGH  KLMNOPQRS  VWXYZ          ',
'_' => 'ABCDEFGH  KLMNOPQRS UVWXYZ          ',
);
return $C;
}
public static function SetConfiguration($config)
{
if (!is_array($config))
throw new InvalidArgumentException('Invalid argument type: $config');
self::$config = $config;
}
public static function GenerateImage()
{
$C = self::GetConfiguration();
$wordsFileName = $C['wordsFileName'];
if ($wordsFileName{0} !== '/')
$wordsFileName = dirname(__FILE__) . '/' . $wordsFileName;
$wordsStr = trim(preg_replace('_#.*?\\n_', '', file_get_contents($wordsFileName)));
$wordsStr = preg_replace('_[ \t\r\n,;]+_', ' ', $wordsStr);
$words = explode(' ', $wordsStr);
$wordsStr = null;
$words = array_map('trim', $words);
$words = array_map('strtoupper', $words);
$text = self::GetWord();
if (!isset($text) || !strlen($text))
{
$text = $words[mt_rand(0, count($words) - 1)];
$addLength = min(2, mt_rand(6, 8) - strlen($text));
if ($addLength < 1) $addLength = 1;
$wordStart = 0;
$wordEnd = strlen($text) - 1;
for ($i = $addLength; $i > 0; $i--)
{
while (true)
{
$pos = mt_rand(0, strlen($text));
if ($pos <= $wordStart || $pos > $wordEnd ||
ctype_alpha($text{$pos - 1}) && ctype_alpha($text{$pos})) break;
}
$a = substr($text, 0, $pos);
$b = substr($text, $pos);
$text = $a . $C['symbols']{mt_rand(0, strlen($C['symbols']) - 1)} . $b;
if ($pos <= $wordStart) $wordStart++;
if ($pos <= $wordEnd) $wordEnd++;
}
UnbSession::SetVar('CaptchaWord', $text);
}
$imgWidth = $C['imgWidth'];
$imgHeight = $C['imgHeight'];
$im = imagecreatetruecolor($imgWidth, $imgHeight);
$backColor = array(mt_rand($C['backRLo'], $C['backRHi']), mt_rand($C['backGLo'], $C['backGHi']), mt_rand($C['backBLo'], $C['backBHi']));
imagefill($im, 0, 0, imagecolorallocate($im, $backColor[0], $backColor[1], $backColor[2]));
$fontPath = $C['fontPath'];
if ($fontPath{0} !== '/')
$fontPath = dirname(__FILE__) . '/' . $fontPath;
$fonts = array();
foreach ($C['knownFonts'] as $knownFont)
{
if (file_exists($fontPath . '/' . $knownFont))
$fonts[] = $fontPath . '/' . $knownFont;
}
if (!count($fonts))
$fonts = glob($fontPath . '/*.ttf');
$color = array(mt_rand($C['foreRLo'], $C['foreRHi']), mt_rand($C['foreGLo'], $C['foreGHi']), mt_rand($C['foreBLo'], $C['foreBHi']));
$x = 5 + max(0, 8 - strlen($text)) * mt_rand(5, 15);
$prevAngle = 0;
$bbox = array(0, 0, 0);
for ($i = 0; $i < strlen($text); $i++)
{
$fontfile = $fonts[mt_rand(0, count($fonts) - 1)];
$size = mt_rand($C['sizeLo'], $C['sizeHi']);
$angle = (mt_rand($C['angleLo'], $C['angleHi']) + $prevAngle) / 2;
$x += round(($bbox[2] - $bbox[0]) * 0.8 - ($angle - $prevAngle) / 20 * 0.2) + 2;
$y =
/* middle */ $imgHeight / 2 + $size / 2 + 3 +
/* deviation */ mt_rand(-($imgHeight / 2 - $size), $imgHeight / 2 - $size * 1.3);
$ch = $text{$i};
$bbox = imagettfbbox($size, $angle, $fontfile, $ch);
if ($i == 0 || $i > 0 && strpos($C['combineLetters'][$text{$i - 1}], $text{$i}) === false)
{
$x += 2;
}
else
{
$overlaps = false;
$im2 = imagecreatetruecolor($imgWidth, $imgHeight);
$startX = $x;
while (!$overlaps && $x > 0 && $x > $startX - 10)
{
$x--;
imagefill($im2, 0, 0, imagecolorallocate($im2, 255, 255, 255));
imagettftext($im2, $size, $angle, $x, $y,
imagecolorallocate($im2, 0, 0, 0),
$fontfile, $ch);
$o_count = 0;
$max_o_x = min($imgWidth, $x + 40);
$max_o_y = $imgHeight - 1;
for ($o_y = 0; $o_y < $max_o_y; $o_y++)
{
for ($o_x = $x - 10; $o_x < $max_o_x; $o_x++)
{
$c1 = imagecolorat($im, $o_x, $o_y);
$c2 = imagecolorat($im2, $o_x, $o_y);
$c1 = (($c1 >> 16) & 0xFF) + (($c1 >> 8) & 0xFF) + ($c1 & 0xFF);
$c2 = (($c2 >> 16) & 0xFF) + (($c2 >> 8) & 0xFF) + ($c2 & 0xFF);
if ($c1 + $c2 < $C['foreRHi'] + $C['foreGHi'] + $C['foreBHi'] + 20 * 3)
$o_count++;
if ($o_count >= 2)
{
$overlaps = true;
$o_y = $imgHeight;
$o_x = $imgWidth;
}
}
}
}
imagedestroy($im2);
}
$x++;
imagettftext($im, $size, $angle, $x, $y,
imagecolorallocate($im, $color[0], $color[1], $color[2]),
$fontfile, $ch);
$prevAngle = $angle;
}
for ($i = 0; $i < 8; $i++)
{
$x1 = mt_rand(0, $imgWidth - 5);
$x2 = mt_rand($x1 + 1, $imgWidth - 1);
$imBak = imagecreatetruecolor($imgWidth, $imgHeight);
imagecopy($imBak, $im, 0, 0, 0, 0, $imgWidth, $imgHeight);
if (mt_rand(0, 9) >= 5)
{
imagecopy($im, $imBak, $x1, 0, $x1, 1, $x2 - $x1, $imgHeight - 1);
}
else
{
imagecopy($im, $imBak, $x1, 1, $x1, 0, $x2 - $x1, $imgHeight - 1);
}
imagedestroy($imBak);
}
$matrix = array(
array(0, 1, 0),
array(1, 3, 1),
array(0, 1, 0));
$div = 7;
if (function_exists('imageconvolution'))
imageconvolution($im, $matrix, $div, 0);
else
self::php_imageconvolution($im, $matrix, $div, 0);
for ($i = 0; $i < 100; $i++)
{
$x = mt_rand(0, $imgWidth - 1);
$y = mt_rand(0, $imgHeight - 1);
imagesetpixel($im, $x, $y,
imagecolorallocate($im, $color[0], $color[1], $color[2]));
}
for ($i = 0; $i < $C['hLines']; $i++)
{
$x1 = mt_rand(0, $imgWidth / 2);
$y1 = mt_rand(0, $imgHeight - 1);
$x2 = mt_rand($imgWidth / 2, $imgWidth - 1);
$y2 = $y1;
imageline($im, $x1, $y1, $x2, $y2,
imagecolorallocatealpha($im, $backColor[0], $backColor[1], $backColor[2], mt_rand($C['hLineOpacLo'], $C['hLineOpacHi'])));
}
for ($i = 0; $i < $C['vLines']; $i++)
{
$x1 = mt_rand(0, $imgWidth - 1);
$y1 = 0;
$x2 = $x1;
$y2 = $imgHeight;
imageline($im, $x1, $y1, $x2, $y2,
imagecolorallocatealpha($im, $backColor[0], $backColor[1], $backColor[2], mt_rand($C['vLineOpacLo'], $C['vLineOpacHi'])));
}
for ($i = 0; $i < $C['rLines']; $i++)
{
$x1 = mt_rand(0, $imgWidth - 1);
$y1 = 0;
$x2 = mt_rand(0, $imgWidth - 1);
$y2 = $imgHeight;
imageline($im, $x1, $y1, $x2, $y2,
imagecolorallocatealpha($im, $backColor[0], $backColor[1], $backColor[2], mt_rand($C['rLineOpacLo'], $C['rLineOpacHi'])));
}
if ($C['negative'])
{
imagefilter($im, IMG_FILTER_NEGATE);
}
header('Pragma: no-cache');
header('Cache-Control: no-cache');
header('Expires: 0');
header('Content-Type: image/jpeg');
imagejpeg($im, null, 90);
exit();
}
public static function GetWord()
{
return UnbSession::GetVar('CaptchaWord');
}
private static function NormaliseWord($text)
{
$out = '';
for ($i = 0; $i < strlen($text); $i++)
{
$ch = $text{$i};
if (ctype_alpha($ch))
$out .= $ch;
}
return strtoupper($out);
}
public static function CheckWord($tryWord)
{
$tryword = strtoupper(trim($tryWord));
if (!strlen($tryword))
return false;
$word = self::NormaliseWord(self::GetWord());
if ($tryword == $word)
return true;
UnbSession::SetVar('CaptchaWord');
return false;
}
public static function ClearWord()
{
UnbSession::SetVar('CaptchaWord');
}
private static function php_imageconvolution($src, $filter, $filter_div, $offset)
{
if ($src==NULL) {
return 0;
}
$sx = imagesx($src);
$sy = imagesy($src);
$srcback = ImageCreateTrueColor ($sx, $sy);
ImageCopy($srcback, $src,0,0,0,0,$sx,$sy);
if($srcback==NULL){
return 0;
}
$pxl = array(1,1);
for ($y=0; $y<$sy; ++$y){
for($x=0; $x<$sx; ++$x){
$new_r = $new_g = $new_b = 0;
$alpha = imagecolorat($srcback, $pxl[0], $pxl[1]);
$new_a = $alpha >> 24;
for ($j=0; $j<3; ++$j) {
$yv = min(max($y - 1 + $j, 0), $sy - 1);
for ($i=0; $i<3; ++$i) {
$pxl = array(min(max($x - 1 + $i, 0), $sx - 1), $yv);
$rgb = imagecolorat($srcback, $pxl[0], $pxl[1]);
$new_r += (($rgb >> 16) & 0xFF) * $filter[$j][$i];
$new_g += (($rgb >> 8) & 0xFF) * $filter[$j][$i];
$new_b += ($rgb & 0xFF) * $filter[$j][$i];
}
}
$new_r = ($new_r/$filter_div)+$offset;
$new_g = ($new_g/$filter_div)+$offset;
$new_b = ($new_b/$filter_div)+$offset;
$new_r = ($new_r > 255)? 255 : (($new_r < 0)? 0:$new_r);
$new_g = ($new_g > 255)? 255 : (($new_g < 0)? 0:$new_g);
$new_b = ($new_b > 255)? 255 : (($new_b < 0)? 0:$new_b);
$new_pxl = ImageColorAllocateAlpha($src, (int)$new_r, (int)$new_g, (int)$new_b, $new_a);
if ($new_pxl == -1) {
$new_pxl = ImageColorClosestAlpha($src, (int)$new_r, (int)$new_g, (int)$new_b, $new_a);
}
if (($y >= 0) && ($y < $sy)) {
imagesetpixel($src, $x, $y, $new_pxl);
}
}
}
imagedestroy($srcback);
return 1;
}
}
class UnbUi
{
public static $data = array(
'img_url' => 'img/',
'plaintype_help_url' => 'plaintype.html',
'plaintype_help_width' => 600,
'plaintype_help_height' => 500,
);
private static $formHandler = null;
private static $formFields = null;
public static function Init($configFilename = 'unb_config.inc.php')
{
UnbEnvironment::SetupEnvironment();
UnbConfiguration::SetFilename($configFilename);
UnbDatabase::GetInstance()->Open();
UnbSession::EnableSession();
}
private static function EnsureHandleForm()
{
if (!isset(self::$formHandler))
{
self::$formHandler = new UnbFormHandler();
self::$formFields = self::$formHandler->HandleForm();
}
}
public static function GetFormHandler()
{
self::EnsureHandleForm();
return self::$formHandler;
}
public static function GetFormHandlerFields($action = null, $field = null)
{
self::EnsureHandleForm();
if (isset($action))
{
if (isset($field))
{
return self::$formFields['actions'][$action][$field];
}
else
{
return self::$formFields['actions'][$action];
}
}
else
{
return self::$formFields;
}
}
public static function IsFormSuccess()
{
self::EnsureHandleForm();
return isset(self::$formFields['allsuccess']);
}
public static function IsFormError()
{
self::EnsureHandleForm();
return isset(self::$formFields['erroraction']);
}
public static function GetFormErrorMessage()
{
self::EnsureHandleForm();
$errorAction = self::$formFields['erroraction'];
return self::$formFields['actions'][$errorAction]['error'];
}
public static function FormHandlerFieldsXML()
{
$fields = self::GetFormHandlerFields();
header('Content-Type: text/xml');
self::FormHandlerFieldsXML_rec('fields', $fields, 0);
}
private static function FormHandlerFieldsXML_rec($parentKey, $data, $level)
{
$pad = str_repeat("\t", $level);
if (is_numeric($parentKey))
$parentKey = 'item';
if (is_array($data))
{
if (count($data))
{
$isAssoc = false;
foreach (array_keys($data) as $aKey)
if (!is_numeric($aKey))
$isAssoc = true;
if ($isAssoc)
echo $pad . '<' . $parentKey . '>' . "\n";
foreach ($data as $key => $value)
{
if ($isAssoc)
self::FormHandlerFieldsXML_rec($key, $value, $level + 1);
else
self::FormHandlerFieldsXML_rec($parentKey, $value, $level);
}
if ($isAssoc)
echo $pad . '</' . $parentKey . '>' . "\n";
}
}
elseif (is_bool($data))
{
if ($data)
echo $pad . '<' . $parentKey . '/>' . "\n";
}
elseif (is_string($data) && strpos($data, "\n") !== false)
{
echo $pad . '<' . $parentKey . '><![CDATA[' . str_replace(']]>', ']]]]><![CDATA[>', $data) . ']]></' . $parentKey . '>' . "\n";
}
else
{
echo $pad . '<' . $parentKey . '>' . htmlspecialchars($data) . '</' . $parentKey . '>' . "\n";
}
}
public static function ReportInfo($msg, $cssClass = 'informationbox', $style = '')
{
echo '<p class="' . $cssClass . '" style="' . $style . '"><strong>' . nl2br(htmlspecialchars($msg)) . '</strong></p>';
}
public static function ReportSuccess($msg, $cssClass = 'successbox', $style = '')
{
echo '<p class="' . $cssClass . '" style="' . $style . '"><strong>' . nl2br(htmlspecialchars($msg)) . '</strong></p>';
}
public static function ReportError($param, $cssClass = 'errorbox', $style = '')
{
if ($param instanceof UnbSecurityException)
{
echo '<p class="' . $cssClass . '" style="' . $style . '">Sorry, there was a problem processing your request:<br /><strong>Security error: ' . nl2br(htmlspecialchars($param->getMessage())) . '</strong></p>';
}
elseif ($param instanceof Exception)
{
echo '<p class="' . $cssClass . '" style="' . $style . '">Sorry, there was a problem processing your request:<br /><strong>' . get_class($param) . ': ' . nl2br(htmlspecialchars($param->getMessage())) . '</strong></p>';
}
else
{
echo '<p class="' . $cssClass . '" style="' . $style . '">Sorry, there was a problem processing your request:<br /><strong>' . nl2br(htmlspecialchars($param)) . '</strong></p>';
}
}
public static function RedirectAndExit($url)
{
while (ob_get_level())
ob_end_clean();
foreach (headers_list() as $h)
{
if (preg_match('_Content-Encoding:\s*gzip_i', $h))
{
ob_start('ob_gzhandler');
break;
}
}
header('HTTP/1.1 303 See other');
header('Location: ' . $url);
echo '<html><head><title>Page redirection</title>';
echo '<meta http-equiv="refresh" content="0; URL=' . htmlspecialchars($url) . '" />';
echo '</head><body>';
echo '<a href="' . htmlspecialchars($url) . '">Click here if redirection is disabled in your browser.</a>';
echo '</body></html>';
exit();
}
public static function DateTime($time, $seconds = false, $html = false)
{
if (!isset($time) || $time === 0)
return 'No time';
if (!is_int($time))
throw new InvalidArgumentException('Invalid argument type: $time');
$formatDate = 'Y-m-d';
$formatTime = 'H:i';
if ($seconds)
$formatTime .= ':s';
if ($html)
return date($formatDate, $time) . '&nbsp;&nbsp;' . date($formatTime, $time);
else
return date($formatDate . ' ' . $formatTime, $time);
}
public static function RelativeTime($time, $valueOnly = false)
{
if (!isset($time) || $time === 0)
return 'Never';
if (!is_int($time))
throw new InvalidArgumentException('Invalid argument type: $time');
$now = time();
$diff = ($now - $time);
$years = intval($diff / (365.2422 * 86400));
$diff %= 365.2422 * 86400;
$months = intval($diff / (30.436 * 86400));
$diff %= 30.436 * 86400;
$days = intval($diff / 86400);
$diff %= 86400;
$hours = intval($diff / 3600);
$diff %= 3600;
$minutes = intval($diff / 60);
$diff %= 60;
$seconds = $diff;
if ($years >= 5) $out = self::TranslateTimeUnit($years, 'yr');
elseif ($years >= 1) $out = self::CombineTimeUnits($years, 'yr', $months, 'mon');
elseif ($months >= 5) $out = self::TranslateTimeUnit($months, 'mon');
elseif ($months >= 1) $out = self::CombineTimeUnits($months, 'mon', $days, 'day');
elseif ($days >= 5) $out = self::TranslateTimeUnit($days, 'day');
elseif ($days >= 1) $out = self::CombineTimeUnits($days, 'day', $hours, 'hr');
elseif ($hours >= 3) $out = self::TranslateTimeUnit($hours, 'hr');
elseif ($hours >= 1) $out = self::CombineTimeUnits($hours, 'hr', $minutes, 'min');
elseif ($minutes >= 5) $out = self::TranslateTimeUnit($minutes, 'min');
elseif ($minutes >= 1) $out = self::CombineTimeUnits($minutes, 'min', $seconds, 'sec');
else $out = self::TranslateTimeUnit($seconds, 'sec');
if ($valueOnly)
return $out;
else
return $out . ' ago';
}
private static function CombineTimeUnits($count1, $unit1, $count2, $unit2)
{
if ($unit2 == 'sec' && $count2 <= 3 ||
$unit2 == 'min' && $count2 <= 3 ||
$unit2 == 'hr' && $count2 <= 1 ||
$unit2 == 'day' && $count2 <= 2)
{
$count2 = 0;
}
if ($unit2 == 'sec' && $count2 >= 57 ||
$unit2 == 'min' && $count2 >= 57 ||
$unit2 == 'hr' && $count2 >= 23 ||
$unit2 == 'day' && $count2 >= 28)
{
$count1++;
$count2 = 0;
}
if (!$count2)
return self::TranslateTimeUnit($count1, $unit1);
else
return self::TranslateTimeUnit($count1, $unit1) . /*' and '*/ ' ' . self::TranslateTimeUnit($count2, $unit2);
}
private static function TranslateTimeUnit($count, $unit)
{
switch ($unit)
{
case 'sec':
if ($count == 1) return '1 second';
return $count . ' seconds';
case 'min':
if ($count == 1) return '1 minute';
return $count . ' minutes';
case 'hr':
if ($count == 1) return '1 hour';
return $count . ' hours';
case 'day':
if ($count == 1) return '1 day';
return $count . ' days';
case 'mon':
if ($count == 1) return '1 month';
return $count . ' months';
case 'yr':
if ($count == 1) return '1 year';
return $count . ' years';
}
return $count . ' ¿' . $unit . '?';
}
public static function GetUserList($keys, $includeSystem = true)
{
if (!is_array($keys))
throw new InvalidArgumentException('Invalid argument type: $keys');
$list = '';
foreach ($keys as $key)
{
if (!$includeSystem && $key <= 9) continue;
$keyUser = UnbUser::FindByUserId($key);
if (isset($keyUser))
$list .= $keyUser->GetDisplayName() . "\n";
else
$list .= '#' . $key . "\n";
}
return rtrim($list);
}
public static function BeginForm($actionUrl = '', $enableUploads = false, $id = null)
{
if (!is_string($actionUrl))
throw new InvalidArgumentException('Invalid argument type: $actionUrl');
$actionUrl = trim($actionUrl);
$id = self::GetId($id, 'unbForm');
echo '<form id="' . $id . '" action="' . htmlspecialchars($actionUrl) . '" method="post"' .
($enableUploads ? ' enctype="multipart/form-data"' : '') . '>';
self::FormUrlKey();
}
public static function EndForm()
{
echo '</form>';
}
public static function FormUrlKey()
{
echo '<input type="hidden" name="urlkey" value="' . UnbSession::GetUrlKey() . '" />';
}
public static function FormUser($userId)
{
if (!is_int($userId))
throw new InvalidArgumentException('Invalid argument type: $userId');
if ($userId <= 0)
throw new InvalidArgumentException('Argument out of range: $userId');
echo '<input type="hidden" name="userid" value="' . $userId . '" />';
}
public static function FormMsgRev($messageId, $revisionNumber)
{
if (!is_int($messageId))
throw new InvalidArgumentException('Invalid argument type: $messageId');
if ($messageId <= 0)
throw new InvalidArgumentException('Argument out of range: $messageId');
if (!is_int($revisionNumber))
throw new InvalidArgumentException('Invalid argument type: $revisionNumber');
if ($revisionNumber <= 0)
throw new InvalidArgumentException('Argument out of range: $revisionNumber');
echo '<input type="hidden" name="messageid" value="' . $messageId . '" />';
echo '<input type="hidden" name="revisionnumber" value="' . $revisionNumber . '" />';
}
public static function FormReplyToMsg($messageId)
{
if (isset($messageId))
{
if (!is_int($messageId))
throw new InvalidArgumentException('Invalid argument type: $messageId');
if ($messageId < 0)
throw new InvalidArgumentException('Argument out of range: $messageId');
if ($messageId == 0)
$messageId = null;
}
if (isset($messageId))
{
echo '<input type="hidden" name="replyto_messageid" value="' . $messageId . '" />';
}
}
public static function FormTag($tagId)
{
if (!is_int($tagId))
throw new InvalidArgumentException('Invalid argument type: $tagId');
if ($tagId <= 0)
throw new InvalidArgumentException('Argument out of range: $tagId');
echo '<input type="hidden" name="tagid" value="' . $tagId . '" />';
}
private static function InsertDefaultMessage($id, $defaultMessage, $emptyFieldName = '')
{
echo '<script type="text/javascript">//<![CDATA[
(function()
{
var o = $("#' . $id . '");
var ef = $("input[name=\"' . $emptyFieldName . '\"]" , o.form);
DOMEvents.add(o, "focus", function (e)
{
if (DOM.hasClass(e.target, "default-message"))
{
e.target.value = "";
DOM.removeClass(e.target, "default-message");
if (ef) ef.value = "0";
}
});
var blurFn = function (e)
{
if (e.target.value.trim() == "" ||
ef && ef.value == "1")
{
DOM.addClass(e.target, "default-message");
e.target.value = "' . str_replace('"', '\\"', $defaultMessage) . '";
if (ef) ef.value = "1";
}
};
DOMEvents.add(o, "blur", blurFn);
blurFn({target: o});
})();
//]]></script>';
}
private static function GetId($id, $defaultId)
{
if (isset($id))
{
if (!is_string($id))
throw new InvalidArgumentException('Invalid argument type: $id');
$id = trim($id);
if (!strlen($id))
$id = null;
}
if (!isset($id))
{
$id = $defaultId;
}
return $id;
}
private static function GetValue($fieldName, $defaultValue = '', $emptyFieldName = null)
{
if ($_POST[$emptyFieldName])
return '';
return htmlspecialchars($_POST ? $_POST[$fieldName] : $defaultValue);
}
public static function UserLoginName($user = null, $id = null, $style = '')
{
$defaultValue = '';
if (isset($user))
{
if (!($user instanceof UnbUser))
throw new InvalidArgumentException('Invalid argument type: $user');
$defaultValue = $user->GetLoginName();
}
$id = self::GetId($id, 'userLoginNameTb');
echo '<input type="text" id="' . $id . '" name="user_loginname" size="20" style="' . htmlspecialchars($style) . '" value="' . self::GetValue('user_loginname', $defaultValue) . '" />
<input type="hidden" name="action[]" value="set_user_loginname" />';
}
public static function UserLoginPassword($user = null, $id = null, $style = '')
{
$defaultValue = '';
if (isset($user))
{
if (!($user instanceof UnbUser))
throw new InvalidArgumentException('Invalid argument type: $user');
$defaultValue = $user->GetLoginPassword() ? '          ' : '';
}
$id = self::GetId($id, 'userLoginPasswordTb');
echo '<input type="password" ' . (isset($user) || $_POST ? 'autocomplete="off" ' : '') . 'id="' . $id . '" name="user_loginpassword" size="20" style="' . htmlspecialchars($style) . '" value="' . self::GetValue('user_loginpassword', $defaultValue) . '" />
<input type="hidden" name="action[]" value="set_user_loginpassword" />';
}
public static function UserRepeatPassword($id = null, $style = '')
{
$id = self::GetId($id, 'userRepeatPasswordTb');
echo '<input type="password" id="' . $id . '" autocomplete="off" name="user_repeatpassword" size="20" style="' . htmlspecialchars($style) . '" value="' . self::GetValue('user_repeatpassword') . '" />';
}
public static function UserCurrentPassword($id = null, $style = '')
{
$id = self::GetId($id, 'userCurrentPasswordTb');
echo '<input type="password" id="' . $id . '" autocomplete="off" name="user_currentpassword" size="20" style="' . htmlspecialchars($style) . '" value="' . self::GetValue('user_currentpassword') . '" />';
}
public static function UserDisplayName($user = null, $id = null, $style = '')
{
$defaultValue = '';
if (isset($user))
{
if (!($user instanceof UnbUser))
throw new InvalidArgumentException('Invalid argument type: $user');
$defaultValue = $user->GetDisplayName();
}
$id = self::GetId($id, 'userDisplayNameTb');
echo '<input type="text" id="' . $id . '" name="user_displayname" size="25" style="' . htmlspecialchars($style) . '" value="' . self::GetValue('user_displayname', $defaultValue) . '" />';
echo '<input type="hidden" name="action[]" value="set_user_displayname" />';
}
public static function IsCaptchaEnabled()
{
return UnbConfiguration::Get('captcha.enabled', true);
}
public static function Captcha($id = null, $text1 = null, $text2 = null)
{
$id = self::GetId($id, 'captchaTb');
if (!isset($text1))
$text1 = 'Please type all the <strong>letters only</strong> of the word<br />from the image into the text field below.<br />';
if (!isset($text2))
$text2 = 'Letters go in here: &nbsp; <small class="light">Lower case is okay.</small><br />';
echo $text1 . '
<img src="captcha.php?r=' . mt_rand(1, 9999999) . '" alt="CAPTCHA image" style="margin: 0.2em 0;" /><br />
' . $text2 . '
<img src="' . self::$data['img_url'] . 'btn-downhere.png" alt="" />
<input type="text" id="' . $id . '" name="captcha_word" autocomplete="off" size="15" value="' . self::GetValue('captcha_word') . '" />';
}
public static function GetUserAdditionalKeys($user)
{
if (!($user instanceof UnbUser))
throw new InvalidArgumentException('Invalid argument type: $user');
return self::GetUserList($user->GetAdditionalKeys(), false);
}
public static function UserAdditionalKeysList($user = null, $id = null, $style = '', $defaultMessage = '')
{
$additionalKeysList = '';
if (isset($user))
{
if (!($user instanceof UnbUser))
throw new InvalidArgumentException('Invalid argument type: $user');
$additionalKeysList = self::GetUserAdditionalKeys($user);
}
$id = self::GetId($id, 'userAdditionalKeysListTb');
echo '<textarea id="' . $id . '" name="user_additionalkeyslist" cols="30" rows="3" style="' . htmlspecialchars($style) . '">' . self::GetValue('user_additionalkeyslist', $additionalKeysList, 'user_additionalkeys_empty') . '</textarea>
<input type="hidden" name="user_additionalkeys_empty" value="0" />
<input type="hidden" name="action[]" value="set_user_additionalkeys" />';
if (strlen($defaultMessage))
self::InsertDefaultMessage($id, $defaultMessage, 'user_additionalkeys_empty');
}
public static function UserAdministrator($user = null, $id = null)
{
$defaultValue = false;
if (isset($user))
{
if (!($user instanceof UnbUser))
throw new InvalidArgumentException('Invalid argument type: $user');
$defaultValue = $user->IsAdministrator();
}
$id = self::GetId($id, 'userAdministratorCb');
echo '<label><input type="checkbox" id="' . $id . '" name="user_is_administrator" value="1" ' . (($_POST ? $_POST['user_is_administrator'] : $defaultValue) ? 'checked="checked"' : '') . ' /> Administrator</label>
<input type="hidden" name="action[]" value="set_user_administrator" />';
}
public static function UserModerator($user = null, $id = null)
{
$defaultValue = false;
if (isset($user))
{
if (!($user instanceof UnbUser))
throw new InvalidArgumentException('Invalid argument type: $user');
$defaultValue = $user->IsModerator();
}
$id = self::GetId($id, 'userModeratorCb');
echo '<label><input type="checkbox" id="' . $id . '" name="user_is_moderator" value="1" ' . (($_POST ? $_POST['user_is_moderator'] : $defaultValue) ? 'checked="checked"' : '') . ' /> Moderator</label>
<input type="hidden" name="action[]" value="set_user_moderator" />';
}
public static function UserTrusted($user = null, $id = null)
{
$defaultValue = false;
if (isset($user))
{
if (!($user instanceof UnbUser))
throw new InvalidArgumentException('Invalid argument type: $user');
$defaultValue = $user->IsTrusted();
}
$id = self::GetId($id, 'userTrustedCb');
echo '<label><input type="checkbox" id="' . $id . '" name="user_is_trusted" value="1" ' . (($_POST ? $_POST['user_is_trusted'] : $defaultValue) ? 'checked="checked"' : '') . ' /> Trusted user</label>
<input type="hidden" name="action[]" value="set_user_trusted" />';
}
public static function UserAlterVcardEntry($type, $entry)
{
if (!self::$data['action_set_user_vcard'])
{
echo '<input type="hidden" name="action[]" value="set_user_vcard" />';
self::$data['action_set_user_vcard'] = true;
}
echo '<input type="text" name="vcard_data_' . $type . '" size="30" value="' . self::GetValue('vcard_data_' . $type, $entry['Data']) . '" /> &nbsp;
Visibility: <select name="vcard_visibility_' . $type . '">
<option value="' . UnbUserVcardVisibility::Public_ . '" ' . (($_POST ? $_POST['vcard_visibility_' . $type] : $entry['Visibility']) == UnbUserVcardVisibility::Public_ ? 'selected="selected"' : '') . '>Public</option>
<option value="' . UnbUserVcardVisibility::Internal . '" ' . (($_POST ? $_POST['vcard_visibility_' . $type] : $entry['Visibility']) == UnbUserVcardVisibility::Internal ? 'selected="selected"' : '') . '>Internal</option>
<option value="' . UnbUserVcardVisibility::Confidential . '" ' . (($_POST ? $_POST['vcard_visibility_' . $type] : $entry['Visibility']) == UnbUserVcardVisibility::Confidential ? 'selected="selected"' : '') . '>Confidential</option>
<option value="' . UnbUserVcardVisibility::Private_ . '" ' . (($_POST ? $_POST['vcard_visibility_' . $type] : $entry['Visibility']) == UnbUserVcardVisibility::Private_ ? 'selected="selected"' : '') . '>Private</option>
</select> &nbsp;
<label><input type="checkbox" name="vcard_remove_' . $type . '" value="1" ' . ($_POST['vcard_remove_' . $type] ? 'checked="checked"' : '') . ' /> Remove this entry</label>';
}
public static function UserVcardTypes()
{
echo '<select name="vcard_newtype">';
$types = array(
'',
'Birthday',
'EMail',
'ICQ',
'Jabber',
'Location',
'MSN',
'RealName',
'Slogan',
'Website',
);
foreach ($types as $type)
{
echo '<option value="' . $type . '" ' . ($type == $_POST['vcard_newtype'] ? ' selected="selected"' : '') . '>' . $type . '</option>';
}
echo '</select>';
}
public static function UserAddVcardEntry()
{
if (!self::$data['action_set_user_vcard'])
{
echo '<input type="hidden" name="action[]" value="set_user_vcard" />';
self::$data['action_set_user_vcard'] = true;
}
echo '<input type="text" name="vcard_newdata" size="30" value="' . self::GetValue('vcard_newdata') . '" /> &nbsp;
Visibility: <select name="vcard_newvisibility">
<option value="' . UnbUserVcardVisibility::Public_ . '" ' . ($_POST['vcard_newvisibility'] == UnbUserVcardVisibility::Public_ ? 'selected="selected"' : '') . '>Public</option>
<option value="' . UnbUserVcardVisibility::Internal . '" ' . ($_POST['vcard_newvisibility'] == UnbUserVcardVisibility::Internal ? 'selected="selected"' : '') . '>Internal</option>
<option value="' . UnbUserVcardVisibility::Confidential . '" ' . ($_POST['vcard_newvisibility'] == UnbUserVcardVisibility::Confidential ? 'selected="selected"' : '') . '>Confidential</option>
<option value="' . UnbUserVcardVisibility::Private_ . '" ' . ($_POST['vcard_newvisibility'] == UnbUserVcardVisibility::Private_ ? 'selected="selected"' : '') . '>Private</option>
</select>';
}
public static function MsgTree($messageId, $selectedMessageId = 0, $level = 0, $seenMessageIds = null)
{
if (!is_int($messageId))
throw new InvalidArgumentException('Invalid argument type: $messageId');
if ($messageId <= 0)
throw new InvalidArgumentException('Argument out of range: $messageId');
if (!isset($seenMessageIds)) $seenMessageIds = array();
$msg = UnbMessage::FindByMessageId($messageId);
if (!isset($msg)) return;
$flags = $msg->GetModerationFlags();
$searchRev = $msg->GetSearchRevision(UnbMessageRevisionContentLevel::Subject, true);
echo '<div class="messageentry' . ($messageId == $selectedMessageId ? ' selected' : '') . '">';
echo '<div class="subjectline">';
if ($searchRev->GetContentLevel() >= UnbMessageRevisionContentLevel::Subject)
{
echo '<a href="view_message.php?messageid=' . $msg->GetMessageId() . '">';
if (strlen($searchRev->GetSubject()))
echo htmlspecialchars($searchRev->GetSubject());
else
echo '<em>No subject set</em>';
echo '</a>';
}
else
{
echo '<em>No subject accessible</em>';
}
echo '</div>';
echo '<div class="infoline" style="margin-bottom: 0.4em;">';
echo 'by <strong>' . $msg->GetOwnerUser()->GetDisplayName() . '</strong>';
$msgCreatedTime = $msg->GetCreatedTime();
echo ', <span title="' . self::DateTime($msgCreatedTime, true, true) . '">' . self::RelativeTime($msgCreatedTime) . '</span>';
$flags_str = '';
if ($searchRev->GetModerationState() == UnbMessageRevisionModerationState::WaitingForApproval)
$flags_str .= ' Unapproved';
if ($flags & UnbMessageModerationFlag::Locked || $searchRev->GetModerationState() == UnbMessageRevisionModerationState::Locked)
$flags_str .= ' Locked';
if ($flags & UnbMessageModerationFlag::Hidden)
$flags_str .= ' Hidden';
if ($flags_str)
echo ' – <span class="modwarning">' . trim($flags_str) . '</span>';
echo '</div>';
echo '</div>';
if (in_array($messageId, $seenMessageIds))
{
echo '<div style="border-left: solid 1px #DDDDDD; padding-left: 1.5em;">';
echo '<em style="color: red;">Reference loop!</em>';
echo '</div>';
return;
}
$seenMessageIds[] = $messageId;
$searchParameters = array();
$searchParameters['sort-createdtime'] = true;
$searchParameters['primary-reference'] = $messageId;
$messageIds = UnbSearch::Query($searchParameters);
if ($messageIds)
{
echo '<div style="border-left: solid 1px #DDDDDD; padding-left: 1.5em;">';
foreach ($messageIds as $messageId)
{
self::MsgTree($messageId, $selectedMessageId, $level + 1, $seenMessageIds);
}
echo '</div>';
}
elseif (!$level)
{
echo '<em>No replies</em>';
}
}
public static function GetMsgRev($messageId, $revNum = null, &$msg, &$rev, &$err, $withHtml = false)
{
if (!is_int($messageId))
throw new InvalidArgumentException('Invalid argument type: $messageId');
if ($messageId <= 0)
throw new InvalidArgumentException('Argument out of range: $messageId');
if (isset($revNum))
{
if (!is_int($revNum))
throw new InvalidArgumentException('Invalid argument type: $revNum');
if ($revNum < 0)
throw new InvalidArgumentException('Argument out of range: $revNum');
}
$msg = UnbMessage::FindByMessageId($messageId);
if (!isset($msg))
{
$err = 'Invalid message ID';
return false;
}
else
{
$cl = $withHtml ? UnbMessageRevisionContentLevel::HtmlContent : UnbMessageRevisionContentLevel::Content;
if ($revNum > 0)
$rev = $msg->GetRevision($revNum, $cl, true);
else
$rev = $msg->GetSearchRevision($cl, true);
if (!isset($rev))
{
$err = 'Invalid revision number';
return false;
}
}
return true;
}
public static function GetQuotedSubject($msg, $rev)
{
if (isset($msg) ^ isset($rev))
throw new InvalidArgumentException('Invalid arguments: Either both $msg and $rev must be set or none at all.');
if (isset($msg))
{
if (!($msg instanceof UnbMessage))
throw new InvalidArgumentException('Invalid argument type: $msg');
}
if (isset($rev))
{
if (!($rev instanceof UnbMessageRevision))
throw new InvalidArgumentException('Invalid argument type: $rev');
}
$quotation = '';
if (isset($msg) && isset($rev))
{
$content = $rev->GetSubject();
$content = preg_replace('_^(re:\s*)+_i', '', $content);
$quotation = 'Re: ' . $content;
}
return $quotation;
}
public static function GetQuotedMsg($msg, $rev)
{
if (isset($msg) ^ isset($rev))
throw new InvalidArgumentException('Invalid arguments: Either both $msg and $rev must be set or none at all.');
if (isset($msg))
{
if (!($msg instanceof UnbMessage))
throw new InvalidArgumentException('Invalid argument type: $msg');
}
if (isset($rev))
{
if (!($rev instanceof UnbMessageRevision))
throw new InvalidArgumentException('Invalid argument type: $rev');
}
$quotation = '';
if (isset($msg) && isset($rev))
{
$content = $rev->GetContent();
if (strlen($content))
{
$content = preg_replace('_^>_m', '>>', $content);
$content = preg_replace('_^$_m', '>', $content);
$content = preg_replace('_^(?!>|$)_m', '> ', $content);
$quotation = $msg->GetOwnerUser()->GetDisplayName() . ' wrote ' .
self::RelativeTime($msg->GetCreatedTime(), true)  . ' before:' . "\n" . $content;
}
}
return $quotation;
}
public static function MsgApproveRevLink($rev, $htmlTemplate)
{
if (!($rev instanceof UnbMessageRevision))
throw new InvalidArgumentException('Invalid argument type: $rev');
if (!is_string($htmlTemplate))
throw new InvalidArgumentException('Invalid argument type: $htmlTemplate');
if ($rev->GetModerationState() == UnbMessageRevisionModerationState::WaitingForApproval)
{
echo str_replace(
'%s',
'action=moderate_messagerevision&amp;urlkey=' . UnbSession::GetUrlKey() . '&amp;messageid=' . $rev->GetMessageId() . '&amp;revisionnumber=' . $rev->GetRevisionNumber() . '&amp;message_modstate=' . UnbMessageRevisionModerationState::Approved,
$htmlTemplate);
}
}
public static function MsgLockRevLink($rev, $htmlTemplate)
{
if (!($rev instanceof UnbMessageRevision))
throw new InvalidArgumentException('Invalid argument type: $rev');
if (!is_string($htmlTemplate))
throw new InvalidArgumentException('Invalid argument type: $htmlTemplate');
if ($rev->GetModerationState() == UnbMessageRevisionModerationState::WaitingForApproval ||
$rev->GetModerationState() == UnbMessageRevisionModerationState::Approved)
{
echo str_replace(
'%s',
'action=moderate_messagerevision&amp;urlkey=' . UnbSession::GetUrlKey() . '&amp;messageid=' . $rev->GetMessageId() . '&amp;revisionnumber=' . $rev->GetRevisionNumber() . '&amp;message_modstate=' . UnbMessageRevisionModerationState::Locked,
$htmlTemplate);
}
}
public static function MsgUnlockRevLink($rev, $htmlTemplate)
{
if (!($rev instanceof UnbMessageRevision))
throw new InvalidArgumentException('Invalid argument type: $rev');
if (!is_string($htmlTemplate))
throw new InvalidArgumentException('Invalid argument type: $htmlTemplate');
if ($rev->GetModerationState() == UnbMessageRevisionModerationState::Locked)
{
echo str_replace(
'%s',
'action=moderate_messagerevision&amp;urlkey=' . UnbSession::GetUrlKey() . '&amp;messageid=' . $rev->GetMessageId() . '&amp;revisionnumber=' . $rev->GetRevisionNumber() . '&amp;message_modstate=' . UnbMessageRevisionModerationState::Approved,
$htmlTemplate);
}
}
public static function CanAlterMsg($msg, $rev)
{
if (!($msg instanceof UnbMessage))
throw new InvalidArgumentException('Invalid argument type: $msg');
if (!($rev instanceof UnbMessageRevision))
throw new InvalidArgumentException('Invalid argument type: $rev');
return $rev->GetContentLevel() >= UnbMessageRevisionContentLevel::Content && $msg->IsAlterAllowed();
}
public static function CanReplyMsg($msg, $rev)
{
if (!($msg instanceof UnbMessage))
throw new InvalidArgumentException('Invalid argument type: $msg');
if (!($rev instanceof UnbMessageRevision))
throw new InvalidArgumentException('Invalid argument type: $rev');
return $rev->GetContentLevel() >= UnbMessageRevisionContentLevel::Content && $msg->IsReplyAllowed();
}
public static function CanSeeSubject($rev)
{
if (!($rev instanceof UnbMessageRevision))
throw new InvalidArgumentException('Invalid argument type: $rev');
return $rev->GetContentLevel() >= UnbMessageRevisionContentLevel::Subject;
}
public static function IsMsgOwner($msg)
{
if (!($msg instanceof UnbMessage))
throw new InvalidArgumentException('Invalid argument type: $msg');
return UnbSession::GetSessionUserId() == $msg->GetOwnerUserId();
}
public static function MsgSubject($newMessage = true, $msg = null, $rev = null, $id = null, $style = '')
{
$id = self::GetId($id, 'msgSubjectTb');
if ($newMessage)
{
if (!self::$data['action_create_message_set'])
{
echo '<input type="hidden" name="action[]" value="create_message" />';
self::$data['action_create_message_set'] = true;
}
$defaultValue = self::GetQuotedSubject($msg, $rev);
}
else
{
if (!($rev instanceof UnbMessageRevision))
throw new InvalidArgumentException('Invalid argument type: $rev');
if (!self::$data['action_alter_message_set'])
{
echo '<input type="hidden" name="action[]" value="alter_message" />';
self::$data['action_alter_message_set'] = true;
}
$defaultValue = $rev->GetSubject();
echo '<input type="hidden" name="message_subject_hash" value="' . md5($defaultValue) . '" />';
}
echo '<input type="text" id="' . $id . '" name="message_subject" size="60" style="' . htmlspecialchars($style) . '" value="' . self::GetValue('message_subject', $defaultValue) . '" />';
}
public static function MsgContent($newMessage = true, $msg = null, $rev = null, $id = null, $style = '')
{
$id = self::GetId($id, 'msgContentTb');
if ($newMessage)
{
if (!self::$data['action_create_message_set'])
{
echo '<input type="hidden" name="action[]" value="create_message" />';
self::$data['action_create_message_set'] = true;
}
$defaultValue = self::GetQuotedMsg($msg, $rev);
}
else
{
if (!($rev instanceof UnbMessageRevision))
throw new InvalidArgumentException('Invalid argument type: $rev');
if (!self::$data['action_alter_message_set'])
{
echo '<input type="hidden" name="action[]" value="alter_message" />';
self::$data['action_alter_message_set'] = true;
}
$defaultValue = $rev->GetContent();
echo '<input type="hidden" name="message_content_hash" value="' . md5($defaultValue) . '" />';
}
echo '<textarea id="' . $id . '" name="message_content" cols="60" rows="15" style="' . htmlspecialchars($style) . '">' . self::GetValue('message_content', $defaultValue) . '</textarea>';
}
public static function MsgSummary($id = null, $style = '')
{
$id = self::GetId($id, 'msgSummaryTb');
if (!self::$data['action_alter_message_set'])
{
echo '<input type="hidden" name="action[]" value="alter_message" />';
self::$data['action_alter_message_set'] = true;
}
echo '<input type="text" id="' . $id . '" name="message_summary" size="60" style="' . htmlspecialchars($style) . '" value="' . self::GetValue('message_summary') . '" />';
}
public static function MsgAttach($newMessage = true)
{
if ($newMessage)
{
if (!self::$data['action_create_message_set'])
{
echo '<input type="hidden" name="action[]" value="create_message" />';
self::$data['action_create_message_set'] = true;
}
echo 'Add attached file: <input type="file" name="message_filedata" size="50" />
<input type="hidden" name="set_messagerevision_filedata" value="1" />';
}
else
{
if (!self::$data['action_alter_message_set'])
{
echo '<input type="hidden" name="action[]" value="alter_message" />';
self::$data['action_alter_message_set'] = true;
}
echo 'Replace/add attached uploaded file: <input type="file" name="message_filedata" size="50" /><br />
<label><input type="checkbox" name="message_cleardata" value="1" ' . ($_POST['message_cleardata'] ? 'checked="checked"' : '') . ' /> Clear attached file</label>
<input type="hidden" name="set_messagerevision_filedata" value="1" />';
}
}
public static function MsgPageName($msg = null, $id = null, $style = '')
{
$defaultValue = '';
if (isset($msg))
{
if (!($msg instanceof UnbMessage))
throw new InvalidArgumentException('Invalid argument type: $msg');
$defaultValue = $msg->GetPageName();
}
$id = self::GetId($id, 'msgPageNameTb');
echo '<input type="text" id="' . $id . '" name="message_pagename" size="40" style="' . htmlspecialchars($style) . '" value="' . self::GetValue('message_pagename', $defaultValue) . '" />
<input type="hidden" name="action[]" value="set_message_pagename" />';
}
public static function MsgDEP($param, $id = null)
{
$defaultValue = false;
if ($param instanceof UnbMessage)
$defaultValue = $param->IsEntryPoint();
elseif (is_bool($param))
$defaultValue = $param;
else
throw new InvalidArgumentException('Invalid argument type: $param');
$id = self::GetId($id, 'msgDEPCb');
echo '<label><input type="checkbox" id="' . $id . '" name="message_entrypoint" value="1" ' . (($_POST ? $_POST['message_entrypoint'] : $defaultValue) ? 'checked="checked"' : '') . ' /> This message is a discussion entry point.</label>
<input type="hidden" name="action[]" value="set_message_entrypoint" />';
}
private static function MsgTags_ListTags($tags, $msgTags, $parent = 0, $level = 0)
{
$divOnThisLevel = false;
foreach ($tags as $tag)
{
if ($tag->GetParentTagId() != $parent) continue;
if ($level > 0 && !$divOnThisLevel)
{
echo '<div style="border-left: solid 1px #dddddd; padding-left: 1.5em; margin-top: ' . (0.2 / $level) . 'em; margin-bottom: ' . (0.4 / $level) . 'em;">';
$divOnThisLevel = true;
}
if ($tag->GetFlags() & UnbTagFlag::NotSelectable)
{
echo '<div style="padding-left: 20px;">' . htmlspecialchars($tag->GetTagName()) . '</div>';
}
else
{
$selected = false;
$userData = '';
foreach ($msgTags as $msgTag)
{
if (is_int($msgTag) && $msgTag == $tag->GetTagId())
{
$selected = true;
break;
}
if (is_array($msgTag) && $msgTag['TagId'] == $tag->GetTagId())
{
$selected = true;
$userData = $msgTag['TagData'];
}
}
echo '<label><input type="checkbox" name="message_tagid[]" value="' . $tag->GetTagId() . '"' . ($selected ? ' checked' : '') . ' /> ' . htmlspecialchars($tag->GetTagName()) . '</label>';
if ($tag->GetFlags() & UnbTagFlag::Invisible)
echo ' [invisible]';
if ($tag->GetFlags() & UnbTagFlag::UserData)
{
echo ':&nbsp; <input type="text" name="message_tagdata_' . $tag->GetTagId() . '" size="30" value="' . htmlspecialchars($userData) . '" />';
}
if (strlen($tag->GetDescription()))
{
echo ' &nbsp;–&nbsp; <small>' . htmlspecialchars($tag->GetDescription()) . '</small>';
}
echo '<br />';
}
self::MsgTags_ListTags($tags, $msgTags, $tag->GetTagId(), $level + 1);
}
if ($divOnThisLevel)
{
echo '</div>';
}
}
public static function MsgTags($rev = null)
{
$msgTags = array();
if (isset($rev))
{
if (!($rev instanceof UnbMessageRevision))
throw new InvalidArgumentException('Invalid argument type: $rev');
$msgTags = $rev->GetTagIds();
}
$tags = UnbTag::Find();
self::MsgTags_ListTags($tags, $msgTags);
echo '<input type="hidden" name="action[]" value="set_messagerevision_tags" />';
}
public static function MsgModerateMsg($msg = null)
{
$locked = false;
$hidden = false;
$enforceApproval = false;
if (isset($msg))
{
if (!($msg instanceof UnbMessage))
throw new InvalidArgumentException('Invalid argument type: $msg');
$locked = $msg->GetModerationFlags() & UnbMessageModerationFlag::Locked;
$hidden = $msg->GetModerationFlags() & UnbMessageModerationFlag::Hidden;
$enforceApproval = $msg->GetModerationFlags() & UnbMessageModerationFlag::EnforceApproval;
}
echo '<label><input type="checkbox" name="message_locked" value="1" ' . (($_POST ? $_POST['message_locked'] : $locked) ? 'checked="checked"' : '') . ' /> Lock this message</label><br />
<label><input type="checkbox" name="message_hidden" value="1" ' . (($_POST ? $_POST['message_hidden'] : $hidden) ? 'checked="checked"' : '') . ' /> Hide this message</label><br />
<label><input type="checkbox" name="message_enforce_approval" value="1" ' . (($_POST ? $_POST['message_enforce_approval'] : $enforceApproval) ? 'checked="checked"' : '') . ' /> Enforce approval of all replies to this message</label><br />
<input type="hidden" name="action[]" value="moderate_message" />';
}
public static function GetMsgReadAccessKeys($msg)
{
if (!($msg instanceof UnbMessage))
throw new InvalidArgumentException('Invalid argument type: $msg');
return self::GetUserList($msg->GetReadAccessKeys());
}
public static function MsgReadAccessList($msg = null, $id = null, $style = '', $defaultMessage = '')
{
$readAccessList = '';
if (isset($msg))
{
if (!($msg instanceof UnbMessage))
throw new InvalidArgumentException('Invalid argument type: $msg');
$readAccessList = self::GetMsgReadAccessKeys($msg);
}
$id = self::GetId($id, 'msgReadAccessListTb');
echo '<textarea id="' . $id . '" name="message_readaccesslist" cols="30" rows="3" style="' . htmlspecialchars($style) . '">' . self::GetValue('message_readaccesslist', $readAccessList, 'message_readaccess_empty') . '</textarea>
<input type="hidden" name="message_readaccess_empty" value="0" />
<input type="hidden" name="action[]" value="set_message_readaccess" />';
if (strlen($defaultMessage))
self::InsertDefaultMessage($id, $defaultMessage, 'message_readaccess_empty');
}
public static function GetMsgAlterAccessKeys($msg)
{
if (!($msg instanceof UnbMessage))
throw new InvalidArgumentException('Invalid argument type: $msg');
return self::GetUserList($msg->GetAlterAccessKeys());
}
public static function MsgAlterAccessList($msg = null, $id = null, $style = '', $defaultMessage = '')
{
$alterAccessList = '';
if (isset($msg))
{
if ($msg instanceof UnbMessage)
$alterAccessList = self::GetMsgAlterAccessKeys($msg);
elseif (is_array($msg))
$alterAccessList = self::GetUserList($msg);
else
throw new InvalidArgumentException('Invalid argument type: $msg');
}
$id = self::GetId($id, 'msgAlterAccessListTb');
echo '<textarea id="' . $id . '" name="message_alteraccesslist" cols="30" rows="3" style="' . htmlspecialchars($style) . '">' . self::GetValue('message_alteraccesslist', $alterAccessList, 'message_alteraccess_empty') . '</textarea>
<input type="hidden" name="message_alteraccess_empty" value="0" />
<input type="hidden" name="action[]" value="set_message_alteraccess" />';
if (strlen($defaultMessage))
self::InsertDefaultMessage($id, $defaultMessage, 'message_alteraccess_empty');
}
public static function GetMsgReplyAccessKeys($msg)
{
if (!($msg instanceof UnbMessage))
throw new InvalidArgumentException('Invalid argument type: $msg');
return self::GetUserList($msg->GetReplyAccessKeys());
}
public static function MsgReplyAccessList($msg = null, $id = null, $style = '', $defaultMessage = '')
{
$replyAccessList = '';
if (isset($msg))
{
if (!($msg instanceof UnbMessage))
throw new InvalidArgumentException('Invalid argument type: $msg');
$replyAccessList = self::GetMsgReplyAccessKeys($msg);
}
$id = self::GetId($id, 'msgReplyAccessListTb');
echo '<textarea id="' . $id . '" name="message_replyaccesslist" cols="30" rows="3" style="' . htmlspecialchars($style) . '">' . self::GetValue('message_replyaccesslist', $replyAccessList, 'message_replyaccess_empty') . '</textarea>
<input type="hidden" name="message_replyaccess_empty" value="0" />
<input type="hidden" name="action[]" value="set_message_replyaccess" />';
if (strlen($defaultMessage))
self::InsertDefaultMessage($id, $defaultMessage, 'message_replyaccess_empty');
}
public static function ButtonEm($textboxId = null, $text = null, $style = '')
{
$textboxId = self::GetId($textboxId, 'msgContentTb');
if (!isset($text))
$text = 'Emphasis';
echo '<input type="button" value="' . htmlspecialchars($text) . '" class="icon13 em" style="font-style: italic;' . $style . '" onclick="PlainTypeEm(\'' . $textboxId . '\')" />';
}
public static function ButtonStrong($textboxId = null, $text = null, $style = '')
{
$textboxId = self::GetId($textboxId, 'msgContentTb');
if (!isset($text))
$text = 'Strong emphasis';
echo '<input type="button" value="' . htmlspecialchars($text) . '" class="icon13 strong" style="font-weight: bold;' . $style . '" onclick="PlainTypeStrong(\'' . $textboxId . '\')" />';
}
public static function ButtonQuote($textboxId = null, $text = null, $style = '')
{
$textboxId = self::GetId($textboxId, 'msgContentTb');
if (!isset($text))
$text = 'Quotation';
echo '<input type="button" value="' . htmlspecialchars($text) . '" class="icon13 quote" style="' . $style . '" onclick="PlainTypeQuote(\'' . $textboxId . '\')" />';
}
public static function ButtonIndent($textboxId = null, $text = null, $style = '')
{
$textboxId = self::GetId($textboxId, 'msgContentTb');
if (!isset($text))
$text = 'Indentation';
echo '<input type="button" value="' . htmlspecialchars($text) . '" class="icon13 indent" style="' . $style . '" onclick="PlainTypeIndent(\'' . $textboxId . '\')" />';
}
public static function ButtonUl($textboxId = null, $text = null, $style = '')
{
$textboxId = self::GetId($textboxId, 'msgContentTb');
if (!isset($text))
$text = 'List';
echo '<input type="button" value="' . htmlspecialchars($text) . '" class="icon13 ul" style="' . $style . '" onclick="PlainTypeUList(\'' . $textboxId . '\')" />';
}
public static function ButtonOl($textboxId = null, $text = null, $style = '')
{
$textboxId = self::GetId($textboxId, 'msgContentTb');
if (!isset($text))
$text = 'Ordered list';
echo '<input type="button" value="' . htmlspecialchars($text) . '" class="icon13 ol" style="' . $style . '" onclick="PlainTypeOList(\'' . $textboxId . '\')" />';
}
public static function ButtonCode($textboxId = null, $text = null, $style = '')
{
$textboxId = self::GetId($textboxId, 'msgContentTb');
if (!isset($text))
$text = 'Code';
echo '<input type="button" value="' . htmlspecialchars($text) . '" class="icon13 code" style="' . $style . '" onclick="PlainTypeCode(\'' . $textboxId . '\')" />';
}
public static function ButtonStrike($textboxId = null, $text = null, $style = '')
{
$textboxId = self::GetId($textboxId, 'msgContentTb');
if (!isset($text))
$text = 'Strike';
echo '<input type="button" value="' . htmlspecialchars($text) . '" class="icon13 strike" style="font-decoration: line-through;' . $style . '" onclick="PlainTypeStrike(\'' . $textboxId . '\')" />';
}
public static function ButtonLink($textboxId = null, $text = null, $style = '')
{
$textboxId = self::GetId($textboxId, 'msgContentTb');
if (!isset($text))
$text = 'Web link';
echo '<input type="button" value="' . htmlspecialchars($text) . '" class="icon13 link" style="' . $style . '" onclick="PlainTypeLink(\'' . $textboxId . '\')" />';
}
public static function Buttons($textboxId = null)
{
echo '<em>Text formatting and structuring:</em>';
echo '<div style="line-height: 200%; margin: 0.5em 0 1em;">';
self::ButtonEm($textboxId);
echo ' ';
self::ButtonStrong($textboxId);
echo '<br />';
self::ButtonQuote($textboxId);
echo ' ';
self::ButtonIndent($textboxId);
echo '<br />';
self::ButtonUl($textboxId);
echo ' ';
self::ButtonOl($textboxId);
echo '<br />';
self::ButtonCode($textboxId);
echo ' ';
self::ButtonStrike($textboxId);
echo '<br />';
self::ButtonLink($textboxId);
echo '</div>';
echo '<a href="' . self::$data['plaintype_help_url'] . '" onclick="return openPopupLink(this, \'plaintypehelp\', ' . self::$data['plaintype_help_width'] . ', ' . self::$data['plaintype_help_height'] . ')">Learn more about the PlainType markup language...</a>';
}
public static function TagName($tag = null, $id = null, $style = '')
{
$defaultValue = '';
if (isset($tag))
{
if (!($tag instanceof UnbTag))
throw new InvalidArgumentException('Invalid argument type: $tag');
$defaultValue = $tag->GetTagName();
}
$id = self::GetId($id, 'tagNameTb');
echo '<input type="text" id="' . $id . '" name="tag_name" size="30" style="' . htmlspecialchars($style) . '" value="' . self::GetValue('tag_name', $defaultValue) . '" />
<input type="hidden" name="action[]" value="set_tag_name" />';
}
public static function TagDescription($tag = null, $id = null, $style = '')
{
$defaultValue = '';
if (isset($tag))
{
if (!($tag instanceof UnbTag))
throw new InvalidArgumentException('Invalid argument type: $tag');
$defaultValue = $tag->GetDescription();
}
$id = self::GetId($id, 'tagDescriptionTb');
echo '<input type="text" id="' . $id . '" name="tag_description" size="50" style="' . htmlspecialchars($style) . '" value="' . self::GetValue('tag_description', $defaultValue) . '" />
<input type="hidden" name="action[]" value="set_tag_description" />';
}
public static function TagFlags($tag = null)
{
$notSelectable = false;
$invisible = false;
$enforceApproval = false;
$userData = false;
if (isset($tag))
{
if (!($tag instanceof UnbTag))
throw new InvalidArgumentException('Invalid argument type: $tag');
$notSelectable = $tag->GetFlags() & UnbTagFlag::NotSelectable;
$invisible = $tag->GetFlags() & UnbTagFlag::Invisible;
$enforceApproval = $tag->GetFlags() & UnbTagFlag::EnforceApproval;
$userData = $tag->GetFlags() & UnbTagFlag::UserData;
}
echo '<label><input type="checkbox" name="tag_flag_notselectable" value="1" ' . (($_POST ? $_POST['tag_flag_notselectable'] : $notSelectable) ? 'checked' : '') . ' /> Not selectable</label><br />
<label><input type="checkbox" name="tag_flag_invisible" value="1" ' . (($_POST ? $_POST['tag_flag_invisible'] : $invisible) ? 'checked' : '') . ' /> Invisible</label><br />
<label><input type="checkbox" name="tag_flag_enforceapproval" value="1" ' . (($_POST ? $_POST['tag_flag_enforceapproval'] : $enforceApproval) ? 'checked' : '') . ' /> Enforce approval</label><br />
<label><input type="checkbox" name="tag_flag_userdata" value="1" ' . (($_POST ? $_POST['tag_flag_userdata'] : $userData) ? 'checked' : '') . ' /> With user data</label><br />
<input type="hidden" name="action[]" value="set_tag_flags" />';
}
public static function ParentTag($tag = null, $id = null, $style = '')
{
$defaultValue = 0;
if (isset($tag))
{
if (!($tag instanceof UnbTag))
throw new InvalidArgumentException('Invalid argument type: $tag');
$defaultValue = $tag->GetParentTagId();
}
$id = self::GetId($id, 'parentTagLb');
echo '<select id="' . $id . '" name="tag_parentid" style="' . htmlspecialchars($style) . '">
<option value="0">(None)</option>';
$tags = UnbTag::Find();
$levels = array();
foreach ($tags as $tag2)
{
$level = $levels[$tag2->GetParentTagId()];
if (!isset($level))
$level = 0;
else
$level++;
$levels[$tag2->GetTagId()] = $level;
if ($tag2->GetFlags() & UnbTagFlag::Invisible) continue;
echo '<option value="' . $tag2->GetTagId() . '"';
if ($_POST && $tag2->GetTagId() == $_POST['tag_parentid'] ||
!$_POST && $tag2->GetTagId() == $defaultValue)
echo ' selected="selected"';
echo '>' .
str_repeat('&nbsp; &nbsp; ', $level) . htmlspecialchars($tag2->GetTagName()) . /*' (' . $tag2->GetTagId() . ')' .*/ '</option>';
}
echo '</select>
<input type="hidden" name="action[]" value="set_tag_parent" />';
}
public static function GetTagReadAccessKeys($tag)
{
if (!($tag instanceof UnbTag))
throw new InvalidArgumentException('Invalid argument type: $tag');
return self::GetUserList($tag->GetReadAccessKeys());
}
public static function TagReadAccessList($tag = null, $id = null, $style = '', $defaultMessage = '')
{
$readAccessList = '';
if (isset($tag))
{
if (!($tag instanceof UnbTag))
throw new InvalidArgumentException('Invalid argument type: $tag');
$readAccessList = self::GetTagReadAccessKeys($tag);
}
$id = self::GetId($id, 'tagReadAccessListTb');
echo '<textarea id="' . $id . '" name="tag_readaccesslist" cols="30" rows="3" style="' . htmlspecialchars($style) . '">' . self::GetValue('tag_readaccesslist', $readAccessList, 'tag_readaccess_empty') . '</textarea>
<input type="hidden" name="tag_readaccess_empty" value="0" />
<input type="hidden" name="action[]" value="set_tag_readaccess" />';
if (strlen($defaultMessage))
self::InsertDefaultMessage($id, $defaultMessage, 'tag_readaccess_empty');
}
public static function GetTagUseAccessKeys($tag)
{
if (!($tag instanceof UnbTag))
throw new InvalidArgumentException('Invalid argument type: $tag');
return self::GetUserList($tag->GetUseAccessKeys());
}
public static function TagUseAccessList($tag = null, $id = null, $style = '', $defaultMessage = '')
{
$useAccessList = '';
if (isset($tag))
{
if (!($tag instanceof UnbTag))
throw new InvalidArgumentException('Invalid argument type: $tag');
$useAccessList = self::GetTagUseAccessKeys($tag);
}
$id = self::GetId($id, 'tagUseAccessListTb');
echo '<textarea id="' . $id . '" name="tag_useaccesslist" cols="30" rows="3" style="' . htmlspecialchars($style) . '">' . self::GetValue('tag_useaccesslist', $useAccessList, 'tag_useaccess_empty') . '</textarea>
<input type="hidden" name="tag_useaccess_empty" value="0" />
<input type="hidden" name="action[]" value="set_tag_useaccess" />';
if (strlen($defaultMessage))
self::InsertDefaultMessage($id, $defaultMessage, 'tag_useaccess_empty');
}
}
?>